//■■GCreatorAnalyzeEngine.java
//Ver.3.4

//■■インポート
import java.awt.*;   //必須
import java.util.*;  //必須
import java.io.*;    //必須
import java.util.List;  //必須（曖昧回避）

//■■描画機能の実現
class GCreatorAnalyzeEngine
{
  //■メンバー変数
  GCreator f; //JFlameの継承インスタンス
  Graphics2D g2;
  GCreatorCanvasEngine can;
  double[] shape; //色塗り系命令文を解析して全部格納
  String command1; //命令文１行分

  //■コンストラクタ
  GCreatorAnalyzeEngine(GCreator flame)
  {
    f = flame;
    can = f.canvas;
  }

  void commandAnalyze(String commandZ)
  {
    //●命令文の全文解析
    //命令の引数が多いものや紛らわしいものだけ、数のチェックを行なう。

    //●コメント文がどうかの判定用
    boolean commentFlag = false;

    //●Import命令の処理
    //Import命令が命令文書の先頭にあれば、指定された既存命令文書を、この命令文書の先頭に挿入する。
    //Import命令が対象とする命令文書は、レイアウト・スケール命令文書などを想定している。
    //レイアウト・スケール命令文書（方眼用紙作成用）は、メニューにその実例がある。
    //Import命令は、命令文書の先頭にだけ許され、一行だけが許される。
    //導入される既存命令文書にImport命令が含まれていても、そのImport命令は無効である。
    //混乱防止のため、既存命令文書にSaveCommands命令やSaveImage命令が含まれないこと。
    //新旧の命令文書が連結されるので、同じ命令による干渉（命令上書き）に注意すること。
    StringTokenizer stI1 = new StringTokenizer(commandZ,";\n"); 
    if(stI1.hasMoreTokens())
    {
      String commandIa = stI1.nextToken().trim();
      StringTokenizer stI2 = new StringTokenizer(commandIa,"(,)");
      if(commandIa.startsWith("Import"))
      {
        String commandIb = stI2.nextToken().trim();
        commandIb = stI2.nextToken().trim();
        String fName;
        fName = Util.trimAffix(commandIb,'\"','\"');//file path
        String strI = TextIO.loadStr(new File(fName));
        if(strI != null)
          commandZ = strI + commandZ;
      }
    }

    //●以下は、一般的な命令文解析
    StringTokenizer st1 = new StringTokenizer(commandZ,";\n"); 
      //命令全文に対して、セミコロンと改行コードで切り分ける準備
    String commandS = ""; //命令1件の命令本体（引数含まず）
    f.scaleLabel2.setText("　(f:x1.0)"); //フリー・スケールの初期化
    f.canvas.freeScale = 1.0;//フリー・スケールの初期化

    try
    {
      for(int i=0; st1.hasMoreTokens(); i++)
      { //●命令文の全文読み込み
        command1 = st1.nextToken().trim(); //一行分の命令文

        if(command1.equals("")) //空行
          continue;

        if(command1.substring(0,1).equals("　") ) //全角スペース
          continue;
        
        if(command1.substring(0,2).equals("//")) //◆一行コメント文
          continue;

        if(command1.substring(0,2).equals("/@")) //◆多行コメント文
        {  //「/@」は、単独で書かねばならず、同じ行の直後に命令文を書いてはならない。
          commentFlag = true;
          continue;
        }
        if(command1.substring(0,2).equals("@/"))
        { //「@/」は、単独で書かねばならず、同じ行の直後に命令文を書いてはならない。
          commentFlag = false;
          continue;
        }
        if(commentFlag == true)
        { //「/@」と「@/」とで挟まれた中間行
          continue;
        }
        
        if(command1.substring(0,5).equals("Break")) //◆これ以降の命令文全部を無視。
          break;

        StringTokenizer st2 = new StringTokenizer(command1,"(,)");
          //引数区切を「括弧」および「コンマ」に設定。
        commandS = st2.nextToken().trim(); //命令1件の命令本体（引数含まず）
          //これ以降のst2は、命令本体が取り除かれた状態（引数群）を維持する。
          //そのため、st2.countTokens()の返却値に注意。
          //st2.countTokens()の返却値は、
          //st2.nextToken()された後に残されているトークンの数。

        //●命令文の逐次解析実施
        if(commandS.equals("Import"))
        { //●既存命令文書の先頭部への導入
          ; //既に処理済み
        }
        else if(commandS.equals("SaveCommands"))
        { //●命令文書の保存ファイル名の指定
          //ボタン操作による文書半自動保存に対応
          can.saveCmdFName = 
            Util.trimAffix(st2.nextToken().trim(),'\"','\"');
        }
        else if(commandS.equals("SaveImage"))
        { //●作成画像の保存ファイル名の指定
          //ボタン操作による画像半自動保存に対応
          can.saveImgFName = 
            Util.trimAffix(st2.nextToken().trim(),'\"','\"');
        }
        else if(commandS.equals("CutAndSaveImage"))
        { //●作成画像の切り出し保存する領域およびファイル名の指定
          //ボタン操作による画像の切り出し半自動保存に対応
          can.cutAndSaveImgX1 = Integer.parseInt(st2.nextToken().trim());
          can.cutAndSaveImgY1 = Integer.parseInt(st2.nextToken().trim());
          can.cutAndSaveImgX2 = Integer.parseInt(st2.nextToken().trim());
          can.cutAndSaveImgY2 = Integer.parseInt(st2.nextToken().trim());
          can.cutAndSaveImgFName = 
            Util.trimAffix(st2.nextToken().trim(),'\"','\"');
        }
        else if(commandS.equals("Antialias"))
        { //●線（輪郭線を含む）のギザギザ緩和
          //この処理で、細い線の色が薄くなる
          String s1 = st2.nextToken().trim();
          if(s1.equals("true"))
            can.antialiasFlag = true;
          else
            can.antialiasFlag = false;
        }
        else if(commandS.equals("Size"))
        { //●画像の大きさ
        //Size(double w, double h)
        //w:    巾(px)
        //h:    丈(px)
          String s1 = st2.nextToken().trim();
          if(Util.countDelimiter(command1, ',') == 0)
          { //文字列１個を指定の場合
            if(s1.equals("\"A4\""))
            {  //常用判型は簡易指定可能
              can.printMM_W = 210; //mm
              can.printMM_H = 297;
              can.imagePX_W = 210*3; //mm→px変換（便宜的独自規格）
              can.imagePX_H = 297*3; //印刷時には、サイズ調整を行なう
            }
            else if(s1.equals("\"B5\""))
            {
              can.printMM_W = 182; //mm
              can.printMM_H = 257;
              can.imagePX_W = 182*3; //mm→px変換（便宜的独自規格）
              can.imagePX_H = 257*3; //印刷時には、サイズ調整を行なう
            }
          }
          else if(Util.countDelimiter(command1, ',') == 1)
          { //数値２個を指定の場合（数値はピクセル単位）
            String s2 = st2.nextToken().trim();
            can.imagePX_W = Double.parseDouble(s1);
            can.imagePX_H = Double.parseDouble(s2);
            can.printMM_W = Double.parseDouble(s1)/3.0;
            can.printMM_H = Double.parseDouble(s2)/3.0;
          }
          else if(Util.countDelimiter(command1, ',') == 2)
          {  //数値２個と単位とを指定の場合
            String s2 = st2.nextToken().trim();
            String s3 = st2.nextToken().trim();
            if(s3.equals("\"px\"")) //ピクセル単位
            {
              can.printMM_W = Double.parseDouble(s1)/3.0;
              can.printMM_H = Double.parseDouble(s2)/3.0;
              can.imagePX_W = Double.parseDouble(s1);
              can.imagePX_H = Double.parseDouble(s2);
            }
            else if(s3.equals("\"mm\"")) //ミリ単位
            { //ミリ単位は、印刷での紙サイズをミリ単位で表わすだけである。
              //その他の描画時の各種サイズ指定は、文字サイズを除き、全てピクセル。
              can.printMM_W = Double.parseDouble(s1);
              can.printMM_H = Double.parseDouble(s2);
              can.imagePX_W = Double.parseDouble(s1)*3.0;
              can.imagePX_H = Double.parseDouble(s2)*3.0;
            }
          }
          else
          {
            f.message(command1 + " >>引数に異常");
            return;
          }
        }
        else if(commandS.equals("BackgroundColor"))
        { //●背景色の指定
        //BackgroundColor(int r, int g, int b)
        //r:     赤色成分　0-255
        //g:     緑色成分　0-255
        //b:     青色成分　0-255
          int red   = Integer.parseInt(st2.nextToken().trim());
          int green = Integer.parseInt(st2.nextToken().trim());
          int blue  = Integer.parseInt(st2.nextToken().trim());
          if(red>255)
          {
            red = 255;
            f.message("色指定の上限は255です。");
          }
          if(green>255)
          {
            green = 255;
            f.message("色指定の上限は255です。");
          }
          if(blue>255)
          {
            blue = 255;
            f.message("色指定の上限は255です。");
          }
          can.backgroundColor = new Color(red, green, blue);
        }
        else if(commandS.equals("Image")) 
        { //●背景画像の表示指定
        //Image(int x, int y, int w, int h, String fileName)
        //x:     表示ｘ座標位置
        //y:     表示ｙ座標位置
        //w:     画像の表示幅
        //h:     画像の表示丈
        //fileName:画像ファイル・ネーム（パス含む）
          can.imageFlag = true;
          int[]  imgI = new int[4]; //複数画像を別位置に表示可能
          imgI[0] = Integer.parseInt(st2.nextToken().trim()); //x
          imgI[1] = Integer.parseInt(st2.nextToken().trim()); //y
          imgI[2] = Integer.parseInt(st2.nextToken().trim()); //w
          imgI[3] = Integer.parseInt(st2.nextToken().trim()); //h
          can.imageListI.add(imgI);
          String imgF;
          imgF = Util.trimAffix(st2.nextToken().trim(),'\"','\"');//file path
          can.imageListF.add(imgF);
        }
        else if(commandS.equals("Stamp")) 
        { //●スタンプ画像の表示指定
        //Stamp(int x, int y, int w, int h, String f)
        //x:     表示ｘ座標位置
        //y:     表示ｙ座標位置
        //w:     画像の表示幅
        //h:     画像の表示丈
        //f:     画像ファイル（パス含む）
          int[]  stmI = new int[4]; //複数画像を別位置に表示可能
          stmI[0] = Integer.parseInt(st2.nextToken().trim()); //x
          stmI[1] = Integer.parseInt(st2.nextToken().trim()); //y
          stmI[2] = Integer.parseInt(st2.nextToken().trim()); //w
          stmI[3] = Integer.parseInt(st2.nextToken().trim()); //h
          can.stampListI.add(stmI);
          String stmF;
          stmF = Util.trimAffix(st2.nextToken().trim(),'\"','\"');//file path
          can.stampListF.add(stmF);
          can.stampFlag = true;
        }
        else if(commandS.equals("StampOp")) 
        { //●スタンプ機能におけるグレー・マスクの灰色を指定色に変更するオプション
        //StampOp(int r, int g, int b)
        //r:     グレー・マスク代替色の赤成分
        //g:     グレー・マスク代替色の緑成分
        //b:     グレー・マスク代替色の青成分
          int r = Integer.parseInt(st2.nextToken().trim());
          int g = Integer.parseInt(st2.nextToken().trim());
          int b = Integer.parseInt(st2.nextToken().trim());
          can.transparentColor = new Color(r, g, b); 
        }
        else if(commandS.equals("StampMulti")) 
        { //●スタンプ画像の表示指定の連続処理
        //StampMulti(int x, int y, int w, int h, int sx, int sy, int tx, int ty, String f)
        //x:     当初表示ｘ座標位置
        //y:     当初表示ｙ座標位置
        //w:     画像の表示幅
        //h:     画像の表示丈
        //sx:    画像の横方向スキップのピクセル数
        //sy:    画像の縦方向スキップのピクセル数
        //tx:    画像の横方向表示反復回数
        //ty:    画像の縦方向表示反復回数
        //f:     画像ファイル（パス含む）
          int x    = Integer.parseInt(st2.nextToken().trim());
          int y    = Integer.parseInt(st2.nextToken().trim());
          int w    = Integer.parseInt(st2.nextToken().trim());
          int h    = Integer.parseInt(st2.nextToken().trim());
          int sx   = Integer.parseInt(st2.nextToken().trim());
          int sy   = Integer.parseInt(st2.nextToken().trim());
          int tx   = Integer.parseInt(st2.nextToken().trim());
          int ty   = Integer.parseInt(st2.nextToken().trim());
          String f = Util.trimAffix(st2.nextToken().trim(),'\"','\"');

          //Stamp命令の反復に変換する
          for(int p=0; p<ty; p++)
          {
            for(int q=0; q<tx; q++)
            {
              int[]  stmI = new int[4];
              stmI[0] = x + sx*q; //x位置
              stmI[1] = y + sy*p; //y位置
              stmI[2] = w; //w
              stmI[3] = h; //h
              can.stampListI.add(stmI);
              can.stampListF.add(f);
            }
          }
          can.stampFlag = true;
        }

        else if(commandS.equals("SprayStart")) 
        { //●スプレー１回（円盤状飛沫集団）の反復作業での共通条件設定。
        //スプレー機能（円盤状飛沫集団形成）は、三角関数と乱数とで実現する。
        //円盤中心点から飛沫一個への距離cは、0から円盤半径までの整数乱数で決定し、
        //円盤中心点から飛沫一個への角度tは、0から360までの整数乱数で決定する。
        //直角三角形⊿で、底辺をa、高さをb、斜辺をc、左下角度（ディグリー）をtとすれば、
        //ただし、左下角を円盤中心、右上角を飛沫位置と見なす。
        //関係する計算式は、以下の通り。

        //SprayStart(int x, int y, int tp, double ss, int sr, int st, int r, int g, int b, int a)
        //x:     スプレイ円盤状飛沫集団（円盤）の中心ｘ座標
        //y:     スプレイ円盤状飛沫集団（円盤）の中心ｙ座標
        //tp:    スプレイ飛沫の分散タイプ
        //ss:    スプレイ飛沫一個のサイズ（px相当）
        //sr:    スプレイ飛沫の円盤状拡散分布半径（px相当）
        //st:    スプレイ飛沫の分布個数
        //r:     スプレイ飛沫の色相、赤成分
        //g:     スプレイ飛沫の色相、緑成分
        //b:     スプレイ飛沫の色相、青成分
        //a:     スプレイ飛沫の色相、透明度
          can.sprayFlag = true;
          can.sprayDiscStatus_List.add(0);
          can.sprayDiscX_List.add(Integer.parseInt(st2.nextToken().trim()));
          can.sprayDiscY_List.add(Integer.parseInt(st2.nextToken().trim()));
          can.sprayUnitTP_List.add(Integer.parseInt(st2.nextToken().trim()));
          can.sprayUnitSS_List.add(Double.parseDouble(st2.nextToken().trim()));
          can.sprayUnitSR_List.add(Integer.parseInt(st2.nextToken().trim()));
          can.sprayUnitST_List.add(Integer.parseInt(st2.nextToken().trim()));
          can.sprayUnitR_List.add(Integer.parseInt(st2.nextToken().trim()));
          can.sprayUnitG_List.add(Integer.parseInt(st2.nextToken().trim()));
          can.sprayUnitB_List.add(Integer.parseInt(st2.nextToken().trim()));
          can.sprayUnitA_List.add(Integer.parseInt(st2.nextToken().trim()));
        }
        else if(commandS.equals("SprayNext")) 
        { //●スプレーの反復作業での個別座標設定
        //SprayNext(int x, int y)
        //x:     スプレイ円盤状飛沫集団の中心ｘ座標
        //y:     スプレイ円盤状飛沫集団の中心ｙ座標
          can.sprayDiscStatus_List.add(1);
          can.sprayDiscX_List.add(Integer.parseInt(st2.nextToken().trim()));
          can.sprayDiscY_List.add(Integer.parseInt(st2.nextToken().trim()));
        }
        else if(commandS.equals("SprayEnd")) 
        { //●//スプレーのユニット終了
        //SprayEnd(int x, int y)
          can.sprayDiscStatus_List.add(2);
          can.sprayDiscX_List.add(Integer.parseInt(st2.nextToken().trim()));
          can.sprayDiscY_List.add(Integer.parseInt(st2.nextToken().trim()));
        }
        else if(commandS.equals("SprayGaussianOp")) 
        { //●//スプレーでの正規分布オプション
        //SprayGaussianOp(double g)
        //g:      正規分布スプレーでの分布操作係数
          can.sprayGaussianOp = Double.parseDouble(st2.nextToken().trim());
        }

        else if(commandS.equals("LinkedLineStart")) 
        { //●折れ線ユニット開始（「折れ線ユニット」とは、連続した折れ線のこと）
        //LinkedLineStart(double x, double y)
        //x:      始点のｘ座標
        //x:      始点のｙ座標
          can.linkedLineFlag = true;
          can.linkedLineStatusList.add(0);
          can.linkedLineXList.add(Double.parseDouble(st2.nextToken().trim()));
          can.linkedLineYList.add(Double.parseDouble(st2.nextToken().trim()));
        }
        else if(commandS.equals("LinkedLineNext")) 
        { //●折れ線ユニット継続
        //LinkedLineNext(double x, double y)
        //x:      継続点のｘ座標
        //x:      継続点のｙ座標
          can.linkedLineStatusList.add(1);
          can.linkedLineXList.add(Double.parseDouble(st2.nextToken().trim()));
          can.linkedLineYList.add(Double.parseDouble(st2.nextToken().trim()));
        }
        else if(commandS.equals("LinkedLineEnd")) 
        { //●折れ線ユニット終了
        //LinkedLineEnd(double x, double y)
        //x:      終点のｘ座標
        //x:      終点のｙ座標
          can.linkedLineStatusList.add(2);
          can.linkedLineXList.add(Double.parseDouble(st2.nextToken().trim()));
          can.linkedLineYList.add(Double.parseDouble(st2.nextToken().trim()));
        }

        else if(commandS.equals("CutAndPasteMask")) 
        { //●切り貼り画像合成でのマルチ・マスク指定
        //CutAndPasteMask(String f)
        //f:      マルチ・マスク画像ファイル（パス含む）
          can.cutAndPasteMaskFlag = true;
          can.capMaskF = Util.trimAffix(st2.nextToken().trim(),'\"','\"');//file path;
        }
        else if(commandS.equals("CutAndPaste")) 
        { //●切り貼り画像合成の実行
        //CutAndPaste(String fileName, int r, int g, int b)
        //fileName:模様画像ファイル（パス含む）
        //r:      マスク画像切り貼り部分の色相赤成分
        //g:      
        //b:      
          can.cutAndPasteFlag = true;
          can.capListF.add(Util.trimAffix(st2.nextToken().trim(),'\"','\"'));
          int[]  capI = new int[3]; //マルチ・マスクの色分け指定色
          capI[0] = Integer.parseInt(st2.nextToken().trim()); //r
          capI[1] = Integer.parseInt(st2.nextToken().trim()); //g
          capI[2] = Integer.parseInt(st2.nextToken().trim()); //b
          can.capListI.add(capI);
        }
        else if(commandS.equals("ChangeColorMultiBase")) 
        { //●連続色相変更でのベース画像指定
        //ChangeColorMultiBase(String fileName)
        //fileName:連続色相変更ベース画像ファイル（パス含む）
          can.changeColorMultiBaseFlag = true;
          can.ccmMaskF = Util.trimAffix(st2.nextToken().trim(),'\"','\"');//file path;
        }
        else if(commandS.equals("ChangeColorMulti")) 
        { //●連続色相変更の実行
        //ChangeColorMulti(int r0, int g0, int b0, int r1, int g1, int b1)
        //r0:      旧色の色相赤成分
        //g0:      
        //b0:      
        //r1:      新色の色相赤成分
        //g1:      
        //b1:      
          can.changeColorMultiFlag = true;
          int[]  ccmI0 = new int[3]; //旧画像　指定色　数値データ
          ccmI0[0] = Integer.parseInt(st2.nextToken().trim()); //r
          ccmI0[1] = Integer.parseInt(st2.nextToken().trim()); //g
          ccmI0[2] = Integer.parseInt(st2.nextToken().trim()); //b
          can.ccmListI0.add(ccmI0);
          int[]  ccmI1 = new int[3]; //新画像　指定色　数値データ
          ccmI1[0] = Integer.parseInt(st2.nextToken().trim()); //r
          ccmI1[1] = Integer.parseInt(st2.nextToken().trim()); //g
          ccmI1[2] = Integer.parseInt(st2.nextToken().trim()); //b
          can.ccmListI1.add(ccmI1);
        }
        else if(commandS.equals("ChangeColorInner")) 
        { //●輪郭線で囲まれた図形の内部を指定色で塗り直す。
          //塗り直し対象領域は、輪郭線で囲まれた図形の内部の代表位置を、
          //xy座標位置で指定しなければならない。
          //その座標位置（１ピクセル分）を初期設定として指定色で彩色した後、
          //プログラムが全体をスキャンしながら指定色のありかを探し出し、その領域拡大を行なう。
        //ChangeColorInner(String f. int r0, int g0, int b0, int r1, int g1, int b1, int x, int y, int mode)
        //f:       塗り直し対象輪郭線画像ファイル名（パス含む）
        //r0:      輪郭線色
        //g0:      
        //b0:      
        //r1:      彩色指定色
        //g1:      
        //b1:      
        //x:       図形内側代表位置のx座標
        //y:       図形内側代表位置のy座標
        //mode     輪郭線も塗り直すかの可否（０は否、１は可）
          can.changeColorInnerFlag = true;
          can.changeColorInnerF = Util.trimAffix(st2.nextToken().trim(),'\"','\"');
          can.cciOutLineColorR  = Integer.parseInt(st2.nextToken().trim());
          can.cciOutLineColorG  = Integer.parseInt(st2.nextToken().trim());
          can.cciOutLineColorB  = Integer.parseInt(st2.nextToken().trim());
          can.cciInnerColorR    = Integer.parseInt(st2.nextToken().trim());
          can.cciInnerColorG    = Integer.parseInt(st2.nextToken().trim());
          can.cciInnerColorB    = Integer.parseInt(st2.nextToken().trim());
          can.cciInnerX         = Integer.parseInt(st2.nextToken().trim());
          can.cciInnerY         = Integer.parseInt(st2.nextToken().trim());
          can.cciMode           = Integer.parseInt(st2.nextToken().trim());
        }
        else if(commandS.equals("ChangeColorInnerMultiMask")) 
        { //●輪郭線で囲まれた図形の特定領域を指定色で連続的に塗り直すための統一指定。
        //輪郭線画像は、白地図形式で塗り分けられる状態にあるものとする。
        //輪郭線（共用輪郭線）は、塗り直し対象外であって、そのままの色で残る。
        //輪郭線を隠すには、最終段階でソフト・フォーカスおよびスタンプで対処。
        //複雑に屈曲した輪郭線図形で、その内部彩色処理を高速化するには、点描画機能などを用い、
        //彩色部の数箇所に指定色を予め塗っておくこと。
        //ChangeColorInnerMultiMask(String f. int r, int g, int b)
        //f:       塗り直し対象輪郭線画像ファイル名（パス含む）
        //r:       輪郭線色
        //g:      
        //b:      
          can.changeColorInnerMultiFlag = true;
          can.changeColorInnerMultiF = Util.trimAffix(st2.nextToken().trim(),'\"','\"');
          can.ccimOutLineColorR  = Integer.parseInt(st2.nextToken().trim());
          can.ccimOutLineColorG  = Integer.parseInt(st2.nextToken().trim());
          can.ccimOutLineColorB  = Integer.parseInt(st2.nextToken().trim());
        }
        else if(commandS.equals("ChangeColorInnerMulti")) 
        { //●輪郭線で囲まれた図形の内部を指定色で連続的に塗り直すための個別指定。
        //ChangeColorInnerMulti(int r, int g, int b, int x, int y,)
        //r:       彩色指定色
        //g:       
        //b:       
        //x:       図形内側彩色領域代表位置のx座標
        //y:       図形内側彩色領域代表位置のy座標
          int[]  intArr = new int[5]; //
          intArr[0] = Integer.parseInt(st2.nextToken().trim()); //r
          intArr[1] = Integer.parseInt(st2.nextToken().trim()); //g
          intArr[2] = Integer.parseInt(st2.nextToken().trim()); //b
          intArr[3] = Integer.parseInt(st2.nextToken().trim()); //x
          intArr[4] = Integer.parseInt(st2.nextToken().trim()); //y
          can.ccimListI.add(intArr);
        }
        else if(commandS.equals("MixImage")) 
        { //●ミックス画像（二重像画像）の合成指定
          if(st2.countTokens() == 2 && Util.countDelimiter(command1, ',') == 1)
          {

            //MixImage(String fileName1, String fileName2)
            //fileName1:元画像ファイル（パス含む）
            //fileName2:追加画像ファイル名（パス含む）
            can.mixFName0 = Util.trimAffix(st2.nextToken().trim(),'\"','\"');//file path
            can.mixFName1 = Util.trimAffix(st2.nextToken().trim(),'\"','\"');//file path
          }
          else if(st2.countTokens() == 3 && Util.countDelimiter(command1, ',') == 2)
          {
            //MixImage(int mixMode, String fileName1, String fileName2)
            //mixMode:画像の拡大縮小を伴わないStamp押印として機能するモードの指定（０を指定）
            //fileName1:元画像ファイル名（パス含む）
            //fileName2:追加画像ファイル名（パス含む）
            can.mixMode = Integer.parseInt(st2.nextToken().trim()); 
            can.mixFName0 = Util.trimAffix(st2.nextToken().trim(),'\"','\"');//file path
            can.mixFName1 = Util.trimAffix(st2.nextToken().trim(),'\"','\"');//file path
            if(can.mixMode!=0 && can.mixMode!=1)
            {
              can.mixMode = 1;
              f.message(command1 + " >>モード指定が異常のため、通常モードにしました。");
            }
          }
          else
          {
            f.message(command1 + " >>引数が異常。");
            return;
          }
        }
        else if(commandS.equals("SoftFocus")) 
        { //●ソフト・フォーカス効果要求
        //SoftFocus(String f, int d)
        //f:     元画像ファイル（パス含む）
        //d:     ソフト・フォーカスの度合い（０～１００）
          can.softFocusFName = Util.trimAffix(st2.nextToken().trim(),'\"','\"');
          can.softFocusDegree = Integer.parseInt(st2.nextToken().trim());
          if(can.softFocusDegree > 100)
            can.softFocusDegree = 100;
          if(can.softFocusDegree < 0)
            can.softFocusDegree = 0;
        }
        else if(commandS.equals("AdjustRGB")) 
        { //●色成分RGB加減調整（特定の色相の微調整）
        //AdjustRGB(String f, int r, int g, int b)
        //f:     元画像ファイル名（パス含む）
        //r:     赤色成分の加減量（－１００～１００％）
        //g:     
        //b:     
          can.adjustFName = Util.trimAffix(st2.nextToken().trim(),'\"','\"');
          can.adjustR = Integer.parseInt(st2.nextToken().trim());
          can.adjustG = Integer.parseInt(st2.nextToken().trim());
          can.adjustB = Integer.parseInt(st2.nextToken().trim());
          if(can.adjustR>100) can.adjustR = 100;
          if(can.adjustG>100) can.adjustG = 100;
          if(can.adjustB>100) can.adjustB = 100;
          if(can.adjustR<-100) can.adjustR = -100;
          if(can.adjustG<-100) can.adjustG = -100;
          if(can.adjustB<-100) can.adjustB = -100;
        }
        else if(commandS.equals("ColorGradient")) 
        { //●色彩階調効果の追加
        //ColorGradient(String fName, int m, int y1, int y2,
        //  double r, double g, double b)
        //fName:  元画像ファイル名（パス含む）
        //y1:     加工開始y軸位置
        //y2:     加工反転y軸位置
        //r:      赤成分の追加数値
        //g:      緑成分の追加数値
        //b:      青成分の追加数値
          if(st2.countTokens() != 6 || Util.countDelimiter(command1, ',') != 5)
          { //引数の個数が多いので、個数をチェック（以下同様）
            f.message(command1 + " >>引数の個数に異常");
            return;
          }
          can.gradientFName = Util.trimAffix(st2.nextToken().trim(),'\"','\"');
          can.gradientY1 = Integer.parseInt(st2.nextToken().trim());
          can.gradientY2 = Integer.parseInt(st2.nextToken().trim());
          can.gradientR = Double.parseDouble(st2.nextToken().trim());
          can.gradientG = Double.parseDouble(st2.nextToken().trim());
          can.gradientB = Double.parseDouble(st2.nextToken().trim());
        }
        else if(commandS.equals("ChangeColor")) 
        { //●単独色相変更（近似色をも含めた単独色相変更）
        //ChangeColor(String fName,int c, int mode,
        //  int r1, int g1, int b1, int r2, int g2, int b2 )
        //ChangeColor(String fName,int c,
        //  int r1, int g1, int b1, int r2, int g2, int b2 )
        //fName:  元画像ファイル名（パス含む）
        //c:      指定色許容値
        //mode:   1が単純な色相変更で、2がグレー・マスクを生成
        //r1:     旧色
        //g1:
        //b1:
        //r2:     新色
        //g2:
        //b2:
          if(st2.countTokens() == 9 && Util.countDelimiter(command1, ',') == 8)
          {
            can.colorChangeFlag = true;
            can.colorChangeFile  = Util.trimAffix(st2.nextToken().trim(),'\"','\"');
            can.acceptableLimitC = Integer.parseInt(st2.nextToken().trim());
            can.colorChangeMode  = Integer.parseInt(st2.nextToken().trim());
            can.colorChange1R    = Integer.parseInt(st2.nextToken().trim());
            can.colorChange1G    = Integer.parseInt(st2.nextToken().trim());
            can.colorChange1B    = Integer.parseInt(st2.nextToken().trim());
            can.colorChange2R    = Integer.parseInt(st2.nextToken().trim());
            can.colorChange2G    = Integer.parseInt(st2.nextToken().trim());
            can.colorChange2B    = Integer.parseInt(st2.nextToken().trim());
          }
          else if(st2.countTokens() == 8 && Util.countDelimiter(command1, ',') == 7)
          {
            can.colorChangeFlag = true;
            can.colorChangeFile  = Util.trimAffix(st2.nextToken().trim(),'\"','\"');
            can.acceptableLimitC = Integer.parseInt(st2.nextToken().trim());
            can.colorChangeMode  = 1;
            can.colorChange1R    = Integer.parseInt(st2.nextToken().trim());
            can.colorChange1G    = Integer.parseInt(st2.nextToken().trim());
            can.colorChange1B    = Integer.parseInt(st2.nextToken().trim());
            can.colorChange2R    = Integer.parseInt(st2.nextToken().trim());
            can.colorChange2G    = Integer.parseInt(st2.nextToken().trim());
            can.colorChange2B    = Integer.parseInt(st2.nextToken().trim());
          }
          else
          {
            f.message(command1 + " >>引数の個数に異常");
            return;
          }
        }

        else if(commandS.equals("ReverseImage")) 
        { //●画像反転（鏡像反転または直角系回転）
        //ReverseImage(String fName,int m)
        //fName:  元画像ファイル名（パス含む）
        //c:      反転モード（１～７）
          can.reverseImageFlag = true;
          can.reverseImageFile = Util.trimAffix(st2.nextToken().trim(),'\"','\"');
          can.reverseImageMode = Integer.parseInt(st2.nextToken().trim());
        }

        else if(commandS.equals("ForegroundColorL"))
        { //●前景色（線の色）の指定
        //ForegroundColorL(int r, int g, int b)
        //r:     赤色成分　0-255
        //g:     緑色成分　0-255
        //b:     青色成分　0-255
          int red   = Integer.parseInt(st2.nextToken().trim());
          int green = Integer.parseInt(st2.nextToken().trim());
          int blue  = Integer.parseInt(st2.nextToken().trim());
          if(red>255)
          {
            red = 255;
            f.message("色指定の上限は255です。");
          }
          if(green>255)
          {
            green = 255;
            f.message("色指定の上限は255です。");
          }
          if(blue>255)
          {
            blue = 255;
            f.message("色指定の上限は255です。");
          }
          can.foregroundColorL = new Color(red, green, blue);
        }
        else if(commandS.equals("ForegroundColorS"))
        { //●前景色（文字の色）の指定
        //ForegroundColorS(int r, int g, int b)
        //r:     赤色成分　0-255
        //g:     緑色成分　0-255
        //b:     青色成分　0-255
          int red   = Integer.parseInt(st2.nextToken().trim());
          int green = Integer.parseInt(st2.nextToken().trim());
          int blue  = Integer.parseInt(st2.nextToken().trim());
          if(red>255)
          {
            red = 255;
            f.message("色指定の上限は255です。");
          }
          if(green>255)
          {
            green = 255;
            f.message("色指定の上限は255です。");
          }
          if(blue>255)
          {
            blue = 255;
            f.message("色指定の上限は255です。");
          }
          can.foregroundColorS = new Color(red, green, blue);
        }
        else if(commandS.equals("ForegroundColorB"))
        { //●前景色（極太文字の色）の指定
        //ForegroundColorB(int r, int g, int b)
        //r:     赤色成分　0-255
        //g:     緑色成分　0-255
        //b:     青色成分　0-255
          int red   = Integer.parseInt(st2.nextToken().trim());
          int green = Integer.parseInt(st2.nextToken().trim());
          int blue  = Integer.parseInt(st2.nextToken().trim());
          if(red>255)
          {
            red = 255;
            f.message("色指定の上限は255です。");
          }
          if(green>255)
          {
            green = 255;
            f.message("色指定の上限は255です。");
          }
          if(blue>255)
          {
            blue = 255;
            f.message("色指定の上限は255です。");
          }
          can.foregroundColorB = new Color(red, green, blue);
        }
        else if(commandS.equals("AffineScale"))
        { //●標準の拡大率の指定（印刷に反映されない）
        //AffineScale(double scale)
        //scale:  表示、画像保存に有効な拡大倍率
          can.affineScale = Double.parseDouble(st2.nextToken().trim());
          f.scaleLabel.setText("　(a:x" + can.affineScale + ")");
        }
        else if(commandS.equals("FreeScale"))
        { //●任意の拡大率の指定（印刷にも反映される）
        //FreeScale(double scale)
        //scale:  表示、画像保存、印刷に有効な拡大倍率
          can.freeScale = Double.parseDouble(st2.nextToken().trim());
          f.scaleLabel2.setText("　(f:x" + can.freeScale + ")");
        }
        else if(commandS.equals("Shift"))
        { //●全体の画像をずらす指定
        //Shift(int mode, double x, double y)
        //mode:   0がシフトなし、1がシフトあり
        //x:    X軸のシフト量
        //y:    Y軸のシフト量
          can.shiftMode = Integer.parseInt(st2.nextToken().trim());
          can.shiftX = Double.parseDouble(st2.nextToken().trim());
          can.shiftY = Double.parseDouble(st2.nextToken().trim());
        }
        else if(commandS.equals("LineWidth"))
        { //●線の太さ指定
        //LineWidth(float width)
        //width:  線の太さ
          can.lineWidth = Float.parseFloat(st2.nextToken().trim());
        }
        else if(commandS.equals("DrawLine"))
        { //●直線の描画指定
        //DrawLine(double x1, double y1, double x2, double y2)
        //x1:  起点座標X
        //y1:  起点座標Y
        //x2:  終点座標X
        //y2:  終点座標Y
          double[] d = new double[4];
          d[0] = Double.parseDouble(st2.nextToken().trim());
          d[1] = Double.parseDouble(st2.nextToken().trim());
          d[2] = Double.parseDouble(st2.nextToken().trim());
          d[3] = Double.parseDouble(st2.nextToken().trim());
          can.lineList.add(d);
        }
        else if(commandS.equals("DrawLine2"))  //マウス操作時に一時的使用
        { //●仮直線の描画指定
        //Draw2(int m; double x1, double y1, double x2, double y2)
        // m:  モード
        //x1:  起点座標X
        //y1:  起点座標Y
        //x2:  終点座標X
        //y2:  終点座標Y
          double[] d = new double[5];
          d[0] = Double.parseDouble(st2.nextToken().trim());
          d[1] = Double.parseDouble(st2.nextToken().trim());
          d[2] = Double.parseDouble(st2.nextToken().trim());
          d[3] = Double.parseDouble(st2.nextToken().trim());
          d[4] = Double.parseDouble(st2.nextToken().trim());
          can.line2List.add(d);
        }
        else if(commandS.equals("DrawParallelLineH"))
        { //●横平行線の描画指定
        //DrawParallelLineH(double x, double y, double l, double w, int t, int m)
        //x:  起点座標X
        //y:  起点座標Y
        //l:  線の長さ
        //w:  線の間隔
        //t:  線の本数
        //m:  線の種類（０：実線、１：密点線、２：粗点線）
        //h:  中間線の有無（０：無し、１：有り）
          if(st2.countTokens() != 7 || Util.countDelimiter(command1, ',') != 6)
          {
            f.message(command1 + " >>引数の個数に異常");
            return;
          }
          can.parallelLineH_x = Double.parseDouble(st2.nextToken().trim());
          can.parallelLineH_y = Double.parseDouble(st2.nextToken().trim());
          can.parallelLineH_l = Double.parseDouble(st2.nextToken().trim());
          can.parallelLineH_w = Double.parseDouble(st2.nextToken().trim());
          can.parallelLineH_t = Integer.parseInt(st2.nextToken().trim());
          can.parallelLineH_m = Integer.parseInt(st2.nextToken().trim());
          can.parallelLineH_h = Integer.parseInt(st2.nextToken().trim());
        }
        else if(commandS.equals("DrawParallelLineV"))
        { //●縦平行線の描画指定
        //DrawParallelLineV(double x, double y, double l, double w, int t, int m)
        //x:  起点座標X
        //y:  起点座標Y
        //l:  線の長さ
        //w:  線の間隔
        //t:  線の本数
        //m:  線の種類（０：実線、１：密点線、２：粗点線）
        //h:  中間線の有無（０：無し、１：有り）
          if(st2.countTokens() != 7 || Util.countDelimiter(command1, ',') != 6)
          {
            f.message(command1 + " >>引数の個数に異常");
            return;
          }
          can.parallelLineV_x = Double.parseDouble(st2.nextToken().trim());
          can.parallelLineV_y = Double.parseDouble(st2.nextToken().trim());
          can.parallelLineV_l = Double.parseDouble(st2.nextToken().trim());
          can.parallelLineV_w = Double.parseDouble(st2.nextToken().trim());
          can.parallelLineV_t = Integer.parseInt(st2.nextToken().trim());
          can.parallelLineV_m = Integer.parseInt(st2.nextToken().trim());
          can.parallelLineV_h = Integer.parseInt(st2.nextToken().trim());
        }
        else if(commandS.equals("DrawParallelLineH2"))
        { //●横平行線の描画指定（補助用）
        //DrawParallelLineH2(double x, double y, double l, double w, int t, int m)
        //x:  起点座標X
        //y:  起点座標Y
        //l:  線の長さ
        //w:  線の間隔
        //t:  線の本数
        //m:  線の種類（０：実線、１：密点線、２：粗点線）
        //h:  中間線の有無（０：無し、１：有り）
          if(st2.countTokens() != 7 || Util.countDelimiter(command1, ',') != 6)
          {
            f.message(command1 + " >>引数の個数に異常");
            return;
          }
          can.parallelLineH2_x = Double.parseDouble(st2.nextToken().trim());
          can.parallelLineH2_y = Double.parseDouble(st2.nextToken().trim());
          can.parallelLineH2_l = Double.parseDouble(st2.nextToken().trim());
          can.parallelLineH2_w = Double.parseDouble(st2.nextToken().trim());
          can.parallelLineH2_t = Integer.parseInt(st2.nextToken().trim());
          can.parallelLineH2_m = Integer.parseInt(st2.nextToken().trim());
          can.parallelLineH2_h = Integer.parseInt(st2.nextToken().trim());
        }
        else if(commandS.equals("DrawParallelLineV2"))
        { //●縦平行線の描画指定（補助用）
        //DrawParallelLineV2(double x, double y, double l, double w, int t, int m)
        //x:  起点座標X
        //y:  起点座標Y
        //l:  線の長さ
        //w:  線の間隔
        //t:  線の本数
        //m:  線の種類（０：実線、１：密点線、２：粗点線）
        //h:  中間線の有無（０：無し、１：有り）
          if(st2.countTokens() != 7 || Util.countDelimiter(command1, ',') != 6)
          {
            f.message(command1 + " >>引数の個数に異常");
            return;
          }
          can.parallelLineV2_x = Double.parseDouble(st2.nextToken().trim());
          can.parallelLineV2_y = Double.parseDouble(st2.nextToken().trim());
          can.parallelLineV2_l = Double.parseDouble(st2.nextToken().trim());
          can.parallelLineV2_w = Double.parseDouble(st2.nextToken().trim());
          can.parallelLineV2_t = Integer.parseInt(st2.nextToken().trim());
          can.parallelLineV2_m = Integer.parseInt(st2.nextToken().trim());
          can.parallelLineV2_h = Integer.parseInt(st2.nextToken().trim());
        }
        else if(commandS.equals("DrawCurve"))
        { //●ベジェ曲線の描画指定
        //DrawCurve(double x1, double y1, double x2, double y2,
        //  double x3, double y3, double x4, double y4)
        //x1:  起点座標X
        //y1:  起点座標Y
        //x2:  第一制御点座標X
        //y2:  第一制御点座標Y
        //x3:  第二制御点座標X
        //y3:  第二制御点座標Y
        //x4:  終点座標X
        //y4:  終点座標Y
          if(st2.countTokens() != 8 || Util.countDelimiter(command1, ',') != 7)
          {
            f.message(command1 + " >>引数の個数に異常");
            return;
          }
          double[] d = new double[8];
          d[0] = Double.parseDouble(st2.nextToken().trim());
          d[1] = Double.parseDouble(st2.nextToken().trim());
          d[2] = Double.parseDouble(st2.nextToken().trim());
          d[3] = Double.parseDouble(st2.nextToken().trim());
          d[4] = Double.parseDouble(st2.nextToken().trim());
          d[5] = Double.parseDouble(st2.nextToken().trim());
          d[6] = Double.parseDouble(st2.nextToken().trim());
          d[7] = Double.parseDouble(st2.nextToken().trim());
          can.curveList.add(d);
        }
        else if(commandS.equals("DrawSplineStart"))
        { //●スプライン曲線の描画指定（起点）
        //DrawSplineStart(double cmd, double x, double y)
        //cmd: 命令識別番号（定数0）
        //x1:  起点座標X
        //y1:  起点座標Y
          double[] d = new double[3];
          d[0] = 0;
          d[1] = Double.parseDouble(st2.nextToken().trim());
          d[2] = Double.parseDouble(st2.nextToken().trim());
          can.splineList.add(d);
        }
        else if(commandS.equals("DrawSplineControl"))
        { //●スプライン曲線の描画指定（制御点）
        //DrawSplineStart(double cmd, double x, double y)
        //cmd: 命令識別番号（定数1）
        //x1:  制御点座標X
        //y1:  制御点座標Y
          double[] d = new double[3];
          d[0] = 1;
          d[1] = Double.parseDouble(st2.nextToken().trim());
          d[2] = Double.parseDouble(st2.nextToken().trim());
          can.splineList.add(d);
        }
        else if(commandS.equals("DrawSplineEnd"))
        { //●スプライン曲線の描画指定（終点）
        //DrawSplineEnd(double cmd, double x, double y)
        //cmd: 命令識別番号（定数2）
        //x1:  終点座標X
        //y1:  終点座標Y
          double[] d = new double[3];
          d[0] = 2;
          d[1] = Double.parseDouble(st2.nextToken().trim());
          d[2] = Double.parseDouble(st2.nextToken().trim());
          can.splineList.add(d);
        }
        else if(commandS.equals("DrawSplineOp"))
        { //●スプライン曲線の描画指定（曲線精細度オプション）
        //DrawSplineOp(int op)
        //op: 曲線を構成する短い直線の本数
          can.splineOption = Integer.parseInt(st2.nextToken().trim());
        }

        //●●色塗り図形描画命令「解読」グループの始まり（命令識別を添加）

        //●自由図形の描画指定
        else if(commandS.equals("DrawShapeStart"))
        //DrawShapeStart(int m, double x,double y, int r, int g, int b)
        //m:  描画モード指定
        //x:  自由図形の起点座標X
        //y:  自由図形の起点座標Y
        //r:  赤色成分　0-255
        //g:  緑色成分　0-255
        //b:  青色成分　0-255
        {
          if(st2.countTokens() != 6 || Util.countDelimiter(command1, ',') != 5)
          {
            f.message(command1 + " >>引数の個数に異常");
            return;
          }
          shape = new double[9];
          shape[0] = 0; //命令識別（DrawShapeStart）
          shape[1] = Double.parseDouble(
            st2.nextToken().trim()); //m 
          shape[2] = Double.parseDouble(
            st2.nextToken().trim()); //x
          shape[3] = Double.parseDouble(
            st2.nextToken().trim()); //y
          shape[4] = Integer.parseInt(
            st2.nextToken().trim()); //r
          shape[5] = Integer.parseInt(
            st2.nextToken().trim()); //g
          shape[6] = Integer.parseInt(
            st2.nextToken().trim()); //b
          shape[7] = 0; //dummy
          shape[8] = 0; //dummy
          can.shapeList.add(shape);
        }
        else if(commandS.equals("DrawShapeLine"))
        //DrawShapeLine(double x,double y)
        //x:  直線終点座標X
        //y:  直線終点座標Y
        {
          shape = new double[9];
          shape[0] = -1; //命令識別（DrawShapeLine）
          shape[1] = Double.parseDouble(
            st2.nextToken().trim()); //x
          shape[2] = Double.parseDouble(
            st2.nextToken().trim()); //y
          shape[3] = 0; //dummy
          shape[4] = 0; //dummy
          shape[5] = 0; //dummy
          shape[6] = 0; //dummy
          shape[7] = 0; //dummy
          shape[8] = 0; //dummy
          can.shapeList.add(shape);
        }
        else if(commandS.equals("DrawShapeCurve"))
        //DrawShapeCurve(double x1,double y1,double x2,double y2,
        //  double x3,double y3)
        //x1:  曲線部第一制御点座標X
        //y1:  曲線部第一制御点座標Y
        //x2:  曲線部第二制御点座標X
        //y2:  曲線部第二制御点座標Y
        //x3:  曲線部終点座標X
        //y3:  曲線部終点座標Y
        {
          if(st2.countTokens() != 6 || Util.countDelimiter(command1, ',') != 5)
          {
            f.message(command1 + " >>引数の個数に異常");
            return;
          }
          shape = new double[9];
          shape[0] = -2; //命令識別（DrawShapeCurve）
          shape[1] = Double.parseDouble(
            st2.nextToken().trim()); //x1
          shape[2] = Double.parseDouble(
            st2.nextToken().trim()); //y1
          shape[3] = Double.parseDouble(
            st2.nextToken().trim()); //x2
          shape[4] = Double.parseDouble(
            st2.nextToken().trim()); //y2
          shape[5] = Double.parseDouble(
            st2.nextToken().trim()); //x3
          shape[6] = Double.parseDouble(
            st2.nextToken().trim()); //y3
          shape[7] = 0; //dummy
          shape[8] = 0; //dummy
          can.shapeList.add(shape);
        }
        else if(commandS.equals("DrawShapeEnd"))
        //DrawShapeEnd()
        //  引数なし。自由図形の終結宣言
        {
          shape = new double[9];
          shape[0] = -3; //命令識別（DrawShapeEnd）
          shape[1] = 0; //dummy
          shape[2] = 0; //dummy
          shape[3] = 0; //dummy
          shape[4] = 0; //dummy
          shape[5] = 0; //dummy
          shape[6] = 0; //dummy
          shape[7] = 0; //dummy
          shape[8] = 0; //dummy
          can.shapeList.add(shape);
        }
        else if(commandS.equals("DrawRect"))
        { //●長方形
        //DrawRect(int m, int x, int y, int w, int h, int r, int g, int b)
        //m:  描画モード、0は描画せず、1は線、2は色塗り、3は線と色
        //    4は線描と透明色塗り
        //x:  ｘ座標
        //y:  ｙ座標
        //w:  幅
        //h:  丈
        //r:  R
        //g:  G
        //b:  B
          if(st2.countTokens() != 8 || Util.countDelimiter(command1, ',') != 7)
          {
            f.message(command1 + " >>引数の個数に異常");
            return;
          }
          double[] d = new double[9];
           d[0] = -4; //命令識別（DrawRect）
           d[1] = Double.parseDouble(st2.nextToken().trim()); //m
           d[2] = Double.parseDouble(st2.nextToken().trim()); //x
           d[3] = Double.parseDouble(st2.nextToken().trim()); //y
           d[4] = Double.parseDouble(st2.nextToken().trim()); //w
           d[5] = Double.parseDouble(st2.nextToken().trim()); //h
           d[6] = Double.parseDouble(st2.nextToken().trim()); //r
           d[7] = Double.parseDouble(st2.nextToken().trim()); //g
           d[8] = Double.parseDouble(st2.nextToken().trim()); //b
           can.shapeList.add(d);
        }
        else if(commandS.equals("DrawEllipse"))
        { //●楕円の描画指定
        //DrawEllipse(double m, double x, double y, 
        //  double hr, double vr, double r, double g, double b)
        //m:  描画モード、0は描画せず、1は線、2は色塗り、3は線と色
        //    4は線描と透明色塗り
        //x:  楕円の中心のｘ座標
        //y:  楕円の中心のｙ座標
        //hr: 横半径
        //vr: 縦半径
        //r:  赤色
        //g:  緑色
        //b:  青色
          if(st2.countTokens() != 8 || Util.countDelimiter(command1, ',') != 7)
          {
            f.message(command1 + " >>引数の個数に異常");
            return;
          }
          double[] d = new double[9];
          d[0] = -5; //命令識別（DrawEllipse）
          d[1] = Double.parseDouble(st2.nextToken().trim()); //m
          d[2] = Double.parseDouble(st2.nextToken().trim()); //x
          d[3] = Double.parseDouble(st2.nextToken().trim()); //y
          d[4] = Double.parseDouble(st2.nextToken().trim()); //hr
          d[5] = Double.parseDouble(st2.nextToken().trim()); //vr
          d[6] = Double.parseDouble(st2.nextToken().trim()); //r
          d[7] = Double.parseDouble(st2.nextToken().trim()); //g
          d[8] = Double.parseDouble(st2.nextToken().trim()); //b
          can.shapeList.add(d);
        }
        else if(commandS.equals("DrawCircle"))
        { //●真円の描画指定
        //DrawCircle(double m, double x, double y, 
        //  double rr, double r, double g, double b)
        //m:  描画モード、0は描画せず、1は線、2は色塗り、3は線と色
        //    4は線描と透明色塗り
        //x:  円の中心のｘ座標
        //y:  円の中心のｙ座標
        //rr: 半径
        //r:  赤色
        //g:  緑色
        //b:  青色
          if(st2.countTokens() != 7 || Util.countDelimiter(command1, ',') != 6)
          {
            f.message(command1 + " >>引数の個数に異常");
            return;
          }
          double[] d = new double[9];
          d[0] = -5; //命令識別（DrawEllipse）
          d[1] = Double.parseDouble(st2.nextToken().trim()); //m
          d[2] = Double.parseDouble(st2.nextToken().trim()); //x
          d[3] = Double.parseDouble(st2.nextToken().trim()); //y
          d[4] = Double.parseDouble(st2.nextToken().trim()); //rr
          d[5] = d[4];
          d[6] = Double.parseDouble(st2.nextToken().trim()); //r
          d[7] = Double.parseDouble(st2.nextToken().trim()); //g
          d[8] = Double.parseDouble(st2.nextToken().trim()); //b
          can.shapeList.add(d);
        }
        //●●色塗り図形描画命令「解読」グループの終わり

        else if(commandS.equals("FontBold"))
        { //●太字指定
        //FontBold(bool b)
        //b:        太字指定ならtrue
          String value = st2.nextToken().trim();
          if(value.equals("true"))
          {
            can.fontBold = true;
          }
          else
          {
            can.fontBold = false;
          }
        }
        else if(commandS.equals("WriteStringH") || commandS.equals("WriteString"))
        { //●文字列の描画指定（省略形の「WriteString」も許容）
        //WriteStringH(double x,double y,int size, string style, string s)
        //x:   ｘ座標
        //y:   ｙ座標
        //size:  フォント・サイズ
        //style: 字形（"sansserif" / "serif"）
        //s:   表示する文字列
          double[] d = new double[3];
          String[] s = new String[2];
          d[0] = Double.parseDouble(st2.nextToken().trim()); //x
          d[1] = Double.parseDouble(st2.nextToken().trim()); //y
          d[2] = Double.parseDouble(st2.nextToken().trim()); //size
          s[0] = Util.trimAffix(st2.nextToken().trim(),'\"','\"');
          s[1] = Util.trimAffix(st2.nextToken().trim(),'\"','\"');
          can.stringHListD.add(d);
          can.stringHListS.add(s);
        }
        else if(commandS.equals("WriteStringB"))
        { //●文字列の描画指定（極太字）補助的用途
        //WriteStringB(double x,double y,int size, string style, string s)
        //x:   ｘ座標
        //y:   ｙ座標
        //size:  フォント・サイズ
        //style: 字形（sansserif / serif）
        //s:    表示する文字列
          double[] d = new double[3];
          String[] s = new String[2];
          d[0] = Double.parseDouble(st2.nextToken().trim()); //x
          d[1] = Double.parseDouble(st2.nextToken().trim()); //y
          d[2] = Double.parseDouble(st2.nextToken().trim()); //size
          s[0] = Util.trimAffix(st2.nextToken().trim(),'\"','\"');
          s[1] = Util.trimAffix(st2.nextToken().trim(),'\"','\"');
          can.stringBListD.add(d);
          can.stringBListS.add(s);
        }
        else if(commandS.equals("WriteStringBMulti"))
        { //●文字列の描画指定（極太字）の機能拡大。
          //極太文字を拡大しつつ輪郭を滑らかにする。
          //複数のWriteStringB命令に変換する。
        //WriteStringBMulti(double x,double y,int size,int px, string style, string s)
        //x:     ｘ座標
        //y:     ｙ座標
        //px:    文字シフトのピクセル数
        //size:  フォント・サイズ
        //style: 字形（sansserif / serif）
        //s:     表示する文字列
          double x = Double.parseDouble(st2.nextToken().trim()); //x
          double y = Double.parseDouble(st2.nextToken().trim()); //y
          double sz = Double.parseDouble(st2.nextToken().trim()); //font size
          double px = Double.parseDouble(st2.nextToken().trim()); //shiht pixel
          String s1 = Util.trimAffix(st2.nextToken().trim(),'\"','\"');
          String s2 = Util.trimAffix(st2.nextToken().trim(),'\"','\"');

          double[] d = new double[3];
          String[] s = new String[2];
          d[0] = x+px;
          d[1] = y+px;
          d[2] = sz;
          s[0] = s1;
          s[1] = s2;
          can.stringBListD.add(d);
          can.stringBListS.add(s);
          d = new double[3];
          s = new String[2];
          d[0] = x-px;
          d[1] = y-px;
          d[2] = sz;
          s[0] = s1;
          s[1] = s2;
          can.stringBListD.add(d);
          can.stringBListS.add(s);
          d = new double[3];
          s = new String[2];
          d[0] = x+px;
          d[1] = y-px;
          d[2] = sz;
          s[0] = s1;
          s[1] = s2;
          can.stringBListD.add(d);
          can.stringBListS.add(s);
          d = new double[3];
          s = new String[2];
          d[0] = x-px;
          d[1] = y+px;
          d[2] = sz;
          s[0] = s1;
          s[1] = s2;
          can.stringBListD.add(d);
          can.stringBListS.add(s);
          //---------------------------
          d = new double[3];
          s = new String[2];
          d[0] = x;
          d[1] = y+px;
          d[2] = sz;
          s[0] = s1;
          s[1] = s2;
          can.stringBListD.add(d);
          can.stringBListS.add(s);
          d = new double[3];
          s = new String[2];
          d[0] = x+px;
          d[1] = y;
          d[2] = sz;
          s[0] = s1;
          s[1] = s2;
          can.stringBListD.add(d);
          can.stringBListS.add(s);
          d = new double[3];
          s = new String[2];
          d[0] = x;
          d[1] = y-px;
          d[2] = sz;
          s[0] = s1;
          s[1] = s2;
          can.stringBListD.add(d);
          can.stringBListS.add(s);
          d = new double[3];
          s = new String[2];
          d[0] = x-px;
          d[1] = y;
          d[2] = sz;
          s[0] = s1;
          s[1] = s2;
          can.stringBListD.add(d);
          can.stringBListS.add(s);
        }
        else if(commandS.equals("WriteStringV"))
        { //●文字列の垂直描画指定（疑似縦書き）
        //「疑似縦書き」を実行すると、画像の精細度が低下するため、
        //この命令（WriteStringV）を使用しないことが望ましい。
        //WriteStringV(double x,double y,int size, 
        //     string style, string s)
        //x:   ｘ座標
        //y:   ｙ座標
        //size:  フォント・サイズ
        //style: 字形（"sansserif" / "serif"）
        //s:   表示する文字列
          double[] d = new double[3];
          String[] s = new String[2];
          d[0] = Double.parseDouble(st2.nextToken().trim()); //x
          d[1] = Double.parseDouble(st2.nextToken().trim()); //y
          d[2] = Double.parseDouble(st2.nextToken().trim()); //size
          s[0] = Util.trimAffix(st2.nextToken().trim(),'\"','\"');
          s[1] = Util.trimAffix(st2.nextToken().trim(),'\"','\"');
          can.stringVListD.add(d);
          can.stringVListS.add(s);
        }
        else if(commandS.equals("Rotate"))
        { //●画像全体の90度回転指定
          //Rotate命令は、正方形の画像に対して行なうこと。
          //長方形画像なら、Size命令で指定した正方形の中にこれを収納し、
          //回転処理し保存した上で、事後的に必要部分を切り取る。
        //Rotate(bool b)
        //b:  trueの時だけ90度回転
          String value = st2.nextToken().trim();
          if(value.equals("true"))
            can.rotateFlag = true;
          else
            can.rotateFlag = false;
        }
        else if(commandS.equals("RotateFree"))
        { //●画像全体の指定角度回転指定
          //マトリックス・パターンの傾斜を想定した命令で、
          //巨大画像を作成し回転させて、事後的に必要部分を切り取る。
          //メニューの「見本」を参照。
        //RotateFree(int degree, int w, int h)
            can.rotateFreeFlag = true;
            can.affineDegreeFree = Integer.parseInt(st2.nextToken().trim());
            can.rotateFreeW = Integer.parseInt(st2.nextToken().trim());
            can.rotateFreeH = Integer.parseInt(st2.nextToken().trim());
        }
        else if(commandS.equals("RotateMulti"))
        { //●画像多段階回転重複描画
          //メニューの「見本」を参照。
        //RotateMulti(int startDegree, int degree, int times, String fName)
        //startDegree:      回転開始角度
        //degree:           回転角度
        //times:            回転回数
        //fName:            回転対象画像ファイル名
            can.rotateMultiFlag = true;
            can.rotateMultiStartDegree = Integer.parseInt(st2.nextToken().trim());
            can.rotateMultiDegree = Integer.parseInt(st2.nextToken().trim());
            can.rotateMultiTimes = Integer.parseInt(st2.nextToken().trim());
            can.rotateMultiF = Util.trimAffix(st2.nextToken().trim(),'\"','\"');
        }
        else if(commandS.equals("StripePattern"))
        { //●背景画像にランダム色のストライプ模様を表示
        //StripePattern(int stripePatternType,int stripeW,
        //  int stripePRedBase, int stripePGreenBase,int stripePBlueBase)
        //stripePatternType: ストライプ無効または水平または垂直、0/1/2
        //stripePatternW:    ストライプ幅
        //stripePRedBase:    赤基調色
        //stripePGreenBase:  緑基調色
        //stripePBlueBase:   青基調色
          if(st2.countTokens() != 5 || Util.countDelimiter(command1, ',') != 4)
          {
            f.message(command1 + " >>引数の個数に異常");
            return;
          }
          can.stripePatternType  = Integer.parseInt(st2.nextToken().trim());
          can.stripePatternW   = Integer.parseInt(st2.nextToken().trim());
          can.stripePRedBase   = Integer.parseInt(st2.nextToken().trim());
          can.stripePGreenBase = Integer.parseInt(st2.nextToken().trim());
          can.stripePBlueBase  = Integer.parseInt(st2.nextToken().trim());
        }
        else if(commandS.equals("MatrixPatternSt"))
        { //●背景画像にマトリックス模様を表示（統一配色の基本設定）
        //MatrixPattern(int matrixPatternNumSt, int matrixPatternWH_St
        //  int matrixPRedBaseSt, int matrixPGreenBaseSt, int matrixPBlueBaseSt
        //  int matrixColorLimitSt)
        //matrixPatternNumSt: 描画無効または柄紋番号
        //matrixPatternWH_St: 柄紋の幅丈
        //matrixPRedBaseSt:   赤基調色
        //matrixPGreenBaseSt: 緑基調色
        //matrixPBlueBaseSt:  青基調色
        //matrixColorLimitSt: 色成分上限設定（1～10）
          if(st2.countTokens() != 6 || Util.countDelimiter(command1, ',') != 5)
          {
            f.message(command1 + " >>引数の個数に異常");
            return;
          }
          can.matrixPatternNumSt = Integer.parseInt(st2.nextToken().trim());
          can.matrixPatternWH_St = Integer.parseInt(st2.nextToken().trim());
          can.matrixPRedBaseSt   = Integer.parseInt(st2.nextToken().trim());
          can.matrixPGreenBaseSt = Integer.parseInt(st2.nextToken().trim());
          can.matrixPBlueBaseSt  = Integer.parseInt(st2.nextToken().trim());
          can.matrixColorLimitSt = Integer.parseInt(st2.nextToken().trim());
          if(can.matrixColorLimitSt<1)
            can.matrixColorLimitSt = 1;
          else if(can.matrixColorLimitSt>10)
            can.matrixColorLimitSt = 10;
        }
        else if(commandS.equals("MatrixPatternStOp"))
        { //●背景画像にマトリックス模様を表示（統一配色のオプション）
        //MatrixPattern(
        //  double matrixShiftH_St, double matrixShiftV_St,
        //  double matrixDropH_St, double matrixDropV_St,
        //  double matrixSkipH_St, double matrixSkipV_St,
        //  int matrixRepeatH_StX, int matrixRepeatV_StX,
        //  matrixIntermittingSt)
        //matrixShiftH_St: 全体横シフト（コマ数）コマ＝小間
        //matrixShiftV_St: 全体縦シフト（コマ数）
        //matrixDropH_St:  柄紋横ドロップ（コマ数）
        //matrixDropV_St:  柄紋縦ドロップ（コマ数）
        //matrixSkipH_St:  柄紋横スキップ（コマ数）
        //matrixSkipV_St:  柄紋縦スキップ（コマ数）
        //matrixRepeatH_StX:  柄紋横リピート指定（回数）
        //matrixRepeatV_StX:  柄紋縦リピート指定（回数）
        //matrixIntermittingSt:  間欠描画（1～10）
          if(st2.countTokens() != 9 || Util.countDelimiter(command1, ',') != 8)
          {
            f.message(command1 + " >>引数の個数に異常");
            return;
          }
          can.matrixShiftH_St    = Double.parseDouble(st2.nextToken().trim());
          can.matrixShiftV_St    = Double.parseDouble(st2.nextToken().trim());
          can.matrixDropH_St     = Double.parseDouble(st2.nextToken().trim());
          can.matrixDropV_St     = Double.parseDouble(st2.nextToken().trim());
          can.matrixSkipH_St     = Double.parseDouble(st2.nextToken().trim());
          can.matrixSkipV_St     = Double.parseDouble(st2.nextToken().trim());
          can.matrixRepeatH_StX  = Integer.parseInt(st2.nextToken().trim());
          can.matrixRepeatV_StX  = Integer.parseInt(st2.nextToken().trim());
          can.matrixIntermittingSt = Integer.parseInt(st2.nextToken().trim());
          if(can.matrixIntermittingSt > 10)
            can.matrixIntermittingSt = 10;
          if(can.matrixIntermittingSt <= 0)
            can.matrixIntermittingSt = 1;
        }
        else if(commandS.equals("MatrixPatternEx"))
        { //●背景画像にマトリックス模様を表示（個別配色の基本設定）
        //MatrixPatternEx(int matrixPatternNumEx, int matrixPatternWH_Ex,
        //  int matrixPRedBase, int matrixPGreenBase, int matrixPBlueBase,
        //  int matrixColorLimitEx)
        //matrixPatternNumEx:  描画無効または柄紋種類
        //matrixPatternWH_Ex:  柄紋の幅丈
        //matrixShiftH_Ex:     全体横シフト（コマ数）
        //matrixShiftV_Ex:     全体縦シフト（コマ数）
        //matrixPRedBaseEx:    赤基調色
        //matrixPGreenBaseEx:  緑基調色
        //matrixPBlueBaseEx:   青基調色
        //matrixColorLimitEx:  色成分上限設定（1～10）
          if(st2.countTokens() != 6 || Util.countDelimiter(command1, ',') != 5)
          {
            f.message(command1 + " >>引数の個数に異常");
            return;
          }
          can.matrixPatternNumEx = Integer.parseInt(st2.nextToken().trim());
          can.matrixPatternWH_Ex = Integer.parseInt(st2.nextToken().trim());
          can.matrixPRedBaseEx   = Integer.parseInt(st2.nextToken().trim());
          can.matrixPGreenBaseEx = Integer.parseInt(st2.nextToken().trim());
          can.matrixPBlueBaseEx  = Integer.parseInt(st2.nextToken().trim());
          can.matrixColorLimitEx = Integer.parseInt(st2.nextToken().trim());
          if(can.matrixColorLimitEx<1)
            can.matrixColorLimitEx = 1;
          else if(can.matrixColorLimitEx>10)
            can.matrixColorLimitEx = 10;
        }
        else if(commandS.equals("MatrixPatternExOp"))
        { //●背景画像にマトリックス模様を表示（個別配色のオプション）
        //MatrixPatternExOp(
        //  double matrixShiftH_Ex, double matrixShiftV_Ex,
        //  double matrixDropH_Ex, double matrixDropV_Ex,
        //  double matrixSkipH_Ex, double matrixSkipV_Ex,
        //  int matrixRepeatH_ExX, int matrixRepeatV_ExX,
        //  matrixIntermittingEx)
        //matrixShiftH_Ex: 全体横シフト（コマ数）コマ＝小間
        //matrixShiftV_Ex: 全体縦シフト（コマ数）
        //matrixDropH_Ex:  柄紋横ドロップ（コマ数）
        //matrixDropV_Ex:  柄紋縦ドロップ（コマ数）
        //matrixSkipH_Ex:  柄紋横スキップ（コマ数）
        //matrixSkipV_Ex:  柄紋縦スキップ（コマ数）
        //matrixRepeatH_ExX:  柄紋横リピート指定（回数）
        //matrixRepeatV_ExX:  柄紋縦リピート指定（回数）
        //matrixIntermittingEx:  間欠描画（1～10）
          if(st2.countTokens() != 9 || Util.countDelimiter(command1, ',') != 8)
          {
            f.message(command1 + " >>引数の個数に異常");
            return;
          }
          can.matrixShiftH_Ex    = Double.parseDouble(st2.nextToken().trim());
          can.matrixShiftV_Ex    = Double.parseDouble(st2.nextToken().trim());
          can.matrixDropH_Ex     = Double.parseDouble(st2.nextToken().trim());
          can.matrixDropV_Ex     = Double.parseDouble(st2.nextToken().trim());
          can.matrixSkipH_Ex     = Double.parseDouble(st2.nextToken().trim());
          can.matrixSkipV_Ex     = Double.parseDouble(st2.nextToken().trim());
          can.matrixRepeatH_ExX  = Integer.parseInt(st2.nextToken().trim());
          can.matrixRepeatV_ExX  = Integer.parseInt(st2.nextToken().trim());
          can.matrixIntermittingEx = Integer.parseInt(st2.nextToken().trim());
          if(can.matrixIntermittingEx > 10)
            can.matrixIntermittingEx = 10;
          if(can.matrixIntermittingEx <= 0)
            can.matrixIntermittingEx = 1;
        }

        else
        { //●命令文が異常
          f.message(command1 + " >>命令スペル・エラー");
          can.commandInspection = false;
          return;
        }
      } // for文の終結
    }
    catch(NumberFormatException e)
    {
      f.message(command1 + " >>引数の数値表現エラー");
      can.commandInspection = false;
      return;
    }
    catch(Exception e)
    {
      f.message(e.toString());
      f.message(command1 + " >>各種エラー");
      can.commandInspection = false;
      return;
    }
  }
} //EOF (GCreatorAnalyzeEngine.java)
