//■■GCreatorCanMouseLis.java
//Ver.3.4

//■■インポート
import java.awt.*;  //必須
import java.awt.event.*;  //必須
import javax.swing.*;  //必須
import java.util.*;  //必須

import java.awt.geom.*; 
import java.awt.image.*;
import java.io.*;
import javax.imageio.*; 
import javax.imageio.stream.*;

//■■キャンバス上のマウスのクリックで呼び出されるリスナー。
//描画命令を自動生成して、cAreaに順次書き込む。
class GCreatorCanMouseLis implements MouseListener
{
  //■メンバー変数
  GCreator f;
  int x1 = 0;
  int y1 = 0;
  int x2 = 0;
  int y2 = 0;
  int x3 = 0;
  int y3 = 0;
  int x4 = 0;
  int y4 = 0;
  double brushSize = 4; //点描画用の筆のデフォルト・サイズ
  int strCnt = 1;  //文字列描画命令における文字列番号
  int curveCnt = 1;  //ベジェ曲線ポイント番号（始点が1、終点が4）
  int linkedLineCnt = 1;  //折れ線ポイント番号（始点が1、継続点が2以上、終点で1に戻される）
  int sprayCnt = 1;  //スプレーのポイント番号（始点が1、それ以外が2以上）
  int splineCnt = 1;  //スプライン曲線ポイント番号（始点が1、制御点が2以上）
  int splineX0;
  int splineY0;
  int canvasMouseMode = 0;  //0が無効、1以上が線描画マウス入力機能種別
  boolean canvasMouseFlag = false;
  boolean startFlag = true;  //線の始点か否かの判定
  boolean horizontFlag = true; //楕円の横半径指定クリックか否かの判定
  String commandX = ""; //自由図形での右クリック曲線制御点での命令文
  boolean okButton = false; //塗り色変更の実行または中止


  //■コンストラクタ
  GCreatorCanMouseLis(GCreator flame)
  {
    f = flame;
  }

  //■イベントハンドラ
  public void mousePressed(MouseEvent eve)
  {
    //●マウス入力モードが両方解除状態なら
    if(canvasMouseFlag == false)
    { 
      clear();
      return;
    }
    //●特殊モードには非対応
    if(f.canvas.rotateFlag  || f.canvas.rotateFreeFlag ||
      f.canvas.shiftMode!=0 || f.canvas.freeScale!=1.0)
    { 
      f.message("Rotate、RotateFree、Shift、FreeScale がある命令文書は、" +
        "マウス操作に非対応。該当命令文をコメント・アウトして下さい。");
      return;
    }
    int x = eve.getX(); //マウスのクリック位置を取得
    int y = eve.getY();
    
    //（注意）マウス入力モードでは、マウス・クリックに応じて命令文が生成され、
    //コマンド・エリアにその命令文が自動的に追加されるため、
    //事前に、コマンド・エリア内でのカーソル位置が、行頭になくてはならない。
    //コメント行の末尾にカーソルがある状態でマウス・クリックがなされれば、
    //その命令文もコメント・アウトされて、機能しなくなる。
    
    //●線描画モードで、左ボタンがクリックされた場合の動作
    if((canvasMouseFlag == true)  &&
      (eve.getButton() == MouseEvent.BUTTON1) )
    {
      f.canvas.markFlag = true; //目印線表示
      if(canvasMouseMode == 1 && startFlag) 
      { //●直線の第一段階
        //マウスクリック位置の端数を四捨五入
        x1 = (int)(x/f.canvas.affineScale +0.5); //端数四捨五入
        y1 = (int)(y/f.canvas.affineScale +0.5);
        startFlag = false;
        f.canvas.currentPointX = x1;
        f.canvas.currentPointY = y1;
      }
      else if(canvasMouseMode == 1 && !startFlag)
      { //●直線の第二段階
        x2 = (int)(x/f.canvas.affineScale +0.5); //端数四捨五入
        y2 = (int)(y/f.canvas.affineScale +0.5);
        String method = "DrawLine(" + 
          x1 + "," + y1 + "," + 
          x2 + "," + y2 + ");";
        f.cArea.append(method + "\n"); 
        startFlag = true;
        f.canvas.currentPointX = x2;
        f.canvas.currentPointY = y2;
        clear();
      }

      else if(canvasMouseMode == 20 && startFlag) 
      { //●四角形の第一段階
        //マウスクリック位置の端数を四捨五入
        x1 = (int)(x/f.canvas.affineScale +0.5); //端数四捨五入
        y1 = (int)(y/f.canvas.affineScale +0.5);
        startFlag = false;
        f.canvas.currentPointX = x1;
        f.canvas.currentPointY = y1;
      }
      else if(canvasMouseMode == 20 && !startFlag)
      { //●四角形の第二段階
        x2 = (int)(x/f.canvas.affineScale +0.5); //端数四捨五入
        y2 = (int)(y/f.canvas.affineScale +0.5);
        //m: 描画モード、0は描画せず、1は線、2は色塗り、3は線と色
        //x: ｘ座標
        //y: ｙ座標
        //w: 幅
        //h: 丈
        //r: R
        //g: G
        //b: B
        int rc = colorRanNum();
        int gc = colorRanNum();
        int bc = colorRanNum();
        int w = x2 - x1;
        int h = y2 - y1;
        if(w<0) w = w* -1; //マイナスは不可
        if(h<0) h = h* -1; //マイナスは不可
        String method = "DrawRect(3," + 
          x1 + "," + y1 + "," + 
          w + "," + h + "," + 
          rc +"," + gc + "," + bc + ");";
        f.cArea.append(method + "\n");
        startFlag = true;
        f.canvas.currentPointX = x2;
        f.canvas.currentPointY = y2;
        clear();
      }

      else if((canvasMouseMode == 2) && startFlag)
      { //●真円の第一段階：円の中心をクリック
        x1 = (int)(x/f.canvas.affineScale +0.5); //端数四捨五入
        y1 = (int)(y/f.canvas.affineScale +0.5);
        f.canvas.currentPointX = x1;
        f.canvas.currentPointY = y1;
        startFlag = false;
      }
      else if((canvasMouseMode == 2)&& !startFlag)
      {  //●真円の第二段階：円の右端か左端をクリック
        x2 = (int)(x/f.canvas.affineScale +0.5)-x1; //端数四捨五入
        if(x2 <0)
          x2 = x2* -1; //絶対値に
        if(x2 == 0)
          x2 = 1; //半径0は、１に強制変更
        if(canvasMouseMode == 2) //●真円
        {
          int rc = colorRanNum();
          int gc = colorRanNum();
          int bc = colorRanNum();
          String method = "DrawCircle(3," +
            x1 + "," + y1 + "," + x2+ "," + 
            rc +"," + gc + "," + bc + ");";
          f.cArea.append(method + "\n");
        }
        startFlag = true;
        clear();
      }

      else if((canvasMouseMode == 8) && startFlag)
      { //●楕円の第一段階：中心をクリック
        x1 = (int)(x/f.canvas.affineScale +0.5); //端数四捨五入
        y1 = (int)(y/f.canvas.affineScale +0.5);
        f.canvas.currentPointX = x1;
        f.canvas.currentPointY = y1;
        startFlag = false;
      }
      else if((canvasMouseMode == 8) && !startFlag && 
        horizontFlag)
      {  //●楕円の第二段階：右端か左端をクリック
        x2 = (int)(x/f.canvas.affineScale +0.5)-x1; //端数四捨五入
        if(x2 <0)
          x2 = x2* -1;
        if(x2 == 0)
          x2 = 1;
        horizontFlag = false;
        f.canvas.currentPointX = x1;
        f.canvas.currentPointY = y1;
      }
      else if((canvasMouseMode == 8) && 
        !startFlag && !horizontFlag)
      {  //●楕円の第三段階：上端か下端をクリック
        y2 = (int)(y/f.canvas.affineScale +0.5)-y1; //端数四捨五入
        if(y2 <0)
          y2 = y2* -1;
        if(y2 == 0)
          y2 = 1;
        if(canvasMouseMode == 8) //●楕円
        {
          int rc = colorRanNum();
          int gc = colorRanNum();
          int bc = colorRanNum();
          String method = "DrawEllipse(3," + 
            x1 + "," + y1 + "," + x2 + "," + y2 + "," +
            rc +"," + gc + "," + bc + ");";
          f.cArea.append(method + "\n");
        }
        startFlag = true;
        horizontFlag = true;
        clear();
      }

      else if(canvasMouseMode == 9 && curveCnt == 1)
      { //●単独ベジェ曲線の第一段階：曲線の始点をクリック
        x1 = (int)(x/f.canvas.affineScale +0.5); //端数四捨五入
        y1 = (int)(y/f.canvas.affineScale +0.5);
        f.canvas.currentPointX = x1;
        f.canvas.currentPointY = y1;
        curveCnt++;
      }
      else if(canvasMouseMode == 9 && curveCnt == 2)
      {  //●単独ベジェ曲線の第二段階：曲線の第二点をクリック
        x2 = (int)(x/f.canvas.affineScale +0.5); //端数四捨五入
        y2 = (int)(y/f.canvas.affineScale +0.5);
        curveCnt++;
        String method = "  DrawLine2(3," + x1 + "," + y1 + "," +
            x2 + "," + y2 + ");";
        f.canvas.currentPointX = x2;
        f.canvas.currentPointY = y2;
        f.cArea.append(method + "\n");
      }
      else if(canvasMouseMode == 9 && curveCnt == 3)
      {  //●単独ベジェ曲線の第三段階：曲線の第三点をクリック
        x3 = (int)(x/f.canvas.affineScale +0.5); //端数四捨五入
        y3 = (int)(y/f.canvas.affineScale +0.5);
        curveCnt++;
        String method = "  DrawLine2(2," + x2 + "," + y2 + "," +
            x3 + "," + y3 + ");";
        f.canvas.currentPointX = x3;
        f.canvas.currentPointY = y3;
        f.cArea.append(method + "\n");
      }
      else if(canvasMouseMode == 9 && curveCnt == 4)
      {  //●単独ベジェ曲線の第四段階：曲線の終点をクリック
        x4 = (int)(x/f.canvas.affineScale +0.5); //端数四捨五入
        y4 = (int)(y/f.canvas.affineScale +0.5);
        String method = "  DrawLine2(3," + x3 + "," + y3 + "," +
            x4 + "," + y4 + ");";
        f.cArea.append(method + "\n");
        method = 
          "DrawCurve(" +
          x1 + "," + y1 + "," +
          x2 + "," + y2 + "," +
          x3 + "," + y3 + "," +
          x4 + "," + y4 +  ");";
        f.cArea.append(method + "\n");
        f.canvas.currentPointX = x4;
        f.canvas.currentPointY = y4;
        curveCnt = 1;
        erase("DrawLine2");
        clear();
      }

      else if(canvasMouseMode == 13)
      { //●文字列描画
        int mX = (int)(x/f.canvas.affineScale +0.5); //端数四捨五入
        int mY = (int)(y/f.canvas.affineScale +0.5);
        String method = "WriteStringH(" + mX + "," + mY + "," +
          15 + "," + "\"sansserif\"" + ",\"□" + strCnt + "\");";
        f.cArea.append(method + "\n");
        strCnt++;
        startFlag = true;
        clear();
      }

      else if(canvasMouseMode == 4)
      { //●位置確認モード（FreeScale命令には、非対応）
        int mX = (int)(x/f.canvas.affineScale +0.5); //端数四捨五入
        int mY = (int)(y/f.canvas.affineScale +0.5);
        f.canvas.currentPointX = mX;
        f.canvas.currentPointY = mY;
        f.message("X=" + x + "  Y=" + y + "  [X=" + mX + "  Y=" + mY + "]");
        startFlag = true;
        clear();
      }

      else if(canvasMouseMode == 21)
      { //●色相確認モード
        //（注意）currentPointXおよびcurrentPointYの目印Ｘ字線をクリック位置に表示してはならない。
        //また、真円や楕円の中心点目印十字線も、中心点をクリックする可能性があり、表示してはならない。
        //これらを表示すると、その目印線の色相データが採取されてしまう。
        f.canvas.markFlag = false; //目印線消去
        int cR = 0;
        int cG = 0;
        int cB = 0;
        BufferedImage bImage = f.io.canvas2BufferedImage(0, 0, 
          (int)(f.canvas.imagePX_W*f.canvas.affineScale*f.canvas.freeScale),  //端数四捨五入
          (int)(f.canvas.imagePX_H*f.canvas.affineScale*f.canvas.freeScale)); //画像全体
        int rgbValue =  bImage.getRGB(x, y); 
          //getRGB(x,y)は、デフォルトRGBカラー・モデル(TYPE_INT_ARGB)および
          //デフォルトsRGBカラー・スペースで整数型ピクセルを返すメソッド。
          //ただし、実際は、互換タイプ（TYPE_INT_RGB）に変換されている。
        String errMsg = "";
        cR = (rgbValue & 0x00ff0000) >>> 16;
        cG = (rgbValue & 0x0000ff00) >>> 8;
        cB = (rgbValue & 0x000000ff);
        String type = 
          errMsg +
          "RGB={" + cR + ", " + cG + ", " + cB +"} " +
          "RGB_hex={" + Integer.toHexString(rgbValue) + "} ";
        int colorType = bImage.getType();
        if(colorType == BufferedImage.TYPE_3BYTE_BGR)
          f.message(type+"TYPE_3BYTE_BGR");
        else if(colorType == BufferedImage.TYPE_4BYTE_ABGR)
          f.message(type+"TYPE_4BYTE_ABGR");
        else if(colorType == BufferedImage.TYPE_4BYTE_ABGR_PRE)
          f.message(type+"TYPE_4BYTE_ABGR_PRE");
        else if(colorType == BufferedImage.TYPE_BYTE_BINARY)
          f.message(type+"TYPE_BYTE_BINARY");
        else if(colorType == BufferedImage.TYPE_BYTE_GRAY)
          f.message(type+"TYPE_BYTE_GRAY");
        else if(colorType == BufferedImage.TYPE_BYTE_INDEXED)
          f.message(type+"TYPE_BYTE_INDEXED");
        else if(colorType == BufferedImage.TYPE_CUSTOM)
          f.message(type+"TYPE_CUSTOM");
        else if(colorType == BufferedImage.TYPE_INT_ARGB)
          f.message(type+"TYPE_INT_ARGB");
        else if(colorType == BufferedImage.TYPE_INT_ARGB_PRE)
          f.message(type+"TYPE_INT_ARGB_PRE");
        else if(colorType == BufferedImage.TYPE_INT_BGR)
          f.message(type+"TYPE_INT_BGR");
        else if(colorType == BufferedImage.TYPE_INT_RGB)
          f.message(type+"TYPE_INT_RGB");
        else if(colorType == BufferedImage.TYPE_USHORT_555_RGB)
          f.message(type+"TYPE_USHORT_555_RGB");
        else if(colorType == BufferedImage.TYPE_USHORT_565_RGB)
          f.message(type+"TYPE_USHORT_565_RGB");
        else if(colorType == BufferedImage.TYPE_USHORT_GRAY)
          f.message(type+"TYPE_USHORT_GRAY");
        else
          f.message(type+"カラー・モデル不明" );
        startFlag = true;
        clear();
      }

      else if(canvasMouseMode == 15)
      { //●命令文検索
        int mX = (int)(x/f.canvas.affineScale +0.5);
        int mY = (int)(y/f.canvas.affineScale +0.5);
        refer(mX,mY);
        startFlag = true;
        clear();
      }

      else if((canvasMouseMode == 6) && startFlag)
      { //●直線の消去における始点位置
        x1 = (int)(x/f.canvas.affineScale +0.5);
        y1 = (int)(y/f.canvas.affineScale +0.5);
        startFlag = false;
      }
      else if((canvasMouseMode == 6) && !startFlag)
      { //●直線の消去における終点位置
        x2 = (int)(x/f.canvas.affineScale +0.5);
        y2 = (int)(y/f.canvas.affineScale +0.5);
        eraseLine(x1, y1, x2, y2);
        startFlag = true;
        clear();
      }
      else if((canvasMouseMode == 32) )
      { //●真円の消去における中心位置
        int xC = (int)(x/f.canvas.affineScale +0.5);
        int yC = (int)(y/f.canvas.affineScale +0.5);
        eraseCircle(xC, yC);
        clear();
      }

      else if((canvasMouseMode == 10) && startFlag)
      { //●位置修正における旧点位置
        x1 = (int)(x/f.canvas.affineScale +0.5);
        y1 = (int)(y/f.canvas.affineScale +0.5);
        startFlag = false;
      }
      else if((canvasMouseMode == 10) && !startFlag)
      { //●位置修正における新点位置
        x2 = (int)(x/f.canvas.affineScale +0.5);
        y2 = (int)(y/f.canvas.affineScale +0.5);
        replacePoint(x1, y1, x2, y2);
        startFlag = true;
        clear();
      }

      else if(canvasMouseMode == 19 )
      { //●パレット混色での指定色で彩色変更。
        //パレットで混色指定済みの上で、図形指定（始点クリック）を行なうことで、
        //彩色変更がなされる。
        Color color = f.palette.mixColor2;
        if(color != null)
        {
          int cx = (int)(x/f.canvas.affineScale +0.5);
          int cy = (int)(y/f.canvas.affineScale +0.5);
          changeColor(cx, cy, color.getRed(), 
            color.getGreen(), color.getBlue());
        }
      }

      //●自由図形は、直線とベジェ曲線とを混用して閉じた輪郭線の図形を描画し、
      //必要に応じて、その内部を色塗りする。
      //最終的に輪郭線を閉じさせるには、自由図形全体の始点に戻って左クリック。
      //大まかに言えば、左クリックは直線の指定で、右クリックは曲線の指定になる。
      //ただし、自由図形全体の始点は、常に左クリックとする。
      //また、自由図形全体の終点（始点への帰還）が曲線の終点（右クリック）でもある場合は、
      //その終点で左クリックを一つ追加して終了させる。
      //なお、正確に言えば、次のルールである。
      //自由図形の始点は、左クリック。
      //曲線部の第一制御点と第二制御点と終点は、右クリック。
      //直線部の終点は、左クリック。
      //最終的に輪郭線を閉じさせるには、自由図形全体の始点に戻って左クリック。
      //ただし、自由図形全体の終点（始点への回帰）が曲線部の終点（右クリック）でもある場合は、
      //その終点で左クリックを一つ追加して終了させる。
      //輪郭線を閉じさせるには、始点への回帰した時、左クリックが必要である。
      else if(canvasMouseMode == 14 && startFlag)
      { //●自由図形の第一段階（▼始点）
        x1 = (int)(x/f.canvas.affineScale +0.5);
        y1 = (int)(y/f.canvas.affineScale +0.5);

        int rc = colorRanNum();
        int gc = colorRanNum();
        int bc = colorRanNum();
        String method = "DrawShapeStart(4," + x1 + "," + y1 + "," +
          rc +"," + gc + "," + bc + ");";
        f.cArea.append(method + "\n");
        f.canvas.currentPointX = x1;
        f.canvas.currentPointY = y1;
        x2=x1; //x1,y1は、自由図形の始点として温存する。
        y2=y1; //x2,y2は、中間線での起点とする。
        startFlag = false;
      }
      else if(canvasMouseMode == 14 && !startFlag)
      { //●自由図形内＜直線＞の第二段階（中間点と終点）
        x3 = (int)(x/f.canvas.affineScale +0.5); //x3,y3は、中間線での
        y3 = (int)(y/f.canvas.affineScale +0.5); //末点として扱う。
        int Q = 2; //許容誤差（特に始点と終点との一致判定での許容誤差）
        if( ((x1+Q) >= x3 && (x1-Q) <= x3)  &&
          ((y1+Q) >= y3 && (y1-Q) <= y3)  )
        { //指定位置の近傍点をクリックした場合、
          //僅かにずれたものと見なして、一致させる。
          x3=x1;
          y3=y1;
        }
        if(!(x1==x3 && y1==y3)) //▼自由図形＜直線＞中間点
        {
          String method = "DrawShapeLine(" +
            x3 + "," + y3 + ");";
          f.cArea.append(method + "\n");
          method = "  DrawLine2(0," + x2 + "," + y2 + "," +
            x3 + "," + y3 + ");"; //直線部の合図として、マイナス値
          f.canvas.currentPointX = x3;
          f.canvas.currentPointY = y3;
          f.cArea.append(method + "\n");
          x2=x3; //次の線の始点を次の段階に伝える
          y2=y3;
        }
        else //▼自由図形終点
        {
          String method = "DrawShapeEnd();"; 
          f.cArea.append(method + "\n");
          method = "  DrawLine2(0," + x2 + "," + y2 + "," +
            x3 + "," + y3 + ");";
          f.cArea.append( method + "\n" + "//\n");
          f.canvas.currentPointX = x1;
          f.canvas.currentPointY = y1;
          startFlag = true;
          erase("DrawLine2");
          clear();
        }
      }
      f.canvas.repaint();
    }

    //●線描画モードで、右ボタンがクリックされる可能性がある場合の動作。
    //自由図形、スプライン曲線、折れ線、スプレーの場合、右ボタンも使われる。

    if((canvasMouseFlag == true) && 
      (eve.getButton() == MouseEvent.BUTTON3))
    {
         //●自由図形内＜ベジェ曲線＞の第一段階：曲線の始点（何もしなくて良い）
         //　自由図形内＜ベジェ曲線＞の始点は、左クリックである。
      if(canvasMouseMode == 14 && curveCnt == 1)
      {  //●自由図形内＜ベジェ曲線＞の第二段階：曲線の第一制御点を右クリック。
        x3 = (int)(x/f.canvas.affineScale +0.5);
        y3 = (int)(y/f.canvas.affineScale +0.5);
        curveCnt++;
        f.canvas.currentPointX = x3;
        f.canvas.currentPointY = y3;
        commandX = "DrawShapeCurve(" + x3 + "," + y3 + ",";
        String method = "  DrawLine2(1," + x2 + "," + y2 + "," +
          x3 + "," + y3 + ");";
        f.cArea.append(method + "\n");
        x2 = x3; //次の線の始点を次の段階に伝える
        y2 = y3;
      }
      else if(canvasMouseMode == 14 && curveCnt == 2)
      {  //●自由図形内＜ベジェ曲線＞の第三段階：曲線の第二制御点を右クリック。
        x3 = (int)(x/f.canvas.affineScale +0.5);
        y3 = (int)(y/f.canvas.affineScale +0.5);
        curveCnt++;
        f.canvas.currentPointX = x3;
        f.canvas.currentPointY = y3;
        commandX = commandX + x3 + "," + y3 + ",";
        String method = "  DrawLine2(2," + x2 + "," + y2 + "," +
          x3 + "," + y3 + ");";
        f.cArea.append(method + "\n");
        x2 = x3; //次の線の始点を次の段階に伝える
        y2 = y3;
      }
      else if(canvasMouseMode == 14 && curveCnt == 3)
      {  //●自由図形内＜ベジェ曲線＞の第四段階：曲線の終点を右クリック。
        x3 = (int)(x/f.canvas.affineScale +0.5);
        y3 = (int)(y/f.canvas.affineScale +0.5);
        commandX =  commandX + x3 + "," + y3 + ");";
        f.cArea.append(commandX + "\n");
        f.canvas.currentPointX = x3;
        f.canvas.currentPointY = y3;
        String method = "  DrawLine2(3," + x2 + "," + y2 + "," +
          x3 + "," + y3 + ");";
        f.cArea.append(method + "\n");
        curveCnt = 1;
        x2 = x3; //次の線の始点を次の段階に伝える
        y2 = y3;
        commandX = "";
      }
      f.canvas.repaint();
    }

    if(canvasMouseMode == 22 && splineCnt == 1 && (eve.getButton() == MouseEvent.BUTTON1))
    { //●スプライン曲線の第一段階：曲線の始点をクリック
      x1 = (int)(x/f.canvas.affineScale +0.5); //端数四捨五入
      y1 = (int)(y/f.canvas.affineScale +0.5);
      f.canvas.currentPointX = x1;
      f.canvas.currentPointY = y1;
      String method;
      method = "DrawSplineStart(" + x1 + "," + y1 + ");";
      f.cArea.append(method + "\n");
      splineX0 = x1;
      splineY0 = y1;
      splineCnt++;
    }
    else if(canvasMouseMode == 22 && splineCnt > 1 && (eve.getButton() == MouseEvent.BUTTON1))
    { //●スプライン曲線の第二段階：曲線の制御点をクリック
      //制御点の位置指定が連続して同じ位置を間違ってクリックした場合、
      //終点を指定した後も、スプライン曲線が描画されず、補助線だけの表示となる。
      x1 = (int)(x/f.canvas.affineScale +0.5); //端数四捨五入
      y1 = (int)(y/f.canvas.affineScale +0.5);
      f.canvas.currentPointX = x1;
      f.canvas.currentPointY = y1;
      String method;
      method = "DrawSplineControl(" + x1 + "," + y1 + ");";
      f.cArea.append(method + "\n");
      method = "  DrawLine2(" + 3 + "," + splineX0 + "," + splineY0 +  "," + x1 + "," + y1 + ");";
      f.cArea.append(method + "\n");
      splineX0 = x1;
      splineY0 = y1;
      splineCnt++;
    }
    else if(canvasMouseMode == 22 && splineCnt > 2 && (eve.getButton() == MouseEvent.BUTTON3))
    { //●スプライン曲線の最終段階：曲線の終点をクリック
      x1 = (int)(x/f.canvas.affineScale +0.5); //端数四捨五入
      y1 = (int)(y/f.canvas.affineScale +0.5);
      f.canvas.currentPointX = x1;
      f.canvas.currentPointY = y1;
      String method = 
        "DrawSplineEnd(" + x1 + "," + y1 + ");";
      f.cArea.append(method + "\n");
      f.cArea.append("//\n");
      clear();
      erase("DrawLine2");
    }

    if(canvasMouseMode == 30 && linkedLineCnt == 1 && (eve.getButton() == MouseEvent.BUTTON1))
    { //●折れ線の第一段階：折れ線の始点をクリック
      x1 = (int)(x/f.canvas.affineScale +0.5); //端数四捨五入
      y1 = (int)(y/f.canvas.affineScale +0.5);
      f.canvas.currentPointX = x1;
      f.canvas.currentPointY = y1;
      String method;
      method = "LinkedLineStart(" + x1 + "," + y1 + ");";
      f.cArea.append(method + "\n");
      splineX0 = x1;
      splineY0 = y1;
      linkedLineCnt++;
    }
    else if(canvasMouseMode == 30 && linkedLineCnt > 1 && (eve.getButton() == MouseEvent.BUTTON1))
    { //●折れ線の途中段階：折れ線の継続点をクリック
      x1 = (int)(x/f.canvas.affineScale +0.5); //端数四捨五入
      y1 = (int)(y/f.canvas.affineScale +0.5);
      f.canvas.currentPointX = x1;
      f.canvas.currentPointY = y1;
      String method;
      method = "LinkedLineNext(" + x1 + "," + y1 + ");";
      f.cArea.append(method + "\n");
      linkedLineCnt++;
    }
    else if(canvasMouseMode == 30 && linkedLineCnt > 2 && (eve.getButton() == MouseEvent.BUTTON3))
    { //●折れ線の最終段階：折れ線の終点をクリック
      x1 = (int)(x/f.canvas.affineScale +0.5); //端数四捨五入
      y1 = (int)(y/f.canvas.affineScale +0.5);
      f.canvas.currentPointX = x1;
      f.canvas.currentPointY = y1;
      String method = 
        "LinkedLineEnd(" + x1 + "," + y1 + ");";
      f.cArea.append(method + "\n");
      f.cArea.append("//\n");
      clear();
    }

    //●点描画の場合
    if( (canvasMouseFlag == true) && (canvasMouseMode == 101) &&
      (eve.getButton() == MouseEvent.BUTTON1) && 
      (brushSize >= 0) ) 
    {
      String methodP = "DrawCircle(2," + 
        (int)(x/(f.canvas.affineScale*f.canvas.freeScale) +0.5) + "," +
        (int)(y/(f.canvas.affineScale*f.canvas.freeScale) +0.5) + 
        "," + brushSize + 
        "," + f.palette.mixColor2.getRed() +
        "," + f.palette.mixColor2.getGreen() +
        "," + f.palette.mixColor2.getBlue() + ");" ;
      f.cArea.append(methodP + "\n");
      f.canvas.repaint();
    }

    if(canvasMouseFlag == true && canvasMouseMode == 102 && sprayCnt == 1 && 
      (eve.getButton() == MouseEvent.BUTTON1))
    { //●スプレーの第一段階：始点を左クリック
      x1 = (int)(x/f.canvas.affineScale +0.5); //端数四捨五入
      y1 = (int)(y/f.canvas.affineScale +0.5);
      f.canvas.currentPointX = x1;
      f.canvas.currentPointY = y1;
      String method;
      method = "SprayStart(" + x1 + "," + y1 + ", 1, 1.0, 30, 2000, 250, 150, 50, 255);";
      //座標値以外は、デフォルトの数値を設定するので、望みの数値に書き換えること。
      f.cArea.append(method + "\n");
      sprayCnt++;
      f.canvas.setInit();  //一般初期化
      f.canvas.repaint();  //描画実行
    }
    else if(canvasMouseFlag == true && canvasMouseMode == 102 && sprayCnt > 1 && 
      (eve.getButton() == MouseEvent.BUTTON1))
    { //●スプレーの中間段階：継続点を左クリック
      x1 = (int)(x/f.canvas.affineScale +0.5); //端数四捨五入
      y1 = (int)(y/f.canvas.affineScale +0.5);
      f.canvas.currentPointX = x1;
      f.canvas.currentPointY = y1;
      String method;
      method = "SprayNext(" + x1 + "," + y1 + ");";
      f.cArea.append(method + "\n");
      sprayCnt++;
      f.canvas.setInit();  //一般初期化
      f.canvas.repaint();  //描画実行
    }
    else if(canvasMouseFlag == true && canvasMouseMode == 102 && sprayCnt > 1 && 
      (eve.getButton() == MouseEvent.BUTTON3))
    { //●スプレーの最終段階：終点を右クリック
      x1 = (int)(x/f.canvas.affineScale +0.5); //端数四捨五入
      y1 = (int)(y/f.canvas.affineScale +0.5);
      f.canvas.currentPointX = x1;
      f.canvas.currentPointY = y1;
      String method;
      method = "SprayEnd(" + x1 + "," + y1 + ");";
      f.cArea.append(method + "\n");
      f.cArea.append("//\n");
      sprayCnt = 1;
    }
  }

  //■座標位置変数の初期化
  void clear()
  { 
    x1 = 0;
    y1 = 0;
    x2 = 0;
    y2 = 0;
    x3 = 0;
    y3 = 0;
    x4 = 0;
    y4 = 0;
    curveCnt = 1;
    splineCnt = 1;  
    linkedLineCnt= 1;
    startFlag = true;
    horizontFlag = true;
    commandX = "";
    erase("DrawLine2");
  }

  //■直線の消去
  //多用される直線については、専用の消去機能を作成。
  void eraseLine(int x1, int y1, int x2, int y2)
  { 
    String commandZ = f.cArea.getText();
    String newCommand = "";
    StringTokenizer st = new StringTokenizer(commandZ,"\n");
    //この時のStringTokenizerは、空行を削除する副作用あり。
    String command1; //命令1件（引数含む）
    for(int i=0; st.hasMoreTokens(); i++)
    { //●命令文を上から順に、全文調べる
      command1 = st.nextToken().trim();
      if(!command1.startsWith("DrawLine("))
      { //●該当しない命令の場合
        newCommand += command1 + "\n";
        continue;
      }
      else
      { //●該当する命令の場合
        StringTokenizer st2 = new StringTokenizer(command1,"(,)");
        st2.nextToken(); //●命令文における旧点位置の取得
        int ox1 = (int)Double.parseDouble((st2.nextToken().trim()));
        int oy1 = (int)Double.parseDouble((st2.nextToken().trim()));
        int ox2 = (int)Double.parseDouble((st2.nextToken().trim()));
        int oy2 = (int)Double.parseDouble((st2.nextToken().trim()));

        if( //●近傍点の判定（精密さのため、許容誤差は最小に）
          ((x1<=ox1+1) && (x1>=ox1-1) && 
          (y1<=oy1+1) && (y1>=oy1-1) &&
          (x2<=ox2+1) && (x2>=ox2-1) &&
          (y2<=oy2+1) && (y2>=oy2-1)) 
            || 
          ((x1<=ox2+1) && (x1>=ox2-1) &&
          (y1<=oy2+1) && (y1>=oy2-1) &&
          (x2<=ox1+1) && (x2>=ox1-1) &&
          (y2<=oy1+1) && (y2>=oy1-1)))
        {
          newCommand += ("");
        }
        else
        { //●該当する直線でないのなら
          newCommand += command1 + "\n";
        }
      }
    }
    f.cArea.setText(newCommand);
  }

  //■真円の消去
  //点描画で多用される真円については、専用の消去機能を作成。
  void eraseCircle(int x1, int y1)
  { 
    String commandZ = f.cArea.getText();
    String newCommand = "";
    StringTokenizer st = new StringTokenizer(commandZ,"\n");
    //この時のStringTokenizerは、空行を削除する副作用あり。
    String command1; //命令1件（引数含む）
    for(int i=0; st.hasMoreTokens(); i++)
    { //●命令文を上から順に、全文調べる
      command1 = st.nextToken().trim();
      if(!command1.startsWith("DrawCircle("))
      { //●該当しない命令の場合
        newCommand += command1 + "\n";
        continue;
      }
      else
      { //●該当する命令の場合
        //DrawCircle(int m, db x, db y, db rr, int r, int g, int b)
        StringTokenizer st2 = new StringTokenizer(command1,"(,)");
        st2.nextToken();
        st2.nextToken(); //モードは無視
        //●命令文における真円中心位置の取得
        int ox1 = (int)Double.parseDouble((st2.nextToken().trim()));
        int oy1 = (int)Double.parseDouble((st2.nextToken().trim()));
        f.message(""+ox1);
        f.message(""+oy1);

        if( //●近傍点の判定
          ((x1<=ox1+2) && (x1>=ox1-2) && 
          (y1<=oy1+2) && (y1>=oy1-2) ))
        {
          newCommand += ("");
        }
        else
        { //●該当する真円でないのなら
          newCommand += command1 + "\n";
        }
      }
    }
    f.cArea.setText(newCommand);
  }

  //■描画命令の座標点修正
  //  DrawLine(),DrawCurve(),
  //  DrawShapeStart(),DrawShapeCurve(),DrawShapeLine(),
  //  SprayStart(),SprayNext(),SprayEnd()
  //  これらの座標ポイントを書き替える。
  //  DrawEllipse(),DrawCircle(),DrawRect(),WriteStringH()系 は、始点を書き替える。
  //  Stamp()は、始点を書き替える。StampMulti()は、書き替えない。
  //  縮小表示では、マウス操作の位置指定で極度の正確さが必要となる。
  //  そのため、等倍表示以上で、位置修正の操作を推奨する。
  void replacePoint(int x1, int y1, int x2, int y2)
    //x1,y1 は、クリックされた移動前の古い座標点。
    //x2,y2 は、クリックされた移動後の新しい座標点。
  { 
    //最初のクリックで旧点を指定し、二回目で移動先の新点を指定。
    String commandZ = f.cArea.getText();
    String newCommand = "";
    StringTokenizer st = new StringTokenizer(commandZ,"\n");
    String command;
    int Q = 2; //●位置指定での許容誤差
    for(int i=0; st.hasMoreTokens(); i++)
    {
      command = st.nextToken().trim();
      if(command.startsWith("DrawLine("))
      { //●直線の位置修正
        StringTokenizer st2 = new StringTokenizer(command,"(,)");
        st2.nextToken(); //●命令文における旧点位置の取得
        int ox1 = (int)Double.parseDouble((st2.nextToken().trim()));
        int oy1 = (int)Double.parseDouble((st2.nextToken().trim()));
        int ox2 = (int)Double.parseDouble((st2.nextToken().trim()));
        int oy2 = (int)Double.parseDouble((st2.nextToken().trim()));
        if((x1<=ox1+Q) && (x1>=ox1-Q) &&   //近傍点の判定
           (y1<=oy1+Q) && (y1>=oy1-Q))
        { //●始点と判定されたら修正実行
          newCommand += ("DrawLine(" + 
            x2 + "," + y2 + "," + 
            ox2 + "," + oy2 + ");\n");
        }
        else if((x1<=ox2+Q) && (x1>=ox2-Q) && //近傍点の判定
            (y1<=oy2+Q) && (y1>=oy2-Q))
        { //●終点と判定されたら修正実行
          newCommand += ("DrawLine(" + 
            ox1 + "," + oy1 + "," + 
            x2 + "," + y2 + ");\n");
        }
        else
        { //●該当点が見付からないなら
          newCommand += command+"\n";
        }
      }
      else if(command.startsWith("DrawSplineStart("))
      { //●スプライン曲線始点の位置修正
        StringTokenizer st2 = new StringTokenizer(command,"(,)");
        st2.nextToken(); //●命令文における旧点位置の取得
        int ox1 = (int)Double.parseDouble((st2.nextToken().trim()));
        int oy1 = (int)Double.parseDouble((st2.nextToken().trim()));
        if((x1<=ox1+Q) && (x1>=ox1-Q) &&   //近傍点の判定
           (y1<=oy1+Q) && (y1>=oy1-Q))
        { //●修正実行
          newCommand += ("DrawSplineStart(" + 
            x2 + "," + y2 + ");\n");
        }
        else
        { //●該当点が見付からないなら
          newCommand += command+"\n";
        }
      }
      else if(command.startsWith("DrawSplineControl("))
      { //●スプライン曲線制御点の位置修正
        StringTokenizer st2 = new StringTokenizer(command,"(,)");
        st2.nextToken(); //●命令文における旧点位置の取得
        int ox1 = (int)Double.parseDouble((st2.nextToken().trim()));
        int oy1 = (int)Double.parseDouble((st2.nextToken().trim()));
        if((x1<=ox1+Q) && (x1>=ox1-Q) &&   //近傍点の判定
           (y1<=oy1+Q) && (y1>=oy1-Q))
        { //●修正実行
          newCommand += ("DrawSplineControl(" + 
            x2 + "," + y2 + ");\n");
        }
        else
        { //●該当点が見付からないなら
          newCommand += command+"\n";
        }
      }
      else if(command.startsWith("DrawSplineEnd("))
      { //●スプライン曲線終点の位置修正
        StringTokenizer st2 = new StringTokenizer(command,"(,)");
        st2.nextToken(); //●命令文における旧点位置の取得
        int ox1 = (int)Double.parseDouble((st2.nextToken().trim()));
        int oy1 = (int)Double.parseDouble((st2.nextToken().trim()));
        if((x1<=ox1+Q) && (x1>=ox1-Q) &&   //近傍点の判定
           (y1<=oy1+Q) && (y1>=oy1-Q))
        { //●修正実行
          newCommand += ("DrawSplineEnd(" + 
            x2 + "," + y2 + ");\n");
        }
        else
        { //●該当点が見付からないなら
          newCommand += command+"\n";
        }
      }

      else if(command.startsWith("LinkedLineStart("))
      { //●折れ線始点の位置修正
        StringTokenizer st2 = new StringTokenizer(command,"(,)");
        st2.nextToken(); //●命令文における旧点位置の取得
        int ox1 = (int)Double.parseDouble((st2.nextToken().trim()));
        int oy1 = (int)Double.parseDouble((st2.nextToken().trim()));
        if((x1<=ox1+Q) && (x1>=ox1-Q) &&   //近傍点の判定
           (y1<=oy1+Q) && (y1>=oy1-Q))
        { //●修正実行
          newCommand += ("LinkedLineStart(" + 
            x2 + "," + y2 + ");\n");
        }
        else
        { //●該当点が見付からないなら
          newCommand += command+"\n";
        }
      }
      else if(command.startsWith("LinkedLineNext("))
      { //●折れ線継続点の位置修正
        StringTokenizer st2 = new StringTokenizer(command,"(,)");
        st2.nextToken(); //●命令文における旧点位置の取得
        int ox1 = (int)Double.parseDouble((st2.nextToken().trim()));
        int oy1 = (int)Double.parseDouble((st2.nextToken().trim()));
        if((x1<=ox1+Q) && (x1>=ox1-Q) &&   //近傍点の判定
           (y1<=oy1+Q) && (y1>=oy1-Q))
        { //●修正実行
          newCommand += ("LinkedLineNext(" + 
            x2 + "," + y2 + ");\n");
        }
        else
        { //●該当点が見付からないなら
          newCommand += command+"\n";
        }
      }
      else if(command.startsWith("LinkedLineEnd("))
      { //●折れ線終点の位置修正
        StringTokenizer st2 = new StringTokenizer(command,"(,)");
        st2.nextToken(); //●命令文における旧点位置の取得
        int ox1 = (int)Double.parseDouble((st2.nextToken().trim()));
        int oy1 = (int)Double.parseDouble((st2.nextToken().trim()));
        if((x1<=ox1+Q) && (x1>=ox1-Q) &&   //近傍点の判定
           (y1<=oy1+Q) && (y1>=oy1-Q))
        { //●修正実行
          newCommand += ("LinkedLineEnd(" + 
            x2 + "," + y2 + ");\n");
        }
        else
        { //●該当点が見付からないなら
          newCommand += command+"\n";
        }
      }

      else if(command.startsWith("DrawCurve("))
      { //●単独ベジェ曲線の位置修正
        StringTokenizer st2 = new StringTokenizer(command,"(,)");
        st2.nextToken(); //●命令文における旧点位置の取得
        int ox1 = (int)Double.parseDouble((st2.nextToken().trim()));
        int oy1 = (int)Double.parseDouble((st2.nextToken().trim()));
        int ox2 = (int)Double.parseDouble((st2.nextToken().trim()));
        int oy2 = (int)Double.parseDouble((st2.nextToken().trim()));
        int ox3 = (int)Double.parseDouble((st2.nextToken().trim()));
        int oy3 = (int)Double.parseDouble((st2.nextToken().trim()));
        int ox4 = (int)Double.parseDouble((st2.nextToken().trim()));
        int oy4 = (int)Double.parseDouble((st2.nextToken().trim()));
        if((x1<=ox1+Q) && (x1>=ox1-Q) &&   //近傍点の判定
           (y1<=oy1+Q) && (y1>=oy1-Q))
        { //●始点と判定されたら修正実行
          newCommand += ("DrawCurve(" + 
            x2 + "," + y2 + "," + 
            ox2 + "," + oy2 + "," + 
            ox3 + "," + oy3 + "," + 
            ox4 + "," + oy4 + ");\n");
        }
        else if((x1<=ox2+Q) && (x1>=ox2-Q) &&   //近傍点の判定
            (y1<=oy2+Q) && (y1>=oy2-Q))
        { //●第一制御点と判定されたら修正実行
          newCommand += ("DrawCurve(" + 
            ox1 + "," + oy1 + "," + 
            x2 + "," + y2 + "," + 
            ox3 + "," + oy3 + "," + 
            ox4 + "," + oy4 + ");\n");
        }
        else if((x1<=ox3+Q) && (x1>=ox3-Q) && //近傍点の判定
            (y1<=oy3+Q) && (y1>=oy3-Q))
        { //●第二制御点と判定されたら修正実行
          newCommand += ("DrawCurve(" + 
            ox1 + "," + oy1 + "," + 
            ox2 + "," + oy2 + "," + 
            x2 + "," + y2 + "," + 
            ox4 + "," + oy4 + ");\n");
        }
        else if((x1<=ox4+Q) && (x1>=ox4-Q) && //近傍点の判定
            (y1<=oy4+Q) && (y1>=oy4-Q))
        { //●終点と判定されたら修正実行
          newCommand += ("DrawCurve(" + 
            ox1 + "," + oy1 + "," + 
            ox2 + "," + oy2 + "," + 
            ox3 + "," + oy3 + "," + 
            x2 + "," + y2 + ");\n");
        }
        else
        { //●該当点が見付からないなら
          newCommand += command+"\n";
        }
      }
      else if(command.startsWith("DrawShapeStart("))
      { //●自由図形の始点の位置修正
        StringTokenizer st2 = new StringTokenizer(command,"(,)");
        st2.nextToken(); //●命令文における旧点位置の取得
        int m   = (int)Double.parseDouble((st2.nextToken().trim()));
        int ox1 = (int)Double.parseDouble((st2.nextToken().trim()));
        int oy1 = (int)Double.parseDouble((st2.nextToken().trim()));
        int r   = (int)Double.parseDouble((st2.nextToken().trim()));
        int g   = (int)Double.parseDouble((st2.nextToken().trim()));
        int b   = (int)Double.parseDouble((st2.nextToken().trim()));
        if((x1<=ox1+Q) && (x1>=ox1-Q) &&   //近傍点の判定
           (y1<=oy1+Q) && (y1>=oy1-Q))
        { //●該当点と判定されたら修正実行
          newCommand += ("DrawShapeStart(" + 
            m + "," + 
            x2 + "," + y2 + "," + 
            r + "," + 
            g + "," + 
            b + ");\n");
        }
        else
        { //●該当点が見付からないなら
          newCommand += command+"\n";
        }
      }
      else if(command.startsWith("DrawShapeLine(") || 
        command.startsWith("DSL("))
      { //●自由図形での直線部分の位置修正
        StringTokenizer st2 = new StringTokenizer(command,"(,)");
        st2.nextToken(); //●命令文における旧点位置の取得
        int ox1 = (int)Double.parseDouble((st2.nextToken().trim()));
        int oy1 = (int)Double.parseDouble((st2.nextToken().trim()));
        if((x1<=ox1+Q) && (x1>=ox1-Q) &&   //近傍点の判定
           (y1<=oy1+Q) && (y1>=oy1-Q))
        { //●該当点と判定されたら修正実行
          newCommand += ("DrawShapeLine(" + 
            x2 + "," + y2 + ");\n");
        }
        else
        { //●該当点が見付からないなら
          newCommand += command+"\n";
        }
      }
      else if(command.startsWith("DrawShapeCurve("))
      { //●自由図形での曲線（ベジェ曲線）部分の位置修正
        StringTokenizer st2 = new StringTokenizer(command,"(,)");
        st2.nextToken(); //●命令文における旧点位置の取得
        int ox1 = (int)Double.parseDouble((st2.nextToken().trim()));
        int oy1 = (int)Double.parseDouble((st2.nextToken().trim()));
        int ox2 = (int)Double.parseDouble((st2.nextToken().trim()));
        int oy2 = (int)Double.parseDouble((st2.nextToken().trim()));
        int ox3 = (int)Double.parseDouble((st2.nextToken().trim()));
        int oy3 = (int)Double.parseDouble((st2.nextToken().trim()));
        if((x1<=ox1+Q) && (x1>=ox1-Q) &&   //近傍点の判定
           (y1<=oy1+Q) && (y1>=oy1-Q))
        { //●第一制御点と判定されたら修正実行
          newCommand += ("DrawShapeCurve(" + 
            x2 + "," + y2 + "," + 
            ox2 + "," + oy2 + "," + 
            ox3 + "," + oy3 + ");\n");
        }
        else if((x1<=ox2+Q) && (x1>=ox2-Q) && //近傍点の判定
            (y1<=oy2+Q) && (y1>=oy2-Q))
        { //●第二制御点と判定されたら修正実行
          newCommand += ("DrawShapeCurve(" + 
            ox1 + "," + oy1 + "," + 
            x2 + "," + y2 + "," + 
            ox3 + "," + oy3 + ");\n");
        }
        else if((x1<=ox3+Q) && (x1>=ox3-Q) && //近傍点の判定
            (y1<=oy3+Q) && (y1>=oy3-Q))
        { //●終点と判定されたら修正実行
          newCommand += ("DrawShapeCurve(" + 
            ox1 + "," + oy1 + "," + 
            ox2 + "," + oy2 + "," + 
            x2 + "," + y2 + ");\n");
        }
        else
        { //●該当点が見付からないなら
          newCommand += command+"\n";
        }
      }
      else if(command.startsWith("DrawCircle("))
      { //●真円の位置修正
        StringTokenizer st2 = new StringTokenizer(command,"(,)");
        st2.nextToken(); //●命令文における旧点位置の取得
        String em = st2.nextToken().trim(); //
        int ex = (int)Double.parseDouble((st2.nextToken().trim())); //
        int ey = (int)Double.parseDouble((st2.nextToken().trim())); //
        String str1 = st2.nextToken().trim(); //半径
        String str2 = st2.nextToken().trim(); //赤色
        String str3 = st2.nextToken().trim(); //緑色
        String str4 = st2.nextToken().trim(); //青色
        if((x1<=ex+Q) && (x1>=ex-Q) &&   //近傍点の判定
           (y1<=ey+Q) && (y1>=ey-Q))
        { //●該当点と判定されたら修正実行
          newCommand += ("DrawCircle(" + em +"," +
            x2 + "," + y2 + "," + 
            str1 +"," + str2 +"," + str3 +"," + str4  +");\n" );
        }
        else
        { //●該当点が見付からないなら
          newCommand += command+"\n"; 
        }
      }
      else if(command.startsWith("DrawEllipse("))
      { //●楕円の位置修正
        StringTokenizer st2 = new StringTokenizer(command,"(,)");
        st2.nextToken(); //●命令文における旧点位置の取得
        String em = st2.nextToken().trim(); //
        int ex = (int)Double.parseDouble((st2.nextToken().trim())); //
        int ey = (int)Double.parseDouble((st2.nextToken().trim())); //
        String str1 = st2.nextToken().trim(); //横半径
        String str2 = st2.nextToken().trim(); //縦半径
        String str3 = st2.nextToken().trim(); //赤色
        String str4 = st2.nextToken().trim(); //緑色
        String str5 = st2.nextToken().trim(); //青色
        if((x1<=ex+Q) && (x1>=ex-Q) &&   //近傍点の判定
           (y1<=ey+Q) && (y1>=ey-Q))
        { //●該当点と判定されたら修正実行
          newCommand += ("DrawEllipse(" + em +"," +
            x2 + "," + y2 + "," + 
            str1 +"," + str2 +"," + str3 +"," + str4 +"," + str5 +");\n" );
        }
        else
        { //●該当点が見付からないなら
          newCommand += command+"\n"; 
        }
      }
      else if(command.startsWith("DrawRect("))
      { //●四角形の位置修正
        StringTokenizer st2 = new StringTokenizer(command,"(,)");
        st2.nextToken(); //●命令文における旧点位置の取得
        String rm = st2.nextToken().trim(); //モード
        int rx = (int)Double.parseDouble((st2.nextToken().trim())); //x
        int ry = (int)Double.parseDouble((st2.nextToken().trim())); //y
        String str1 = st2.nextToken().trim(); //幅
        String str2 = st2.nextToken().trim(); //丈
        String str3 = st2.nextToken().trim(); //赤色
        String str4 = st2.nextToken().trim(); //緑色
        String str5 = st2.nextToken().trim(); //青色
        if((x1<=rx+Q) && (x1>=rx-Q) &&   //近傍点の判定
           (y1<=ry+Q) && (y1>=ry-Q))
        { //●該当点と判定されたら修正実行
          newCommand += ("DrawRect(" + rm +"," +
            x2 + "," + y2 + "," + 
            str1 +"," + str2 +"," + str3 +"," + str4 +"," +str5 +");\n" );
        }
        else
        { //●該当点が見付からないなら
          newCommand += command+"\n"; 
        }
      }

      else if(command.startsWith("SprayStart("))
      { //●スプレー起点の位置修正
        StringTokenizer st2 = new StringTokenizer(command,"(,)");
        st2.nextToken(); //●命令文における旧点位置の取得
        int rx = (int)Double.parseDouble((st2.nextToken().trim())); //x
        int ry = (int)Double.parseDouble((st2.nextToken().trim())); //y
        String str1 = st2.nextToken().trim(); //分布タイプ
        String str2 = st2.nextToken().trim(); //飛沫大きさ
        String str3 = st2.nextToken().trim(); //飛沫円盤半径
        String str4 = st2.nextToken().trim(); //飛沫個数
        String str5 = st2.nextToken().trim(); //赤色
        String str6 = st2.nextToken().trim(); //緑色
        String str7 = st2.nextToken().trim(); //青色
        String str8 = st2.nextToken().trim(); //透明度
        if((x1<=rx+Q) && (x1>=rx-Q) &&   //近傍点の判定
           (y1<=ry+Q) && (y1>=ry-Q))
        { //●該当点と判定されたら修正実行
          newCommand += ("SprayStart(" +
            x2 + "," + y2 + "," + 
            str1 +"," + str2 +"," + str3 + "," + str4 + "," + str5 + "," + 
            str6 + "," + str7 + "," + str8 + ");\n" );
        }
        else
        { //●該当点が見付からないなら
          newCommand += command+"\n"; 
        }
      }
      else if(command.startsWith("SprayNext("))
      { //●スプレー継続点の位置修正
        StringTokenizer st2 = new StringTokenizer(command,"(,)");
        st2.nextToken(); //●命令文における旧点位置の取得
        int rx = (int)Double.parseDouble((st2.nextToken().trim())); //x
        int ry = (int)Double.parseDouble((st2.nextToken().trim())); //y
        if((x1<=rx+Q) && (x1>=rx-Q) &&   //近傍点の判定
           (y1<=ry+Q) && (y1>=ry-Q))
        { //●該当点と判定されたら修正実行
          newCommand += ("SprayNext(" +
            x2 + "," + y2  + ");\n" );
        }
        else
        { //●該当点が見付からないなら
          newCommand += command+"\n"; 
        }
      }
      else if(command.startsWith("SprayEnd("))
      { //●スプレー終点の位置修正
        StringTokenizer st2 = new StringTokenizer(command,"(,)");
        st2.nextToken(); //●命令文における旧点位置の取得
        int rx = (int)Double.parseDouble((st2.nextToken().trim())); //x
        int ry = (int)Double.parseDouble((st2.nextToken().trim())); //y
        if((x1<=rx+Q) && (x1>=rx-Q) &&   //近傍点の判定
           (y1<=ry+Q) && (y1>=ry-Q))
        { //●該当点と判定されたら修正実行
          newCommand += ("SprayEnd(" +
            x2 + "," + y2  + ");\n" );
        }
        else
        { //●該当点が見付からないなら
          newCommand += command+"\n"; 
        }
      }

      else if(command.startsWith("Stamp("))
      { //●前景画像の位置修正
        StringTokenizer st2 = new StringTokenizer(command,"(,)");
        st2.nextToken(); //●命令文における旧点位置の取得
        int rx = Integer.parseInt((st2.nextToken().trim())); //x
        int ry = Integer.parseInt((st2.nextToken().trim())); //y
        String str1 = st2.nextToken().trim(); //幅
        String str2 = st2.nextToken().trim(); //丈
        String str3 = Util.trimAffix(st2.nextToken(),'\"','\"'); //ファイル名
        if((x1<=rx+Q) && (x1>=rx-Q) &&   //近傍点の判定
           (y1<=ry+Q) && (y1>=ry-Q))
        { //●該当点と判定されたら修正実行
          newCommand += ("Stamp(" +  x2 + "," + y2 + "," + 
            str1 +"," + str2 +"," + str3+ ");\n" );
        }
        else
        { //●該当点が見付からないなら
          newCommand += command+"\n"; 
        }
      }
      else if(command.startsWith("WriteStringH(") || 
        command.startsWith("WriteString(") || 
        command.startsWith("WriteStringB(") || 
        command.startsWith("WriteStringV("))
      { //●文字列（並字、極太字、縦書き）の位置修正
        StringTokenizer st2 = new StringTokenizer(command,"(,)");
        st2.nextToken(); //●命令文における旧点位置の取得
        int ox1 = (int)Double.parseDouble((st2.nextToken().trim()));
        int oy1 = (int)Double.parseDouble((st2.nextToken().trim()));
        String str1 = st2.nextToken().trim();
        String str2 = st2.nextToken().trim();
        String str3 = st2.nextToken().trim();
        if((x1<=ox1+Q) && (x1>=ox1-Q) &&   //近傍点の判定
           (y1<=oy1+Q) && (y1>=oy1-Q))
        { //●該当点と判定されたら修正実行
          if(command.startsWith("WriteStringH("))
            newCommand += ("WriteStringH(" + 
            x2 + "," + y2 + "," + 
            str1 + "," + str2 + "," + str3 + ");\n" );
          else if(command.startsWith("WriteString("))
            newCommand += ("WriteString(" + 
            x2 + "," + y2 + "," + 
            str1 + "," + str2 + "," + str3 + ");\n" );
          else if(command.startsWith("WriteStringB("))
            newCommand += ("WriteStringB(" + 
            x2 + "," + y2 + "," + 
            str1 + "," + str2 + "," + str3 + ");\n" );
          else if(command.startsWith("WriteStringV("))
            newCommand += ("WriteStringV(" + 
            x2 + "," + y2 + "," + 
            str1 + "," + str2 + "," + str3 + ");\n" );
        }
        else
        { //●該当点が見付からないなら
          newCommand += command+"\n";
        }
      }
      else
      { //●いずれにも該当しない命令の場合
        newCommand += command+"\n";
        continue;
      }
    }
    f.cArea.setText(newCommand);
  }

  //■指定のコマンドを全部消去する。
  void erase(String commandE)
  {  //一時使用するだけの「DrawLine2」コマンドにのみ利用
    String commandZ = f.cArea.getText(); //コマンド全文取得
    String newCommandZ = ""; //新しいコマンド全文
    StringTokenizer st = new StringTokenizer(commandZ,"\n");
    String command1;  //検査対象の描画コマンド1件
    for(int i=0; st.hasMoreTokens(); i++)
    {
      command1 = st.nextToken().trim();
      if(!(command1.startsWith(commandE)) )
      {  //不一致
        newCommandZ += command1+"\n";
        continue;
      }
      else
      {  //一致
        ; //拾わずに捨てる（消去）
      }
    }
    f.cArea.setText(newCommandZ);  //編集済みコマンド全文の格納
    f.canvas.repaint();
  }
  
  //■描画命令文検索
  //  クリックされた位置を始点とする線描画命令を探し出し、
  //  命令文の末尾に、コメント（//★★★）を付けて、表示する。
  void refer(int x, int y)
  { 
    String commandZ = f.cArea.getText();
    String newCommand = "";
    StringTokenizer st = new StringTokenizer(commandZ,"\n"); //行毎に分解
    String command;
    for(int i=0; st.hasMoreTokens(); i++)
    {
      command = st.nextToken().trim(); //命令１行を取得
      StringTokenizer st2 = new StringTokenizer(command,"(),;"); //命令１行を分解
      st2.nextToken().trim(); //コマンド名の部分を捨てる
      if(command.endsWith("//★★★"))
      { //●命令文の末尾に、既に「//★★★」があれば、削除する
        command = command.substring(0,command.length()-5);
      }

      if(command.startsWith("DrawLine"))
      { //●直線の始点または終点
        String x1, y1, x2, y2;
        x1 = st2.nextToken().trim();
        y1 = st2.nextToken().trim();
        x2 = st2.nextToken().trim();
        y2 = st2.nextToken().trim();
        if(nearEquals(x, x1) && nearEquals(y, y1))
          newCommand += command+"//★★★\n";
        else if(nearEquals(x, x2) && nearEquals(y, y2))
          newCommand += command+"//★★★\n";
        else
          newCommand += command+"\n";
      }
      else if(command.startsWith("DrawCurve"))
      { //●単独ベジェ曲線
        String x1, y1, x2, y2, x3, y3, x4, y4;
        x1 = st2.nextToken().trim();
        y1 = st2.nextToken().trim();
        x2 = st2.nextToken().trim();
        y2 = st2.nextToken().trim();
        x3 = st2.nextToken().trim();
        y3 = st2.nextToken().trim();
        x4 = st2.nextToken().trim();
        y4 = st2.nextToken().trim();
        if(nearEquals(x, x1) && nearEquals(y, y1))
          newCommand += command+"//★★★\n";
        else if(nearEquals(x, x2) && nearEquals(y, y2))
          newCommand += command+"//★★★\n";
        else if(nearEquals(x, x3) && nearEquals(y, y3))
          newCommand += command+"//★★★\n";
        else if(nearEquals(x, x4) && nearEquals(y, y4))
          newCommand += command+"//★★★\n";
        else
          newCommand += command+"\n";
      }
      else if(
        command.startsWith("DrawSplineStart") ||
        command.startsWith("DrawSplineControl") ||
        command.startsWith("DrawSplineEnd"))
      { //●スプライン曲線
        if(nearEquals(x, st2.nextToken().trim()) &&
        nearEquals(y, st2.nextToken().trim()) )
          newCommand += command+"//★★★\n";
        else
          newCommand += command+"\n";
      }

      else if(
        command.startsWith("LinkedLineStart") ||
        command.startsWith("LinkedLineNext") ||
        command.startsWith("LinkedLineEnd"))
      { //●折れ線
        if(nearEquals(x, st2.nextToken().trim()) &&
        nearEquals(y, st2.nextToken().trim()) )
          newCommand += command+"//★★★\n";
        else
          newCommand += command+"\n";
      }

      else if(command.startsWith("DrawShapeStart"))
      { //●自由図形の始点
        st2.nextToken().trim(); //描画モードは不用
        if(nearEquals(x, st2.nextToken().trim()) &&
        nearEquals(y, st2.nextToken().trim()) )
          newCommand += command+"//★★★\n";
        else
          newCommand += command+"\n";
      }
      else if(command.startsWith("DrawShapeLine"))
      { //●自由図形の直線部終点
        if(nearEquals(x, st2.nextToken().trim()) &&
        nearEquals(y, st2.nextToken().trim()) )
          newCommand += command+"//★★★\n";
        else
          newCommand += command+"\n";
      }
      else if(command.startsWith("DrawShapeCurve"))
      { //●自由図形の曲線部
        String x1, y1, x2, y2, x3, y3;
        x1 = st2.nextToken().trim();
        y1 = st2.nextToken().trim();
        x2 = st2.nextToken().trim();
        y2 = st2.nextToken().trim();
        x3 = st2.nextToken().trim();
        y3 = st2.nextToken().trim();
        if((nearEquals(x, x1) && nearEquals(y, y1)) ||
          (nearEquals(x, x2) && nearEquals(y, y2)) ||
          (nearEquals(x, x3) && nearEquals(y, y3)) )
        {
          newCommand += command+"//★★★\n";
        }
        else
          newCommand += command+"\n";
      }

      else if(command.startsWith("DrawEllipse"))
      { //●楕円の始点（中心点）
        st2.nextToken().trim(); //描画モードは不用
        if(nearEquals(x, st2.nextToken().trim()) &&
        nearEquals(y, st2.nextToken().trim()) )
          newCommand += command+"//★★★\n";
        else
          newCommand += command+"\n";
      }
      else if(command.startsWith("DrawCircle"))
      { //●真円の始点（中心点）
        st2.nextToken().trim(); //描画モードは不用
        if(nearEquals(x, st2.nextToken().trim()) &&
        nearEquals(y, st2.nextToken().trim()) )
          newCommand += command+"//★★★\n";
        else
          newCommand += command+"\n";
      }
      else if(command.startsWith("DrawRect"))
      { //●四角形の始点（左上点）
        st2.nextToken().trim(); //描画モードは不用
        if(nearEquals(x, st2.nextToken().trim()) &&
        nearEquals(y, st2.nextToken().trim()) )
          newCommand += command+"//★★★\n";
        else
          newCommand += command+"\n";
      }

      else if(command.startsWith("WriteStringH") || command.startsWith("WriteString"))
      { //●文字列の始点（左下点）
        if(nearEquals(x, st2.nextToken().trim()) &&
        nearEquals(y, st2.nextToken().trim()) )
          newCommand += command+"//★★★\n";
        else
          newCommand += command+"\n";
      }

      else if(
        command.startsWith("SprayStart") ||
        command.startsWith("SprayNext") ||
        command.startsWith("SprayEnd"))
      { //●スプレー
        if(nearEquals(x, st2.nextToken().trim()) &&
        nearEquals(y, st2.nextToken().trim()) )
          newCommand += command+"//★★★\n";
        else
          newCommand += command+"\n";
      }

      else
      { 
        newCommand += command+"\n";
      }
    }
    f.cArea.setText(newCommand);
    f.canvas.repaint();
  }

  int colorRanNum()
  { //●ランダムなRGBのどれか一つを作る
    Random ran = new Random();
    return (10 + ran.nextInt(240));
  }

  boolean nearEquals(int a, String s)
  { //●近似値判定、（sは整数または実数で数値表現された文字列）
    double d = Double.parseDouble(s);
    if((a+2 >= d) && (a-2 <= d))
      return true;
    else
      return false;
  }
  
  boolean nearEquals(double a, double b)
  { //●近似値判定
    if((a+2 >= b) && (a-2 <= b))
      return true;
    else
      return false;
  }

  //■図形彩色変更
  //  クリックされた位置を始点とする図形描画命令を探し出し、
  //  パレットで混色されている色を使って、
  //  命令の彩色指定部分を書き変える。
  void changeColor(int x, int y, int r, int g, int b)
  { 
    String commandZ = f.cArea.getText();
    String newCommand = "";
    StringTokenizer st = new StringTokenizer(commandZ,"\n");
    String command = "";
    for(int i=0; st.hasMoreTokens(); i++)
    {
      command = st.nextToken().trim();
      String[] strArray = Util.csv2Array(command,0);

      if( //●自由図形の彩色変更
        (strArray[0].startsWith("DrawShapeStart")))
      {
        int x0 = (int)Double.parseDouble(strArray[1]);
        int y0 = (int)Double.parseDouble(strArray[2]);
        if(nearEquals(x0,x) && nearEquals(y0,y))
        {
          newCommand += strArray[0] + "," + 
            strArray[1] + "," + strArray[2] + "," + 
            r  + "," + g  + "," + b  + ");\n" ;
        }
        else
        { 
          newCommand += command+"\n" ;
        }
      }
      else if( //●真円の彩色変更
        (strArray[0].startsWith("DrawCircle")))
      {
        int x0 = (int)Double.parseDouble(strArray[1]);
        int y0 = (int)Double.parseDouble(strArray[2]);
        if(nearEquals(x0,x) && nearEquals(y0,y))
        {
          newCommand += strArray[0] + "," + 
            strArray[1] + "," + strArray[2] + "," + 
            strArray[3] +  "," + 
            r  + "," + g  + "," + b  + ");\n" ;
        }
        else
        { 
          newCommand += command+"\n" ;
        }
      }
      else if( //●楕円の彩色変更
        (strArray[0].startsWith("DrawEllipse")))
      {
        int x0 = (int)Double.parseDouble(strArray[1]);
        int y0 = (int)Double.parseDouble(strArray[2]);
        if(nearEquals(x0,x) && nearEquals(y0,y))
        {
          newCommand += strArray[0] + "," + 
            strArray[1] + "," + strArray[2] + "," + 
            strArray[3] + "," + strArray[4] + "," + 
            r  + "," + g  + "," + b  + ");\n" ;
        }
        else
        { 
          newCommand += command+"\n" ;
        }
      }
      else if( //●四角形の彩色変更
        (strArray[0].startsWith("DrawRect")))
      {
        int x0 = (int)Double.parseDouble(strArray[1]);
        int y0 = (int)Double.parseDouble(strArray[2]);
        if(nearEquals(x0,x) && nearEquals(y0,y))
        {
          newCommand += strArray[0] + "," + 
            strArray[1] + "," + strArray[2] + "," + 
            strArray[3] + "," + strArray[4] + "," + 
            r  + "," + g  + "," + b  + ");\n" ;
        }
        else
        { 
          newCommand += command+"\n" ;
        }
      }
      else
      { 
        newCommand += command+"\n" ;
      }
    }
    f.cArea.setText(newCommand);
    f.canvas.repaint();
  }

  //以下は、MouseListenerの実装で、無反応にする部分
  
  public void mouseEntered(MouseEvent e)
  {}
  public void mouseExited(MouseEvent e)
  {}
  public void mouseClicked(MouseEvent e)
  {}
  public void mouseReleased(MouseEvent e)
  {}
} //EOF (GCreatorCanMouseLis.java)
