//■■GCreatorCanvasEngine.java
//Ver.3.4

//■■インポート
import java.awt.*;  //必須
import java.awt.geom.*;  //必須
import java.awt.image.*;  //必須
import java.util.List;  //必須（曖昧回避）
import java.util.*;  //必須
import javax.swing.*;  //必須
import java.io.*;  //必須
import javax.imageio.*;  //必須
import javax.imageio.stream.*;  //必須

//■■■描画機能の実現
class GCreatorCanvasEngine extends JLabel
{
  //GCreatorCanvasEngineが、JLabelを継承するのは、
  //JLabelがカラーのベクター方式の文字画像を描くことができ、
  //JLabel自体が、文字に限らず多様なベクター画像処理能力があるからである。
  //さらに、ラスター画像も表示できるからである。
  //ただし、JLabelよりも上位のJComponentに変更しても、正常に作動する。

  //■■変数宣言
  //■メンバー変数
  GCreator f; //JFlameの継承インスタンス
  Graphics2D g2;

  //●命令文関連の変数
  double imagePX_W = 0;  //指定画像サイズ幅（ピクセル単位）
  double imagePX_H = 0;  //指定画像サイズ丈（ピクセル単位）
  double printMM_W = 0;  //指定画像サイズ幅（ミリ単位）
  double printMM_H = 0;  //指定画像サイズ丈（ミリ単位）

  double affineScale = 1.0; //表示倍率
  double freeScale   = 1.0; //追加倍率

  int  shiftMode  = 0; //シフト区分（画像のシフト）
  double shiftX   = 0; //シフト量X
  double shiftY   = 0; //シフト量Y
  
  double affineDegree       = 0.0;   //90度回転角度
  double affineDegreeFree   = 0.0;   //指定角度回転角度

  boolean rotateFlag        = false; //90度回転モード
  boolean rotateFreeFlag    = false; //指定角度回転モード
  int rotateFreeW           = 0;     //回転に伴って必要なx軸原点移動
  int rotateFreeH           = 0;     //回転に伴って必要なy軸原点移動

  //RoteteMulti命令（画像多段階回転重複描画）
  boolean rotateMultiFlag        = false;
  int rotateMultiStartDegree     = 0;    //回転開始角度（ディグリー）
  int rotateMultiDegree          = 0;    //回転角度（ディグリー）
  int rotateMultiTimes           = 0;    //回転回数
  String rotateMultiF            = null; //回転対象画像のファイル名（パス付き）
  BufferedImage rotateMultiImage = null; //完成画像の格納領域

  boolean markFlag          = true; //描画起点や曲線制御点などの目印線
  boolean commandInspection = false; //命令の記述エラー検査
  boolean canSaveMode       = false; //画像保存モード

  long randomSeed = //模様作成乱数初期値
    new GregorianCalendar().get(Calendar.MILLISECOND);
  Random ran = new Random(randomSeed);
  long randomSeed2 =  //間欠描画およびスプレーに用いる乱数初期値
    new GregorianCalendar().get(Calendar.MILLISECOND);
  Random ran2St = null;
  Random ran2Ex = null;
  Random ranSpray = new Random(new GregorianCalendar().get(Calendar.MILLISECOND));
  
  //SaveCommands命令
  String saveCmdFName = ""; //命令文書の保存ファイル名（パス付き）
  //SaveImage命令
  String saveImgFName = ""; //作成画像の保存ファイル名（パス付き）

  //CutAndSave命令
  int cutAndSaveImgX1 = 0; //作成画像の切り出し領域の左上ｘ座標
  int cutAndSaveImgY1 = 0; //作成画像の切り出し領域の左上ｙ座標
  int cutAndSaveImgX2 = 0; //作成画像の切り出し領域の右下ｘ座標
  int cutAndSaveImgY2 = 0; //作成画像の切り出し領域の右下ｙ座標
  String cutAndSaveImgFName = ""; //作成画像の切り取り保存ファイル名称（パス付き）

  //ChangeColor命令（単独色相変更）
  String colorChangeFile  = ""; //色変換処理対象画像のファイル名
  boolean colorChangeFlag = false; //色変換要求の有無
  int acceptableLimitC = 0;    //微妙な色の変動許容量
  int colorChangeMode  = 0;    //色相変更モード
  int colorChange1R    = 0;    //色変換の旧色
  int colorChange1G    = 0;
  int colorChange1B    = 0;
  int colorChange2R    = 0;    //色変換の新色
  int colorChange2G    = 0;
  int colorChange2B    = 0;
  BufferedImage colorChangeImage0 = null;
  BufferedImage colorChangeImage1 = null;

  //ReverseImage命令
  boolean reverseImageFlag    = false; //画像反転要求の有無
  String reverseImageFile     = "";    //画像反転の加工前対象画像ファイル名
  int reverseImageMode        = 0;     //反転方式の指定
  BufferedImage reverseImage0 = null;  //加工前画像
  BufferedImage reverseImage1 = null;  //加工後画像
  BufferedImage reverseImageX = null;  //90度または270度回転加工後画像

  //ColorGradient命令
  String gradientFName = ""; //線形階調加工の対象ファイル名
  int gradientY1 = 0;        //加工開始y軸位置
  int gradientY2 = 0;        //加工終了y軸位置
  double gradientR = 0.0;
  double gradientG = 0.0;
  double gradientB = 0.0;
  BufferedImage gradientImage0 = null; 
  BufferedImage gradientImage1 = null;
  
  //MixImage命令
  String mixFName0        = ""; //二重像画像処理の対象ファイル名
  String mixFName1        = ""; //二重像画像処理の対象ファイル名
  int mixMode             = 1;  //通常のモード
  BufferedImage mixImage0 = null;
  BufferedImage mixImage1 = null;
  BufferedImage mixImage2 = null;

  //AdjustRGB命令
  String adjustFName = ""; //色調調整の対象ファイル名
  int adjustR = 0;
  int adjustG = 0;
  int adjustB = 0;
  BufferedImage adjustImage0 = null; 
  BufferedImage adjustImage1 = null;

  //SoftFocus命令
  String softFocusFName = "";  //ソフト・フォーカス効果の対象画像ファイル名
  int softFocusDegree   = 1;   //ソフト・フォーカス効果の度合い（０～１０）
  BufferedImage softFocusImage0 = null;
  BufferedImage softFocusImage1 = null;

  //マスク関連の用語定義
  //「マスク」は、「Stamp」や「CutAndPaste」や「ChangeColorMalti」などで用いる領域区分画像。
  //「Stamp」では、シングル・マスク（グレー・マスク）を用いる。
  //グレー・マスクは、透過部分をグレー色とし、それ以外の部分を表示したい図柄とする。
  //「CutAndPaste」や「ChangeColorMalti」では、マルチ・マスク（カラー・マスク）を用いる。
  //マルチ・マスクは、多色で色分けされた領域区分図であり、表示したい図柄が含まれない。
  //マルチ・マスクを作成する前段階の輪郭線図形も、「アウトライン・マスク」と呼ぶことがある。

  //CutAndPaste命令
  boolean             cutAndPasteMaskFlag = false;
  boolean             cutAndPasteFlag     = false;
  String              capMaskF  = "";   //CutAndPaste用マルチ・マスク画像のファイル名
  BufferedImage       capImageM = null; //マルチ・マスク画像
  BufferedImage       capImage0 = null; //旧画像（切り貼り処理前画像）
  BufferedImage       capImage1 = null; //新画像（切り貼り処理完了画像）
  List<String>        capListF  = null; //前景画像（切り貼り元模様画像）リスト
  List<int[]>         capListI  = null; //指定色数値データリスト

  //ChangeColorMulti命令
  boolean             changeColorMultiBaseFlag = false;
  boolean             changeColorMultiFlag     = false;
  String              ccmMaskF  = "";   //ChangeColorMalti用　元画像のファイル名
  BufferedImage       ccmImage0 = null; //旧画像（連続色相変更処理前画像）
  BufferedImage       ccmImage1 = null; //新画像（連続色相変更処理完了画像）
  List<int[]>         ccmListI0 = null; //旧色指定リスト
  List<int[]>         ccmListI1 = null; //新色指定リスト

  //ChangeColorInner命令（図形内部単独彩色）
  //輪郭線で囲まれた閉じた図形領域を指定色で彩色。
  //彩色対象画像は、輪郭線で囲まれた図形内部に、
  //プログラムがスキャンしながら彩色位置を探し出し彩色し、輪郭線まで領域拡大を行なう。
  //輪郭線が予想外の理由により途切れることがあれば、正しく機能しない。
  boolean             changeColorInnerFlag = false;
  String              changeColorInnerF = ""; //彩色対象ファイル名（パス含む）
  int                 cciOutLineColorR = 0;   //輪郭線色
  int                 cciOutLineColorG = 0;
  int                 cciOutLineColorB = 0;
  int                 cciInnerColorR   = 0;   //彩色指定
  int                 cciInnerColorG   = 0;
  int                 cciInnerColorB   = 0;
  int                 cciInnerX        = 0;   //彩色領域の代表位置X
  int                 cciInnerY        = 0;   //彩色領域の代表位置Y
  int                 cciMode          = 0;   //輪郭線をどの色にするかの選択肢
  BufferedImage       cciImage         = null;//彩色対象画像のイメージ・バッファ

  //ChangeColorInnerMulti命令（図形内部連続彩色）
  //輪郭線で囲まれた閉じた複数の図形領域を指定色で連続的に彩色。
  //彩色対象画像は、輪郭線で囲まれた図形（白地図形式）の内部各所に、
  //プログラムがスキャンしながら彩色位置を探し出し彩色し、輪郭線まで彩色領域拡大を行なう。
  //ただし、輪郭線が予想外の理由により途切れることがあれば、正しく機能しない。
  //また、複雑に屈曲した輪郭線が入り交じる図形を内部連続彩色するとなると、相当な時間を要する。
  //これを便宜的手法によって高速化するとなると、点描画機能を用いるなどして、
  //図形内部の何箇所か（箇所数は多い方が高速）を部分的に指定色で予め彩色しておくと良い。
  boolean             changeColorInnerMultiFlag = false;
  String              changeColorInnerMultiF = ""; //彩色対象ファイル名（パス含む）
  int                 ccimOutLineColorR = 0;   //輪郭線色
  int                 ccimOutLineColorG = 0;
  int                 ccimOutLineColorB = 0;
  BufferedImage       ccimImage         = null;//彩色対象画像のイメージ・バッファ
  List<int[]>         ccimListI;               //数値データ（R,G,B,X,Y）のリスト

  //Spray命令（エアー・ブラシ）
  boolean             sprayFlag = false; //スプレー命令の有無
  double              sprayGaussianOp; //正規分布スプレーでの分散係数
  List<Integer>       sprayDiscStatus_List; //スプレー命令のステータス
  List<Integer>       sprayDiscX_List; //スプレイ円盤状飛沫集団（円盤）の中心ｘ座標
  List<Integer>       sprayDiscY_List; //スプレイ円盤状飛沫集団（円盤）の中心ｙ座標
  List<Integer>       sprayUnitTP_List; //スプレイ飛沫の分布タイプ
  List<Double>        sprayUnitSS_List; //スプレイ飛沫一個のサイズ（px相当）
  List<Integer>       sprayUnitSR_List; //スプレイ飛沫の円盤状拡散分布半径（px相当）
  List<Integer>       sprayUnitST_List; //スプレイ飛沫の分布個数
  List<Integer>       sprayUnitR_List; //スプレイ飛沫の赤成分
  List<Integer>       sprayUnitG_List; //スプレイ飛沫の緑成分
  List<Integer>       sprayUnitB_List; //スプレイ飛沫の青成分
  List<Integer>       sprayUnitA_List; //スプレイ飛沫の透明度（alpha）

  //LinkedLine命令（折れ線）
  boolean             linkedLineFlag = false; //折れ線命令の有無
  List<Integer>       linkedLineStatusList; //折れ線命令のステータス
  List<Double>        linkedLineXList; //折れ線ｘ座標
  List<Double>        linkedLineYList; //折れ線ｙ座標

  //Stamp命令
  boolean             stampFlag = false;
  List<String>        stampListF; //Stamp画像のファイル名リスト
  List<int[]>         stampListI; //Stampの数値データ（位置、サイズ）リスト
  List<BufferedImage> stampListB; //Stamp画像のリスト
  //（注）StampMulti命令は、途中で複数のStamp命令に変換されている。
  Color transparentColor = new Color(200,200,200);  //グレー・マスク作成用の色
  //（注）一部透明化対象画像（スタンプ元画像）の透過部分は、この色で作成する。

  //Image命令
  boolean             imageFlag = false;
  List<String>        imageListF; //背景画像のファイル名リスト
  List<int[]>         imageListI; //背景画像の数値データ（位置、サイズ）リスト
  List<BufferedImage> imageListB; //背景画像リスト
  
  //Antialias命令
  boolean antialiasFlag = false; //線（輪郭線含む）のギザギザ緩和
  float lineWidth;       //線幅（統一）

  //各種の線描画
  double[][] lineArr;    //直線
  double[][] line2Arr;   //仮直線
  double[][] curveArr;   //ベジェ曲線
  double[][] splineArr;  //スプライン曲線（命令文の単純羅列）
  double[][] splineArrX;  //スプライン曲線X座標（曲線毎に分解）
  double[][] splineArrY;  //スプライン曲線Y座標（曲線毎に分解）
  int splineOption = 1000;  //スプライン曲線精細度

  //ParallelLine…命令
  double parallelLineH_x = 0; //横平行線の始点ｘ
  double parallelLineH_y = 0; //横平行線の始点ｙ
  double parallelLineH_l = 0; //線の長さ
  double parallelLineH_w = 0; //線の幅
  int parallelLineH_t    = 0; //線の本数
  int parallelLineH_m    = 0; //線の種類
  int parallelLineH_h    = 0; //中間線の有無
  
  double parallelLineV_x = 0; //縦平行線
  double parallelLineV_y = 0;
  double parallelLineV_l = 0;
  double parallelLineV_w = 0;
  int parallelLineV_t    = 0;
  int parallelLineV_m    = 0;
  int parallelLineV_h    = 0;

  double parallelLineH2_x = 0; //横平行線の始点ｘ（補助用）
  double parallelLineH2_y = 0; //横平行線の始点ｙ
  double parallelLineH2_l = 0; //線の長さ
  double parallelLineH2_w = 0; //線の幅
  int parallelLineH2_t    = 0; //線の本数
  int parallelLineH2_m    = 0; //線の種類
  int parallelLineH2_h    = 0; //中間線の有無
  
  double parallelLineV2_x = 0; //縦平行線（補助用）
  double parallelLineV2_y = 0;
  double parallelLineV2_l = 0;
  double parallelLineV2_w = 0;
  int parallelLineV2_t    = 0;
  int parallelLineV2_m    = 0;
  int parallelLineV2_h    = 0;

  GeneralPath gp;          //自由図形の描画に必要
  double[][] shapeArr;     //自由図形のコマンド全行分のデータ

  double[][] stringHArrD;  //水平文字列（並字）表示でのdouble型データ
  String[][] stringHArrS;  //水平文字列（並字）表示でのString型データ
  double[][] stringBArrD;  //水平文字列（極太字）表示でのdouble型データ
  String[][] stringBArrS;  //水平文字列（極太字）表示でのString型データ
  double[][] stringVArrD;  //垂直文字列（並字）表示でのdouble型データ
  String[][] stringVArrS;  //垂直文字列（並字）表示でのString型データ

  boolean fontBold = false; //太字指定（並字）

  Color backgroundColor;  //背景色
  Color foregroundColorL; //前景色（線色）
  Color foregroundColorS; //前景色（文字色）
  Color foregroundColorB; //前景色（極太文字色）

  int stripePatternType = 0; //描画無視／横縞／縦縞
  int stripePatternW    = 0;  //ストライプ幅
  int stripePRedBase    = 0; //赤基調色
  int stripePGreenBase  = 0; //緑基調色
  int stripePBlueBase   = 0; //青基調色

  //matrixPatternでの用語定義
  //ユニット unit：柄紋一個をユニットと称する。
  //セル cell（コマ／小間）：柄紋一個が６ｘ６の構成なら、その構成単位１個。
  //ピクセル pixel ：セルは複数（例えば８ｘ８）のピクセルで構成される。
  //シフト shift：柄紋を縦横に反復された文様全体の起点を移動させるコマ数。
  //ドロップ drop：柄紋配列の行または列の奇数偶数で、起点位置をずらすコマ数。
  //スキップ skip：柄紋と柄紋との間隔を調整するプラスまたはマイナスのコマ数。
  //リピート repeat：柄紋の反復回数
  //インターミッティング intermitting：１で常時描画、２～１０で間欠描画。

  int matrixPatternNumSt   = 0; //柄紋番号（４桁）
  int matrixPatternWH_St   = 1; //柄紋の幅丈（幅丈兼用コマ数）最低数１
  double matrixShiftH_St   = 0; //横起点シフトのコマ数
  double matrixShiftV_St   = 0; //縦起点シフトのコマ数
  double matrixDropH_St    = 0; //柄紋横ドロップのコマ数
  double matrixDropV_St    = 0; //柄紋縦ドロップのコマ数
  double matrixSkipH_St    = 0; //柄紋横スキップのコマ数
  double matrixSkipV_St    = 0; //柄紋縦スキップのコマ数
  int matrixRepeatH_StX    = 0; //柄紋横リピートの回数
  int matrixRepeatV_StX    = 0; //柄紋縦リピートの回数
  int matrixPRedBaseSt     = 0; //赤基調色
  int matrixPGreenBaseSt   = 0; //緑基調色
  int matrixPBlueBaseSt    = 0; //青基調色
  int matrixColorLimitSt   = 10; //色成分の上限制限（1～10の値）
  int matrixRepeatH_St     = 100; //リピート横（数値は有効なダミー）
  int matrixRepeatV_St     = 100; //リピート縦（数値は有効なダミー）
  int matrixUnitSt         = 6; //コマ数（数値は有効なダミー）
  int matrixIntermittingSt = 10; //間欠描画指定。1～9が間欠描画、10が通常描画
  short[][] rgbSt;            //ランダム配色（統一配色）

  int matrixPatternNumEx   = 0; //柄紋番号（４桁）
  int matrixPatternWH_Ex   = 1; //柄紋の幅丈（幅丈兼用コマ数）最低数１
  double matrixShiftH_Ex   = 0; //横起点シフトのコマ数
  double matrixShiftV_Ex   = 0; //縦起点シフトのコマ数
  double matrixDropH_Ex    = 0; //柄紋横ドロップのコマ数
  double matrixDropV_Ex    = 0; //柄紋縦ドロップのコマ数
  double matrixSkipH_Ex    = 0; //柄紋横スキップのコマ数
  double matrixSkipV_Ex    = 0; //柄紋縦スキップのコマ数
  int matrixRepeatH_ExX    = 0; //柄紋横リピートの指定値の回数
  int matrixRepeatV_ExX    = 0; //柄紋縦リピートの指定値の回数
  int matrixPRedBaseEx     = 0; //赤基調色
  int matrixPGreenBaseEx   = 0; //緑基調色
  int matrixPBlueBaseEx    = 0; //青基調色
  int matrixColorLimitEx   = 10; //色成分の上限制限（1～10の値）
  int matrixRepeatH_Ex     = 100; //リピート横計算値（数値は有効なダミー）
  int matrixRepeatV_Ex     = 100; //リピート縦計算値（数値は有効なダミー）
  int matrixUnitEx         = 6; //コマ数（数値は有効なダミー）
  int matrixIntermittingEx = 10; //間欠描画指定。1～9が間欠描画、10が通常描画
  short[][][][][][] rgbEx;    //ランダム配色（個別配色）

  String[] manualArr;
  String[] patternArr;
  final int pages = 1;    //印刷で用いる形式的な総ページ数
  float quality   = 1.0f; //高画質保存（jpeg）
    //jpegでの画質は、０から１を指定できる（１が最高画質）
  double currentPointX;   //最新クリック位置（マウス入力モード時）
  double currentPointY;   //最新クリック位置（マウス入力モード時）

  List<double[]> lineList   ;   //直線
  List<double[]> line2List  ;   //一時補助線
  List<double[]> curveList  ;   //ベジェ曲線
  List<double[]> splineList ;   //スプライン曲線
  List<double[]> ellipseList  ; //真円、楕円
  List<double[]> rectList   ;   //四角形
  List<double[]> shapeList  ;   //自由図形
  List<double[]> stringHListD ; //水平文字関連でのdouble型データ
  List<String[]> stringHListS ; //水平文字関連でのString型データ
  List<double[]> stringBListD ; //極太文字関連でのdouble型データ
  List<String[]> stringBListS ; //極太文字関連でのString型データ
  List<double[]> stringVListD ; //垂直文字関連でのdouble型データ
  List<String[]> stringVListS ; //垂直文字関連でのString型データ

  //■コンストラクタ
  GCreatorCanvasEngine(GCreator flame)
  {
    f = flame;
    affineScale = 1.0; //拡大率初期設定
    freeScale   = 1.0; //拡大率初期設定
    f.scaleLabel.setText("  (a:x1.0)");  //表示拡大率設定
    f.scaleLabel2.setText("  (f:x1.0)"); //表示拡大率設定
    manualArr  = TextIO.loadStrArr(new File("./sys/GCreator_Manual.txt"));
    patternArr = TextIO.loadStrArr(new File("./sys/GCreator_Pattern.txt"));
    rgbSt = getRandom_rgbSt(); //ランダム色の要求（事前準備）
    rgbEx = getRandom_rgbEx(); //ランダム色の要求（事前準備）
    imageListB = new LinkedList<BufferedImage>();
    stampListB = new LinkedList<BufferedImage>();
  }

  //■■変数初期化
  void setInit()
  { //●内部プログラムから呼び出される
    g2.setPaint(new Color(255,255,255)); //残留画像の消去
    g2.fill(new Rectangle2D.Double(
      0,0,f.canvas.getPreferredSize().width,
      f.canvas.getPreferredSize().height));

    //●変数の初期化
    antialiasFlag = false;
    imagePX_W     = 720; //描画範囲初期設定
    imagePX_H     = 1008; //描画範囲初期設定
    printMM_W     = imagePX_W/3.0; //mmとpxとの換算比率（便宜的独自規格）
    printMM_H     = imagePX_H/3.0; //mmとpxとの換算比率（便宜的独自規格）
    freeScale     = 1.0; //印刷に反映される拡大倍率
    shiftMode     = 0;
    shiftX        = 0;
    shiftY        = 0;
    lineWidth     = 0.5f; //デフォルトの線幅
    f.matrixStUnitLabel.setText("");
    f.matrixExUnitLabel.setText("");
    
    affineDegree     = 0.0; //90度回転角度
    affineDegreeFree = 0.0; //指定角度回転角度
    rotateFlag       = false;//90度回転モード
    rotateFreeFlag   = false;//指定角度回転モード
    rotateFreeW      = 0; //回転に伴って必要なx軸原点移動
    rotateFreeH      = 0; //回転に伴って必要なy軸原点移動

    rotateMultiFlag        = false;
    rotateMultiF           = null;
    rotateMultiStartDegree = 0;
    rotateMultiDegree      = 0;
    rotateMultiTimes       = 0;

    parallelLineH_x = 0; //横平行線
    parallelLineH_y = 0;
    parallelLineH_l = 0;
    parallelLineH_w = 0;
    parallelLineH_t = 0;
    parallelLineH_m = 0;
    parallelLineH_h = 0;
    parallelLineV_x = 0; //縦平行線
    parallelLineV_y = 0;
    parallelLineV_l = 0;
    parallelLineV_w = 0;
    parallelLineV_t = 0;
    parallelLineV_m = 0;
    parallelLineV_h = 0;

    parallelLineH2_x = 0; //横平行線（補助用）
    parallelLineH2_y = 0;
    parallelLineH2_l = 0;
    parallelLineH2_w = 0;
    parallelLineH2_t = 0;
    parallelLineH2_m = 0;
    parallelLineH2_h = 0;
    parallelLineV2_x = 0; //縦平行線
    parallelLineV2_y = 0;
    parallelLineV2_l = 0;
    parallelLineV2_w = 0;
    parallelLineV2_t = 0;
    parallelLineV2_m = 0;
    parallelLineV2_h = 0;
    
    splineArr  = null;   //スプライン曲線（命令文の単純羅列）
    splineArrX = null;   //スプライン曲線X座標（曲線毎に分解）
    splineArrY = null;   //スプライン曲線Y座標（曲線毎に分解）
    splineOption = 1000; //スプライン曲線の精細度のデフォルト

    commandInspection = true;
    backgroundColor   = Color.white; //背景色
    foregroundColorL  = Color.black; //線色
    foregroundColorS  = Color.black; //文字色
    foregroundColorB  = Color.red;   //極太字色（ダミー）

    fontBold = false;

    colorChangeImage0 = null;
//    colorChangeImage1 = null;//加工後の画像　画像を保持

    saveCmdFName = "";  //命令文書の保存ファイル名称（パス付き）
    saveImgFName = "";  //作成画像の保存ファイル名称（パス付き）

    changeColorInnerFlag = false;
    changeColorInnerF = ""; //図形内部彩色対象ファイル名（パス含む）
    cciOutLineColorR = 0;   //輪郭線色デフォルト黒色
    cciOutLineColorG = 0;
    cciOutLineColorB = 0;
    cciInnerColorR   = 255; //彩色デフォルト赤色
    cciInnerColorG   = 0;
    cciInnerColorB   = 0;
    cciInnerX        = 0;
    cciInnerY        = 0;
    cciMode          = 0;   //輪郭線をどの色にするかの選択肢
    //cciImage         = null;//彩色対象画像のイメージ・バッファ　画像を保持

    changeColorInnerMultiFlag = false;
    changeColorInnerMultiF = ""; //彩色対象ファイル名（パス含む）
    ccimOutLineColorR = 0;   //輪郭線色
    ccimOutLineColorG = 0;
    ccimOutLineColorB = 0;
    ccimListI         = new ArrayList<int[]>(); //数値データ（R,G,B,X,Y）のリスト
    //ccimImage         = null; //彩色対象画像（輪郭線画像）のイメージ・バッファ

    sprayFlag             = false; //スプレー
    sprayGaussianOp       = 0.45;
    sprayDiscStatus_List  = new LinkedList<Integer>();
    sprayDiscX_List       = new LinkedList<Integer>();
    sprayDiscY_List       = new LinkedList<Integer>();
    sprayUnitTP_List      = new LinkedList<Integer>();
    sprayUnitSS_List      = new LinkedList<Double>();
    sprayUnitSR_List      = new LinkedList<Integer>();
    sprayUnitST_List      = new LinkedList<Integer>();
    sprayUnitR_List       = new LinkedList<Integer>();
    sprayUnitG_List       = new LinkedList<Integer>();
    sprayUnitB_List       = new LinkedList<Integer>();
    sprayUnitA_List       = new LinkedList<Integer>();

    linkedLineFlag        = false; //折れ線
    linkedLineStatusList  = new LinkedList<Integer>();
    linkedLineXList       = new LinkedList<Double>();
    linkedLineYList       = new LinkedList<Double>();

    cutAndSaveImgFName = ""; //作成画像の切り取り保存ファイル名称（パス付き）
    cutAndSaveImgX1    = 0; //作成画像の切り出し領域の左上ｘ座標
    cutAndSaveImgY1    = 0; //作成画像の切り出し領域の左上ｙ座標
    cutAndSaveImgX2    = 0; //作成画像の切り出し領域の右下ｘ座標
    cutAndSaveImgY2    = 0; //作成画像の切り出し領域の右下ｙ座標

    reverseImageFile    = "";    //画像反転の対象画像ファイル名（加工前）
    reverseImageFlag    = false; //画像反転要求の有無
    reverseImageMode    = 0;     //反転方式の指定
    reverseImage0       = null;  //加工前画像
//    reverseImage1       = null;  //加工後画像（初期化しない）　画像を保持
//    reverseImageX       = null;  //90度、270度、回転加工後画像（初期化しない）　画像を保持

    mixFName0 = ""; //二重像画像処理のファイル名（パス付き）
    mixFName1 = ""; //二重像画像処理のファイル名（パス付き）
    mixMode   = 1; //通常のモード
    mixImage0 = null;
    mixImage1 = null;
//    mixImage2 = null; //加工後の画像　　repaint()のために画像を保持

    softFocusFName = ""; //ソフト・フォーカスのファイル名（パス付き）
    adjustFName    = "";    //色相調整のファイル名（パス付き）
    gradientFName  = "";  //階調加工処理のファイル名（パス付き）
    
    colorChangeFile  = "";    //単独色相変換処理のファイル名（パス付き）
    colorChangeFlag  = false; //単独色相変更要求の有無
    acceptableLimitC = 0;     //単独色相変動許容値
    colorChangeMode  = 0;     //単独色相変更モード
    colorChange1R    = 0;     //単独色相変更の旧色
    colorChange1G    = 0;
    colorChange1B    = 0;
    colorChange2R    = 0;     //単独色相変更の新色
    colorChange2G    = 0;
    colorChange2B    = 0;

    stripePatternType = 0;

    matrixPatternNumSt   = 0; //描画しない（空白）
    matrixRepeatH_St     = 100; //有効な初期値
    matrixRepeatV_St     = 100; //有効な初期値
    matrixShiftH_St      = 0; //全体横シフト
    matrixShiftV_St      = 0; //全体縦シフト
    matrixDropH_St       = 0; //図柄横ドロップ
    matrixDropV_St       = 0; //図柄縦ドロップ
    matrixSkipH_St       = 0; //図柄横スキップ
    matrixSkipV_St       = 0; //図柄縦スキップ
    matrixRepeatH_St     = 0; //図柄横リピート
    matrixRepeatV_St     = 0; //図柄縦リピート
    matrixIntermittingSt = 10; //間欠描画
    matrixColorLimitSt   = 10; //色成分の上限制限（1～10の値）
//      matrixUnitSt       = 6; 初期化不可

    matrixPatternNumEx   = 0; //描画しない（空白）
    matrixRepeatH_Ex     = 100; //有効な初期値
    matrixRepeatV_Ex     = 100; //有効な初期値
    matrixShiftH_Ex      = 0; //全体横シフト
    matrixShiftV_Ex      = 0; //全体縦シフト
    matrixDropH_Ex       = 0; //図柄横ドロップ
    matrixDropV_Ex       = 0; //図柄縦ドロップ
    matrixSkipH_Ex       = 0; //図柄横スキップ
    matrixSkipV_Ex       = 0; //図柄縦スキップ
    matrixRepeatH_Ex     = 0; //図柄横リピート
    matrixRepeatV_Ex     = 0; //図柄縦リピート
    matrixIntermittingEx = 10; //間欠描画
    matrixColorLimitEx   = 10; //色成分の上限制限（1～10の値）
//    matrixUnitEx         = 6; 初期化不可

    lineList   = new ArrayList<double[]>(); //直線
    line2List  = new ArrayList<double[]>(); //一時補助線
    curveList  = new ArrayList<double[]>(); //ベジェ曲線
    splineList  = new ArrayList<double[]>(); //スプライン曲線
    ellipseList  = new ArrayList<double[]>(); //真円、楕円
    rectList   = new ArrayList<double[]>(); //四角形
    shapeList  = new ArrayList<double[]>(); //自由図形
    stringHListD = new ArrayList<double[]>(); //水平文字
    stringHListS = new ArrayList<String[]>(); //
    stringBListD = new ArrayList<double[]>(); //極太文字
    stringBListS = new ArrayList<String[]>(); //
    stringVListD = new ArrayList<double[]>(); //垂直文字
    stringVListS = new ArrayList<String[]>(); //

//    imageFlag    = false; //setinit2() で初期化する
//    imageListI   = new LinkedList<int[]>(); //背景画像
//    imageListF   = new LinkedList<String>();
//    imageListB   = new LinkedList<BufferedImage>();

//    stampFlag  = false; 初期化不可 //setinit2() で初期化する
//    stampListI = new LinkedList<int[]>();
//    stampListF = new LinkedList<String>();
//    stampListB = new LinkedList<BufferedImage>();

//    cutAndPasteMaskFlag = false;
//    cutAndPasteFlag     = false;
//    capMaskF = "";     //切り貼り用マルチ・マスクのファイル名（パス付き）
//    capImageM = null;  //マルチ・マスクの画像
//    capImage0 = null;  //切り貼り元模様画像
//    capImage1 = null;  //加工後の画像　画像を保持
//    capListF  = new LinkedList<String>(); //切り貼り元画像
//    capListI  = new LinkedList<int[]>();  //色相指定リスト

//    changeColorMultiBaseFlag = false;
//    changeColorMultiFlag     = false;
//    ccmMaskF  = "";   //ChangeColorMalti用　元画像のファイル名
//    ccmImage0 = null; //旧画像（連続色相変更処理前画像）
//    ccmImage1 = null; //新画像（連続色相変更処理完了画像）　画像を保持
//    ccmListI0 = new LinkedList<int[]>(); //旧色指定リスト
//    ccmListI1 = new LinkedList<int[]>(); //新色指定リスト

    //------------------------------------------------------------

    //●命令文の全文を解析し、命令文で指定された内容を変数に格納
    String commandZ = f.cArea.getText();
    f.analyze.commandAnalyze(commandZ); //解析

    //------------------------------------------------------------

    //●直線 数値の取得
    Object[] lineArr0 = lineList.toArray();
    lineArr  = new double[lineArr0.length][4];
    for(int i=0; i<lineArr0.length; i++)
    {
      lineArr[i][0] = ((double[])(lineArr0[i]))[0];
      lineArr[i][1] = ((double[])(lineArr0[i]))[1];
      lineArr[i][2] = ((double[])(lineArr0[i]))[2];
      lineArr[i][3] = ((double[])(lineArr0[i]))[3];
    }
    //●仮直線Draw2 数値の取得
    Object[] line2Arr0 = line2List.toArray();
    line2Arr  = new double[line2Arr0.length][5];
    for(int i=0; i<line2Arr0.length; i++)
    {
      line2Arr[i][0] = ((double[])(line2Arr0[i]))[0];
      line2Arr[i][1] = ((double[])(line2Arr0[i]))[1];
      line2Arr[i][2] = ((double[])(line2Arr0[i]))[2];
      line2Arr[i][3] = ((double[])(line2Arr0[i]))[3];
      line2Arr[i][4] = ((double[])(line2Arr0[i]))[4];
    }
    //●ベジェ曲線 数値の取得
    Object[] curveArr0 = curveList.toArray();
    curveArr  = new double[curveArr0.length][8];
    for(int i=0; i<curveArr0.length; i++)
    {
      curveArr[i][0] = ((double[])(curveArr0[i]))[0];
      curveArr[i][1] = ((double[])(curveArr0[i]))[1];
      curveArr[i][2] = ((double[])(curveArr0[i]))[2];
      curveArr[i][3] = ((double[])(curveArr0[i]))[3];
      curveArr[i][4] = ((double[])(curveArr0[i]))[4];
      curveArr[i][5] = ((double[])(curveArr0[i]))[5];
      curveArr[i][6] = ((double[])(curveArr0[i]))[6];
      curveArr[i][7] = ((double[])(curveArr0[i]))[7];
    }

    //●スプライン曲線 数値の取得
    Object[] splineArr0 = splineList.toArray();
    splineArr  = new double[splineArr0.length][3];
    for(int i=0; i<splineArr0.length; i++)
    {
      splineArr[i][0] = ((double[])(splineArr0[i]))[0];
      splineArr[i][1] = ((double[])(splineArr0[i]))[1];
      splineArr[i][2] = ((double[])(splineArr0[i]))[2];
    }
    int arrCnt0 = 0; //スプライン曲線の本数をカウント
    for(int i=0; i<splineArr.length; i++)
    {
      if(splineArr[i][0]==0) //スプライン曲線の起点
        arrCnt0++;
    }
    if(arrCnt0>0) //スプライン曲線があれば
    {
      splineArrX = new double[arrCnt0][]; //スプライン曲線毎に分解
      splineArrY = new double[arrCnt0][]; //スプライン曲線毎に分解
    }

    int arrCnt1 = 0; //スプライン曲線の制御点を個別にカウント
    arrCnt0 = 0; //スプライン曲線の本数カウントを初期化
    int scale = 1;
    int k=0; //複数あるスプライン曲線の特定番号
    for(int i=0; i<splineArr.length; i++)
    {
      if(splineArr[i][0]==0) //或るスプライン曲線の起点
      {
        ;
      }
      else if(splineArr[i][0]==1) //或るスプライン曲線の制御点
      {
        arrCnt1++; //制御点のカウント
      }
      else if(splineArr[i][0]==2) //或るスプライン曲線の終点
      {
        //スプライン曲線における制御点の個数は、一個以上で、個数に上限なし。
        splineArrX[k] = new double[arrCnt1+2]; //起点＋制御点数個＋終点
        splineArrY[k] = new double[arrCnt1+2];
        for(int j=0; j<arrCnt1+2; j++)
        { //座標データをコピー
          splineArrX[k][j] = splineArr[i-(arrCnt1+1)+j][1];
          splineArrY[k][j] = splineArr[i-(arrCnt1+1)+j][2];
        }
        k++; //次のスプライン曲線へ移行
        arrCnt1 = 0; //制御点カウント
        arrCnt0++; //スプライン曲線の本数カウント
      }
    }

    //●色塗り自由図形 数値の取得
    Object[] shapeArr0 = shapeList.toArray();
    shapeArr  = new double[shapeArr0.length][9];
    for(int i=0; i<shapeArr0.length; i++)
    {
      shapeArr[i][0] = (((double[])(shapeArr0[i]))[0]);
      shapeArr[i][1] = (((double[])(shapeArr0[i]))[1]);
      shapeArr[i][2] = (((double[])(shapeArr0[i]))[2]);
      shapeArr[i][3] = (((double[])(shapeArr0[i]))[3]);
      shapeArr[i][4] = (((double[])(shapeArr0[i]))[4]);
      shapeArr[i][5] = (((double[])(shapeArr0[i]))[5]);
      shapeArr[i][6] = (((double[])(shapeArr0[i]))[6]);
      shapeArr[i][7] = (((double[])(shapeArr0[i]))[7]);
      shapeArr[i][8] = (((double[])(shapeArr0[i]))[8]);
    }

    //●文字列（並字）　データの取得
    Object[] stringHArrD0 = stringHListD.toArray();
    Object[] stringHArrS0 = stringHListS.toArray();
    stringHArrD = new double[stringHArrD0.length][3];
    stringHArrS = new String[stringHArrS0.length][2];
    for(int i=0; i<stringHArrD0.length; i++)
    {
      stringHArrD[i][0] = ((double[])(stringHArrD0[i]))[0];
      stringHArrD[i][1] = ((double[])(stringHArrD0[i]))[1];
      stringHArrD[i][2] = ((double[])(stringHArrD0[i]))[2];
    }
    for(int i=0; i<stringHArrD0.length; i++)
    {
      stringHArrS[i][0] = ((String[])(stringHArrS0[i]))[0];
      stringHArrS[i][1] = ((String[])(stringHArrS0[i]))[1];
    }

    //●文字列（極太字）　データの取得
    Object[] stringBArrD0 = stringBListD.toArray();
    Object[] stringBArrS0 = stringBListS.toArray();
    stringBArrD = new double[stringBArrD0.length][3];
    stringBArrS = new String[stringBArrS0.length][2];
    for(int i=0; i<stringBArrD0.length; i++)
    {
      stringBArrD[i][0] = ((double[])(stringBArrD0[i]))[0];
      stringBArrD[i][1] = ((double[])(stringBArrD0[i]))[1];
      stringBArrD[i][2] = ((double[])(stringBArrD0[i]))[2];
    }
    for(int i=0; i<stringBArrD0.length; i++)
    {
      stringBArrS[i][0] = ((String[])(stringBArrS0[i]))[0];
      stringBArrS[i][1] = ((String[])(stringBArrS0[i]))[1];
    }

    //●垂直文字列（疑似縦書き）　データの取得
    Object[] stringVArrD0 = stringVListD.toArray();
    Object[] stringVArrS0 = stringVListS.toArray();
    stringVArrD = new double[stringVArrD0.length][3];
    stringVArrS = new String[stringVArrS0.length][2];
    for(int i=0; i<stringVArrD0.length; i++)
    {
      stringVArrD[i][0] = ((double[])(stringVArrD0[i]))[0];
      stringVArrD[i][1] = ((double[])(stringVArrD0[i]))[1];
      stringVArrD[i][2] = ((double[])(stringVArrD0[i]))[2];
    }
    for(int i=0; i<stringVArrD0.length; i++)
    {
      stringVArrS[i][0] = ((String[])(stringVArrS0[i]))[0];
      stringVArrS[i][1] = ((String[])(stringVArrS0[i]))[1];
    }
  } //setInit()

  void setInit2()
  { //●内部プログラムから呼び出される特殊な初期化
    //メニュー・イベント発生またはボタン・イベント発生でのみ初期化される
    imageFlag   = false;
    imageListI  = new LinkedList<int[]>(); //背景画像
    imageListF  = new LinkedList<String>();
    imageListB  = new LinkedList<BufferedImage>();

    stampFlag  = false; 
    stampListI = new LinkedList<int[]>();
    stampListF = new LinkedList<String>();
    stampListB = new LinkedList<BufferedImage>();
    transparentColor = new Color(200,200,200);
      
    cutAndPasteMaskFlag = false;
    cutAndPasteFlag     = false;
    capMaskF = "";     //切り貼り用マルチ・マスクのファイル名（パス付き）
    capImageM = null;  //マルチ・マスクの画像
    capImage0 = null;  //切り貼り元模様画像
    capImage1 = null;  //加工後の画像　画像を保持
    capListF  = new LinkedList<String>(); //切り貼り元画像
    capListI  = new LinkedList<int[]>();  //色相指定リスト

    changeColorMultiBaseFlag = false;
    changeColorMultiFlag     = false;
    ccmMaskF  = "";   //ChangeColorMalti用　元画像のファイル名
    ccmImage0 = null; //旧画像（連続色相変更処理前画像）
    ccmImage1 = null; //新画像（連続色相変更処理完了画像）　画像を保持
    ccmListI0 = new LinkedList<int[]>(); //旧色指定リスト
    ccmListI1 = new LinkedList<int[]>(); //新色指定リスト
  } //setInit2()

  void setInit3()
  { //●内部プログラムから呼び出される特殊な初期化の追加補足
    //メニュー・イベント発生またはボタン・イベント発生でのみ初期化される
      if(imageFlag==true)
        image(); //背景画像表示の下準備
      if(stampFlag==true)
        f.canvas.stamp(); //スタンプ押印の下準備
      if(cutAndPasteMaskFlag==true && cutAndPasteFlag==true)
        cutAndPaste(); //切り貼り処理の下準備
      if(changeColorMultiBaseFlag==true && changeColorMultiFlag==true)
        changeColorMulti(); //連続色相変更処理の下準備

      softFocus(); //ソフト・フォーカス処理の下準備
      changeColorInner(); //図形内部単独彩色
      changeColorInnerMulti(); //図形内部連続彩色
      mixImage(); //二重画像処理の下準備
      reverseImage(); //画像反転処理の下準備
      rotateMulti(); //画像多段回転処理の下準備
      adjustRGB(); //色調全域調整の下準備
      colorChange(f.canvas.colorChangeMode); //色相部分領域変更処理の下準備
      colorGradient(); //色相グラデーション加工の下準備
  } //setInit3()

  //■■描画処理
  public void paintComponent(Graphics g)
  {
    drawPage(g, false);
  }

  //■１ページ分の描画（現状では、１ページ分のみ）
  //drawPage()は、印刷時にも呼び出される。
  //このプログラムでは、改ページの概念を設定していないが、
  //改ページできる形にプログラムを改造することも可能。
  void drawPage(Graphics g, boolean printMode)
  {
    //●描画の初期対応
    g2 = (Graphics2D)g;

    setInit();  //変数初期化
    g2.scale(affineScale*freeScale, affineScale*freeScale); //拡大
    g2.setPaint(new Color(255,255,255)); //残留画像の消去
    g2.fill(new Rectangle2D.Double(
      0,0,f.canvas.getPreferredSize().width,
      f.canvas.getPreferredSize().height));
    g2.setColor(backgroundColor);  //画像サイズ指定領域の画像消去
    g2.fill(new Rectangle2D.Double(0,0,imagePX_W,imagePX_H));

    if(commandInspection == false) 
    {  //命令記述に誤りがあれば
      return;
    }

    if(rotateFlag) //画像全体を90度回転させる
    {
      g2.translate(imagePX_H, 0); //移動
      affineDegree = 90; //90度回転指定
      g2.rotate(affineDegree*Math.PI/180.0);
      //座標原点を中心に回転（指定順注意）
    }
    if(rotateFreeFlag) //画像全体を指定角度回転させる
    {
      g2.translate(rotateFreeW, rotateFreeH); //指定数値で移動
      affineDegree = affineDegreeFree; //指定角度回転指定
      g2.rotate(affineDegreeFree*Math.PI/180.0);
      //座標原点を中心に回転（指定順注意）
    }

    if(antialiasFlag == true) //線のギザギザ緩和
      //細い線の色が、周囲との中間色化で、非常に薄くなる
      g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
        RenderingHints.VALUE_ANTIALIAS_ON);
    else
      g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
        RenderingHints.VALUE_ANTIALIAS_OFF);

    //●シフト処理の計算
    double sX = shiftX;
    double sY = shiftY;
    if(shiftMode==1)
    {  //シフトあり
      ;
    }
    else
    {  //シフトなし
      shiftX = sX = 0;
      shiftY = sY = 0;
    }

    //●線の描画準備
    BasicStroke solidLine = new BasicStroke( //太さ可変の実線の定義
      lineWidth, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_MITER);
    BasicStroke solidLineX = new BasicStroke( //太さ固定の細実線の定義
      0.5f, BasicStroke.CAP_SQUARE, 
      BasicStroke.JOIN_MITER);

    BasicStroke dashLine = new BasicStroke( //太さ固定の点線の定義
      2.0f, BasicStroke.CAP_SQUARE, 
      BasicStroke.JOIN_MITER, 5f, new float[] {4f,6f,10f,6f,4f,6f}, 0f);

    BasicStroke dotLine = new BasicStroke( //太さ固定で密の点線の定義
      0.5f, BasicStroke.CAP_SQUARE, 
      BasicStroke.JOIN_MITER, 5f, new float[] {4f,4f}, 0f);
    BasicStroke dotLine2 = new BasicStroke( //太さ固定で粗の点線の定義
      0.5f, BasicStroke.CAP_SQUARE, 
      BasicStroke.JOIN_MITER, 5f, new float[] {4f,12f}, 0f);

    g2.setStroke(solidLine); //太さ可変の実線に設定
    g2.setColor(foregroundColorL);

    //------------------------------------------------------------

    //●●以下は、各種の「描画処理」
    //一般描画処理とは、線描画、面描画、点描画、ストライブ模様描画、マトリックス模様描画。
    //特殊描画処理とは、背景画像描画、スタンプ描画、各種画像変形処理。
    //全種類の描画処理（一般描画処理＋特殊描画処理）の順序は、全面描画が先で部分描画が後の順序。
    //また、面描画、線描画、文字描画の順序で並べる。
    //具体的には、以下に始まる描画順序である。
    //この順序に不都合があれば、途中で画像保存し、その画像をImage命令で再表示して作業を継続する。
    //複雑な構成の画像は、部品画像を別途多数作成し保存し、最終的にそれらを寄せ合わせて仕上げる。

    //●Image 背景画像（特殊描画処理の一種）の描画。
    if(f.canvas.imageFlag==true && imageListB!=null && imageListI!=null)
    {
      for(int i=0; i<imageListB.size(); i++)
      {
        //画像サイズ計算
        int w = imageListB.get(i).getWidth(this);
        int h = imageListB.get(i).getHeight(this);
        int imageX = 0;
        int imageY = 0;
        int imageW = 0;
        int imageH = 0;
        boolean err = false;
        try
        {
          imageX = imageListI.get(i)[0];
          imageY = imageListI.get(i)[1];
          imageW = imageListI.get(i)[2];
          imageH = imageListI.get(i)[3];
        }
        catch (Exception e) 
        {
          f.message(e.toString());
          err = true;
        }
        if(imageListB.get(i) != null && err == false)
        { //描画する
          g2.drawImage(imageListB.get(i), imageX, imageY, imageW, imageH, this);
        }
      }
    }

    //------------------------------------------------------------

    //以下は、各種メソッドで指定された画像をキャンバスに表示する。

    //●ランダム・ストライプ描画
    if(stripePatternType >= 1)
    {
      //  int stripePatternType; //無効／水平／垂直、0/1/2
      //  int stripePatternW //ストライプ幅
      //  int stripePRedBase  //赤基調色
      //  int stripePGreenBase //緑基調色
      //  int stripePBlueBase //青基調色
      int sizeW = (int)imagePX_W; //描画サイズ幅
      int sizeH = (int)imagePX_H; //描画サイズ幅
      int repeat; //ストライプ本数
      if(stripePatternType == 1) //水平
        repeat = (sizeH/stripePatternW)+1;
      else  //垂直
        repeat = (sizeW/stripePatternW)+1;
      if(stripePRedBase > 240)  stripePRedBase = 240;
      if(stripePGreenBase > 240)  stripePGreenBase = 240;
      if(stripePBlueBase > 240)  stripePBlueBase = 240;
      int r1,g1,b1;
      for(int i=0;i<repeat;i++)
      {
        r1 = stripePRedBase + ran.nextInt(255-stripePRedBase);
        g1 = stripePGreenBase + ran.nextInt(255-stripePGreenBase);
        b1 = stripePBlueBase + ran.nextInt(255-stripePBlueBase);
        g2.setColor(new Color(r1,g1,b1));
        if(stripePatternType==1) //水平
          g2.fillRect(0, i*stripePatternW , sizeW, stripePatternW);
        else if(stripePatternType==2)  //垂直
          g2.fillRect(i*stripePatternW ,0, stripePatternW, sizeH);
      }
      stripePatternType =0;
    }

    //●マトリックス・パターン描画
    if(matrixPatternNumSt >= 1 || matrixPatternNumEx >= 1)
    {
      int sizeW = (int)imagePX_W; //有効描画範囲幅
      int sizeH = (int)imagePX_H; //有効描画範囲丈
      int cSt = matrixPatternWH_St;  //１コマのピクセル数（横）
      int cEx = matrixPatternWH_Ex;  //１コマのピクセル数（横）
      matrixRepeatH_St = sizeW/(int)(cSt*matrixUnitSt+matrixSkipH_St)+2;  //横リピート回数
      matrixRepeatV_St = sizeH/(int)(cSt*matrixUnitSt+matrixSkipV_St)+2;  //縦リピート回数
      matrixRepeatH_Ex = sizeW/(int)(cEx*matrixUnitEx+matrixSkipH_Ex)+2;  //横リピート回数
      matrixRepeatV_Ex = sizeH/(int)(cEx*matrixUnitEx+matrixSkipV_Ex)+2;  //縦リピート回数
      int x1 = 0;  //Ｘ軸座標位置
      int y1 = 0;  //Ｙ軸座標位置
      int matrixPatternSt[][] = getPatternData(matrixPatternNumSt, "St");
      int matrixPatternEx[][] = getPatternData(matrixPatternNumEx, "Ex");
      if(matrixPatternNumSt>0)
        f.matrixStUnitLabel.setText("　　St:" + matrixUnitSt +"x"+ matrixUnitSt);
      else
        f.matrixStUnitLabel.setText("");
      if(matrixPatternNumEx>0)
        f.matrixExUnitLabel.setText("　　Ex:" + matrixUnitEx +"x"+ matrixUnitEx);
      else
        f.matrixExUnitLabel.setText("");

      //配色描画実施
      //標準メソッドStの使用（統一配色）
      try
      {
        if(matrixPatternNumSt>0)
        {
          //色指定は、１番色から９番色まで使用可能。
          boolean colorFlagSt = true; //描画の有無
          double matrixSkipH_StX = 0; //柄紋横スキップ累積の初期値
          double matrixSkipV_StY = 0; //柄紋縦スキップ累積の初期値
          double dropX =0; //偶数行で柄ずらし（ドロップ）の初期値
          double dropY =0; //偶数列で柄ずらし（ドロップ）の初期値
          int K2 =0;
          ran2St = new Random(randomSeed2);
          int randomSt = 0; 
          
          //リピート計算およびリピート指定
          if(matrixSkipH_St < 0) //マイナス・スキップ対応のリピート追加
            matrixRepeatH_St = (int)((sizeW/(cSt*matrixUnitSt+matrixSkipH_St))*2.5);
            //ただし、大きな数値のマイナス・スキップには対応できない。
            //マイナス・スキップが大きい場合、リピート指定を行なう
          if(matrixSkipV_St < 0) //マイナス・スキップ対応のリピート追加
            matrixRepeatV_St = (int)((sizeH/(cSt*matrixUnitSt+matrixSkipV_St))*2.5);
            //ただし、大きな数値のマイナス・スキップには対応できない。
            //マイナス・スキップが大きい場合、リピート指定を行なう
          if(matrixRepeatH_StX >= 1) //リピート指定あれば、それに従う
            matrixRepeatH_St = matrixRepeatH_StX;
          if(matrixRepeatV_StX >= 1) //リピート指定あれば、それに従う
            matrixRepeatV_St = matrixRepeatV_StX;

          for( int i=0; i<matrixRepeatH_St; i++) //横リピート
          {
            matrixSkipH_StX = matrixSkipH_StX + matrixSkipH_St; //横スキップの累積
            for(int j=0; j<matrixRepeatV_St; j++) //縦リピート
            {
              if(j==0 && i==0) //縦スキップの準備
                matrixSkipV_StY = matrixSkipV_St;
              else if(j==0)
                matrixSkipV_StY = 0;
              if(K2==matrixUnitSt-1)
                matrixSkipV_StY = matrixSkipV_StY + matrixSkipV_St; //縦スキップの累積
              randomSt = ran2St.nextInt(10);
              for(int K=0; K<matrixUnitSt; K++) //横コマ数で反復
              {
                K2 = K;
                for(int L=0; L<matrixUnitSt; L++) //縦コマ数で反復
                {
                  if     (matrixPatternSt[K][L] == 1)
                    g2.setColor(new Color(rgbSt[0][0],rgbSt[0][1],rgbSt[0][2]));
                  else if(matrixPatternSt[K][L] == 2)
                    g2.setColor(new Color(rgbSt[1][0],rgbSt[1][1],rgbSt[1][2]));
                  else if(matrixPatternSt[K][L] == 3)
                    g2.setColor(new Color(rgbSt[2][0],rgbSt[2][1],rgbSt[2][2]));
                  else if(matrixPatternSt[K][L] == 4)
                    g2.setColor(new Color(rgbSt[3][0],rgbSt[3][1],rgbSt[3][2]));
                  else if(matrixPatternSt[K][L] == 5)
                    g2.setColor(new Color(rgbSt[4][0],rgbSt[4][1],rgbSt[4][2]));
                  else if(matrixPatternSt[K][L] == 6)
                    g2.setColor(new Color(rgbSt[5][0],rgbSt[5][1],rgbSt[5][2]));
                  else if(matrixPatternSt[K][L] == 7)
                    g2.setColor(new Color(rgbSt[6][0],rgbSt[6][1],rgbSt[6][2]));
                  else if(matrixPatternSt[K][L] == 8)
                    g2.setColor(new Color(rgbSt[7][0],rgbSt[7][1],rgbSt[7][2]));
                  else if(matrixPatternSt[K][L] == 9)
                    g2.setColor(new Color(rgbSt[8][0],rgbSt[8][1],rgbSt[8][2]));
                  else if(matrixPatternSt[K][L] == 0) //配色なし（描画せず）
                    colorFlagSt = false;
                  else 
                    g2.setColor(Color.black);  //警告色
                  x1 = (int)(matrixShiftH_St*cSt);
                  y1 = (int)(matrixShiftV_St*cSt);
                  if(j%2 == 1) //柄紋の横ドロップ処理
                    dropX = matrixDropH_St;
                  else
                    dropX = 0;
                  if(i%2 == 1) //柄紋の縦ドロップ処理
                    dropY = (int)matrixDropV_St;
                  else
                    dropY  = 0;
                  if((colorFlagSt == true) && 
                    (randomSt < matrixIntermittingSt)) //間欠描画
                    g2.fillRect(x1+i*matrixUnitSt*cSt+L*cSt+
                      (int)(matrixSkipH_StX*cSt)+(int)(dropX*cSt), 
                      y1+j*matrixUnitSt*cSt+K*cSt+
                      (int)(matrixSkipV_StY*cSt)+(int)(dropY*cSt), 
                      cSt, cSt); //x, y, 幅, 高
                  else //色番号が０の場合、描画しない
                    colorFlagSt = true;
                }
              }
            }
          }
        }
      }
      catch(ArrayIndexOutOfBoundsException e)
      { 
        if(matrixPatternNumSt>0)
          rgbSt = getRandom_rgbSt();
        repaint(); //表示エラー回復の試み。
      }

      //拡張メソッドExの使用（個別配色）
      try
      {
        if(matrixPatternNumEx>0)
        {
          //色指定は、１番色から９番色まで使用可能。
          boolean colorFlagEx = true; //描画の有無
          double matrixSkipH_ExX = 0; //柄紋横スキップ累積の初期値
          double matrixSkipV_ExY = 0; //柄紋縦スキップ累積の初期値
          double dropX =0; //偶数行で柄ずらし（ドロップ）の初期値
          double dropY =0; //偶数列で柄ずらし（ドロップ）の初期値
          int K2 =0;
          ran2Ex = new Random(randomSeed2+567);
          int randomEx = 0; 
          
          //リピート計算およびリピート指定
          if(matrixSkipH_Ex < 0) //マイナス・スキップ対応のリピート追加
            matrixRepeatH_Ex = (int)((sizeW/(cEx*matrixUnitEx+matrixSkipH_Ex))*2.5);
            //ただし、異常な数値のスキップは想定していないので、リピート指定を行なう
          if(matrixSkipV_Ex < 0) //マイナス・スキップ対応のリピート追加
            matrixRepeatV_Ex = (int)((sizeH/(cEx*matrixUnitEx+matrixSkipH_Ex))*2.5);
            //ただし、異常な数値のスキップは想定していないので、リピート指定を行なう
          if(matrixRepeatH_ExX >= 1) //リピート指定あれば、それに従う
            matrixRepeatH_Ex = matrixRepeatH_ExX;
          if(matrixRepeatV_ExX >= 1) //リピート指定あれば、それに従う
            matrixRepeatV_Ex = matrixRepeatV_ExX;
          //特殊なマトリックス指定をする場合、プログラムでのリピート計算では対処不能と
          //なることもあるため、その場合、マトリックス命令文の中でリピート指定を行なう。
          //通常の使用なら、リピート数値指定は、０指定（リピート自動計算）で構わない。

          for( int i=0; i<matrixRepeatH_Ex; i++) //横リピート
          {
            matrixSkipH_ExX = matrixSkipH_ExX + matrixSkipH_Ex; //横スキップの累積
            for(int j=0; j<matrixRepeatV_Ex; j++) //縦リピート
            {
              if(j==0 && i==0) //縦スキップの準備
                matrixSkipV_ExY = matrixSkipV_Ex;
              else if(j==0)
                matrixSkipV_ExY = 0;
              if(K2==matrixUnitEx-1)
                matrixSkipV_ExY = matrixSkipV_ExY + matrixSkipV_Ex; //縦スキップの累積
              randomEx = ran2Ex.nextInt(10);
              for(int K=0; K<matrixUnitEx; K++) //横コマ数で反復
              {
              	K2 = K;
                for(int L=0; L<matrixUnitEx; L++) //縦コマ数で反復
                {
                  if     (matrixPatternEx[K][L] == 1) //色を設定
                    g2.setColor(new Color(rgbEx[i][j][K][L][0][0],
                      rgbEx[i][j][K][L][0][1],rgbEx[i][j][K][L][0][2]));
                  else if(matrixPatternEx[K][L] == 2)
                    g2.setColor(new Color(rgbEx[i][j][K][L][1][0],
                      rgbEx[i][j][K][L][1][1],rgbEx[i][j][K][L][1][2]));
                  else if(matrixPatternEx[K][L] == 3)
                    g2.setColor(new Color(rgbEx[i][j][K][L][2][0],
                      rgbEx[i][j][K][L][2][1],rgbEx[i][j][K][L][2][2]));
                  else if(matrixPatternEx[K][L] == 4)
                    g2.setColor(new Color(rgbEx[i][j][K][L][3][0],
                      rgbEx[i][j][K][L][3][1],rgbEx[i][j][K][L][3][2]));
                  else if(matrixPatternEx[K][L] == 5)
                    g2.setColor(new Color(rgbEx[i][j][K][L][4][0],
                      rgbEx[i][j][K][L][4][1],rgbEx[i][j][K][L][4][2]));
                  else if(matrixPatternEx[K][L] == 6)
                    g2.setColor(new Color(rgbEx[i][j][K][L][5][0],
                      rgbEx[i][j][K][L][5][1],rgbEx[i][j][K][L][5][2]));
                  else if(matrixPatternEx[K][L] == 7)
                    g2.setColor(new Color(rgbEx[i][j][K][L][6][0],
                      rgbEx[i][j][K][L][6][1],rgbEx[i][j][K][L][6][2]));
                  else if(matrixPatternEx[K][L] == 8)
                    g2.setColor(new Color(rgbEx[i][j][K][L][7][0],
                      rgbEx[i][j][K][L][7][1],rgbEx[i][j][K][L][7][2]));
                  else if(matrixPatternEx[K][L] == 9)
                    g2.setColor(new Color(rgbEx[i][j][K][L][8][0],
                      rgbEx[i][j][K][L][8][1],rgbEx[i][j][K][L][8][2]));
                  else if(matrixPatternEx[K][L] == 0) //配色なし（描画せず）
                    colorFlagEx = false;
                  else 
                    g2.setColor(Color.black);  //警告色

                  x1 = (int)(matrixShiftH_Ex*cEx);
                  y1 = (int)(matrixShiftV_Ex*cEx);
                  if(j%2 == 1) //柄紋の横ドロップ処理
                    dropX = matrixDropH_Ex;
                  else
                    dropX = 0;
                  if(i%2 == 1) //柄紋の縦ドロップ処理
                    dropY = (int)matrixDropV_Ex;
                  else
                    dropY  = 0;

                  if((colorFlagEx == true) && 
                    (randomEx < matrixIntermittingEx)) //間欠描画
                    g2.fillRect(x1+i*matrixUnitEx*cEx+L*cEx+
                      (int)(matrixSkipH_ExX*cEx)+(int)(dropX*cEx), 
                      y1+j*matrixUnitEx*cEx+K*cEx+
                      (int)(matrixSkipV_ExY*cEx)+(int)(dropY*cEx), 
                      cEx, cEx); //x, y, 幅, 高
                  else //色番号が０の場合、描画しない
                    colorFlagEx = true;
                }
              }
            }
          }
        }
      }
      catch(ArrayIndexOutOfBoundsException e)
      { 
        if(matrixPatternNumEx>0)
          rgbEx = getRandom_rgbEx();
        repaint(); //表示エラー回復の試み。
      }
    }


    //●●色塗り図形描画命令「実行」グループの始まり

    //●自由図形の色塗り描画（色塗りを先、輪郭線は後）
    int comCnt = 0; //命令カウンター
    int mS = 0; //自由図形描画モード（線描きか色塗りかのモード）
    Color shapeColor = Color.yellow; //不透明色（ダミー）
    Color shapeColorT = new Color(200,200,0,0); //透明色（ダミー）
    for(; comCnt<shapeArr.length;)
    {
      if(shapeArr[comCnt][0] == 0)
      { //▼DrawShapeStart()　（起点での処理）
        gp = new GeneralPath();
        mS = (int)shapeArr[comCnt][1]; //線描きか色塗りかのモード
        gp.moveTo(shapeArr[comCnt][2]+sX,shapeArr[comCnt][3]+sY);
        shapeColor = new Color((int)shapeArr[comCnt][4],
          (int)shapeArr[comCnt][5],(int)shapeArr[comCnt][6]);
        shapeColorT = new Color((int)shapeArr[comCnt][4],
          (int)shapeArr[comCnt][5],(int)shapeArr[comCnt][6],191);
          //「191」は、透明度で「(256/4)*3-1」の計算
        if(markFlag == true) //▼起点十字線および丸印の描画
        {
          drawStartMark(shapeArr[comCnt][2], shapeArr[comCnt][3]); //十字線
          drawPointMark(shapeArr[comCnt][2], shapeArr[comCnt][3]); //丸印
        }
        comCnt++;
      }
      else if(shapeArr[comCnt][0] == -1)
      { //▼DrawShapeLine()　（中間での直線部処理）
        gp.lineTo(shapeArr[comCnt][1]+sX,shapeArr[comCnt][2]+sY);
        if(markFlag == true)
        {
          drawPointMark(shapeArr[comCnt][1], shapeArr[comCnt][2]); //丸印
        }
        comCnt++;
      }
      else if(shapeArr[comCnt][0] == -2)
      { //▼DrawShapeCurve()　（中間での曲線処理）
        gp.curveTo(
          shapeArr[comCnt][1]+sX,shapeArr[comCnt][2]+sY,
          shapeArr[comCnt][3]+sX,shapeArr[comCnt][4]+sY,
          shapeArr[comCnt][5]+sX,shapeArr[comCnt][6]+sY);
        //曲線部補助線描画
        if(markFlag == true)
        {
          g2.setColor(new Color(200,0,0));
          g2.setStroke(solidLineX);
          g2.draw(new Line2D.Double(
            shapeArr[comCnt][1]+sX,shapeArr[comCnt][2]+sY,
            shapeArr[comCnt][3]+sX,shapeArr[comCnt][4]+sY));
          drawPointMarkX(shapeArr[comCnt][1], shapeArr[comCnt][2]); //二重丸印
          drawPointMarkX(shapeArr[comCnt][3], shapeArr[comCnt][4]);

          g2.setStroke(solidLineX);
          g2.setColor(new Color(200,0,0));
          g2.draw(new Line2D.Double(
            shapeArr[comCnt][3]+sX,shapeArr[comCnt][4]+sY,
            shapeArr[comCnt][5]+sX,shapeArr[comCnt][6]+sY));
          drawPointMark(shapeArr[comCnt][5], shapeArr[comCnt][6]); //丸印

          g2.setStroke(solidLineX);
          g2.setColor(new Color(200,0,0));
          if(shapeArr[comCnt-1][0] == 0) 
          //DrawShapeCurveの直前がDrawShapeStartの場合
            g2.draw(new Line2D.Double(
              shapeArr[comCnt-1][2]+sX,shapeArr[comCnt-1][3]+sY,
              shapeArr[comCnt][1]+sX,shapeArr[comCnt][2]+sY));
          else if(shapeArr[comCnt-1][0] == -1) 
          //DrawShapeCurveの直前がDrawShapeLineの場合
            g2.draw(new Line2D.Double(
              shapeArr[comCnt-1][1]+sX,shapeArr[comCnt-1][2]+sY,
              shapeArr[comCnt][1]+sX,shapeArr[comCnt][2]+sY));
          else if(shapeArr[comCnt-1][0] == -2) 
          //DrawShapeCurveの直前がDrawShapeCurveの場合
            g2.draw(new Line2D.Double(
              shapeArr[comCnt-1][5]+sX,shapeArr[comCnt-1][6]+sY,
              shapeArr[comCnt][1]+sX,shapeArr[comCnt][2]+sY));
          else
            ; //命令文の記述ミス以外には有り得ないケース
          g2.setStroke(solidLine);
        }
        comCnt++;
      }
      else if(shapeArr[comCnt][0] == -3)
      { //▼DrawShapeEnd()　（終点での処理）
        gp.closePath();
        if(mS==2 || mS==3) //描画モード
        {
          g2.setColor(shapeColor);
          g2.fill(gp); //色塗り
        }
        if(mS==4 || mS == 5) //描画モード（透明色）
        {
          g2.setColor(shapeColorT);
          g2.fill(gp); //透明色塗り
        }
        if(mS==1 || mS==3 || mS==4)
        {
          g2.setColor(foregroundColorL);
          g2.draw(gp); //線描き（輪郭線）
        }
        comCnt++;
      }

    //●四角形の描画
      else if(shapeArr[comCnt][0] == -4)
      { //▼DrawRect()
        int mR   = (int)shapeArr[comCnt][1]; // 線描きか色塗りか
        double xR  = shapeArr[comCnt][2]; // x座標
        double yR  = shapeArr[comCnt][3]; // y座標
        double hrR = shapeArr[comCnt][4]; // 幅
        double vrR = shapeArr[comCnt][5]; // 高さ
        int crR  = (int)shapeArr[comCnt][6]; //color
        int cgR  = (int)shapeArr[comCnt][7]; //color
        int cbR  = (int)shapeArr[comCnt][8]; //color
        if(mR==2 || mR==3) //色塗り描画（不透明色）
        {
          g2.setColor(new Color(crR, cgR, cbR));
          g2.fill(new Rectangle2D.Double(xR,yR,hrR,vrR));
        }
        if(mR==4 || mR == 5) //色塗り描画（透明色）
        {
          g2.setColor(new Color(crR, cgR, cbR,191));
          g2.fill(new Rectangle2D.Double(xR,yR,hrR,vrR));
        }
        if(mR==1 || mR==3 || mR==4) //輪郭線描画
        {
          g2.setColor(foregroundColorL);
          g2.draw(new Rectangle2D.Double(xR,yR,hrR,vrR));
        }
        if(markFlag == true) //▼起点目印十字線の描画
        {
          drawPointMark(xR, yR);
          drawStartMark(xR, yR);
        }
        comCnt++;
      }

    //●円（楕円）の描画
      else if(shapeArr[comCnt][0] == -5)
      { //DrawEllipse()
        double mE  = shapeArr[comCnt][1]; // 線描か色塗りか
        double xE  = shapeArr[comCnt][2]; // x座標
        double yE  = shapeArr[comCnt][3]; // y座標
        double hrE = shapeArr[comCnt][4]; // 幅
        double vrE = shapeArr[comCnt][5]; // 高さ
        int crE  = (int)shapeArr[comCnt][6]; //color
        int cgE  = (int)shapeArr[comCnt][7]; //color
        int cbE  = (int)shapeArr[comCnt][8]; //color
        xE = xE-hrE;
        yE = yE-vrE;
        if(mE == 2 || mE == 3) 
        {//▼円または楕円の色塗り
          g2.setColor(new Color(crE, cgE, cbE));
          g2.fill(new Ellipse2D.Double(
          xE+sX, yE+sY, hrE*2.0, vrE*2.0));
            //x座標, y座標, 幅, 高さ
        }
        if(mE == 4 || mE == 5) 
        {//▼円または楕円の色塗り（透明色）
          g2.setColor(new Color(crE, cgE, cbE,191));
          g2.fill(new Ellipse2D.Double(
          xE+sX, yE+sY, hrE*2.0, vrE*2.0));
            //x座標, y座標, 幅, 高さ
        }
        if(mE == 1 || mE == 3 || mE==4) 
        {//▼円または楕円の輪郭線
          g2.setColor(foregroundColorL);
          g2.draw(new Ellipse2D.Double(
          xE+sX, yE+sY, hrE*2.0, vrE*2.0));
          //x座標, y座標, 幅, 高さ
        }
        if(markFlag == true) //▼起点目印十字線の描画
          drawStartMark(xE+hrE, yE+vrE);
        comCnt++;
      }
    }

    //●スプレーの描画
    //sprayDiscStatus_List; //スプレー命令のステータス
    //sprayDiscX_List; //スプレイ円盤状飛沫集団（円盤）の中心ｘ座標
    //sprayDiscY_List; //スプレイ円盤状飛沫集団（円盤）の中心ｙ座標
    //sprayUnitTP_List; //スプレイ飛沫の分布タイプ
    //sprayUnitSS_List; //スプレイ飛沫一個のサイズ（px）
    //sprayUnitSR_List; //スプレイ飛沫の円盤状拡散分布半径（px）
    //sprayUnitST_List; //スプレイ飛沫の分布個数
    //sprayUnitR_List; //スプレイ飛沫の赤成分
    //sprayUnitG_List; //スプレイ飛沫の緑成分
    //sprayUnitB_List; //スプレイ飛沫の青成分
    //sprayUnitA_List; //スプレイ飛沫の透明度
    if(sprayFlag==true)
    {
      int unitCnt = 0;
      for(int i=0; i<sprayDiscStatus_List.size(); i++)
      {
        ranSpray = new Random(randomSeed2+i);
        double[][] sprayPoint = new double[0][0];
        if(sprayUnitTP_List.get(unitCnt) == 0) //円筒型飛沫タイプ（フラット）
        {
          sprayPoint = spray0(sprayUnitSR_List.get(unitCnt), sprayUnitST_List.get(unitCnt));
        }
        else if(sprayUnitTP_List.get(unitCnt) == 1) //ベル型飛沫タイプ（正規分布）
        {
          sprayPoint = spray1(sprayUnitSR_List.get(unitCnt), sprayUnitST_List.get(unitCnt));
        }
        else if(sprayUnitTP_List.get(unitCnt) == 2) //円錐型飛沫タイプ（中央集中）
        {
          sprayPoint = spray2(sprayUnitSR_List.get(unitCnt), sprayUnitST_List.get(unitCnt));
        }

        g2.setColor(new Color(sprayUnitR_List.get(unitCnt), sprayUnitG_List.get(unitCnt), 
          sprayUnitB_List.get(unitCnt), sprayUnitA_List.get(unitCnt)));
        for(int j=0; j<sprayPoint.length; j++)
        {
          g2.fill(new Ellipse2D.Double(
            sprayDiscX_List.get(i) + sprayPoint[j][0]+sX, 
            sprayDiscY_List.get(i) + sprayPoint[j][1]+sY, 
            sprayUnitSS_List.get(unitCnt), 
            sprayUnitSS_List.get(unitCnt)));
            //x座標, y座標, 楕円の幅, 楕円の高さ
        }
        if(sprayDiscStatus_List.get(i) == 2) //SprayEnd
        {
          unitCnt++;
        }
      }
      if(markFlag == true) //スプレー円盤中心の目印描画
      {
        g2.setStroke(solidLineX);
        for(int i=0; i<sprayDiscStatus_List.size(); i++)
        {
          g2.setColor(Color.red);
          g2.draw(new Ellipse2D.Double(
            sprayDiscX_List.get(i) +sX -4, 
            sprayDiscY_List.get(i) +sY -4, 
            8, 8));
            //x座標, y座標, 楕円の幅, 楕円の高さ
          g2.setColor(Color.blue);
          g2.draw(new Line2D.Double(
            sprayDiscX_List.get(i)+sX+6, sprayDiscY_List.get(i)+sY,
            sprayDiscX_List.get(i)+sX-6, sprayDiscY_List.get(i)+sY));
          g2.draw(new Line2D.Double(
            sprayDiscX_List.get(i)+sX, sprayDiscY_List.get(i)+sY+6,
            sprayDiscX_List.get(i)+sX, sprayDiscY_List.get(i)+sY-6));
        }
      }
      g2.setColor(foregroundColorL);
    }

    //●●色塗り図形描画命令「実行」グループの終わり

    //●●特殊画像処理命令「実行」グループの始まり
    //●●各種「特殊描画命令」の実行位置は、この位置であることに意味あり。
    //●●Image命令（背景画像描画）の描画実行だけは、先行して実行済み

    //●ソフト・フォーカス処理済み画像の表示
    if(softFocusFName!=null && !softFocusFName.equals(""))
    {
      g2.drawImage(softFocusImage1, 0, 0, (int)imagePX_W, (int)imagePX_H, this);
    }
    //●ミックス・イメージ（二重像）処理済み画像の表示
    if(mixImage2!=null && !mixFName0.equals(""))
    { //描画する
      g2.drawImage(mixImage2, 0, 0, (int)imagePX_W, (int)imagePX_H, this);
    }
    //●図形内部彩色処理済み画像の表示
    if(changeColorInnerFlag==true && cciImage != null)
    { //描画する
      g2.drawImage(cciImage, 0, 0, (int)imagePX_W, (int)imagePX_H, this);
    }
    //●図形内部連続彩色処理済み画像の表示
    if(changeColorInnerMultiFlag==true && ccimImage != null)
    { //描画する
      g2.drawImage(ccimImage, 0, 0, (int)imagePX_W, (int)imagePX_H, this);
    }
    //●画像反転処理済み画像の表示
    if(reverseImage1!=null && !reverseImageFile.equals("") && 
      (reverseImageMode<=4 || reverseImageMode==6))
    { //描画する
      g2.drawImage(reverseImage1, 0, 0, (int)imagePX_W, (int)imagePX_H, this);
    }
    if(reverseImage1!=null && !reverseImageFile.equals("") && 
      (reverseImageMode==5 || reverseImageMode==7))
    { //90度、270度の回転処理画像を描画する（回転処理済みの画像保存を含む）
      g2.drawImage(reverseImage1, 0, 0, (int)imagePX_H, (int)imagePX_W, this);
    }
    //●画像多段回転処理済み画像の表示
    if(rotateMultiImage!=null && rotateMultiFlag==true)
    { //描画する
      g2.drawImage(rotateMultiImage, 0, 0, (int)imagePX_W, (int)imagePX_H, this);
    }
    //●単独色相変更処理済み画像の表示
    if(colorChangeImage1!=null && !colorChangeFile.equals("") && 
      (colorChangeMode==1 ||colorChangeMode==2))
    { //描画する
      g2.drawImage(colorChangeImage1, 0, 0, (int)imagePX_W, (int)imagePX_H, this);
    }
    //●アジャストＲＧＢ処理済み画像の表示
    if(adjustFName!=null && !adjustFName.equals(""))
    {
      g2.drawImage(adjustImage1, 0, 0, (int)imagePX_W, (int)imagePX_H, this);
    }
    //●階調加工済み画像の表示
    if(gradientImage1!=null && !gradientFName.equals(""))
    {
      g2.drawImage(gradientImage1, 0, 0, (int)imagePX_W, (int)imagePX_H, this);
    }
    //●連続色相変更処理済み画像の表示
    if(changeColorMultiBaseFlag==true && changeColorMultiFlag==true)
    { 
      g2.drawImage(ccmImage1, 0, 0, (int)imagePX_W, (int)imagePX_H, this);
    }
    //●カット・アンド・ペースト処理済み画像の表示
    if(f.canvas.cutAndPasteMaskFlag==true && f.canvas.cutAndPasteFlag==true)
    { 
      g2.drawImage(capImage1, 0, 0, (int)imagePX_W, (int)imagePX_H, this);
    }
    //●スタンプ画像（透明化処理済み）の上書き描画。
    if(stampFlag==true && stampListB!=null && stampListI!=null)
    {
      for(int i=0; i<stampListB.size(); i++)
      {
        BufferedImage transparentImage = stampListB.get(i);
        //画像サイズ計算
        int w = stampListB.get(i).getWidth(this);
        int h = stampListB.get(i).getHeight(this);
        int stampX = 0;
        int stampY = 0;
        int stampW = 0;
        int stampH = 0;
        boolean err = false;
        try
        {
          stampX = stampListI.get(i)[0];
          stampY = stampListI.get(i)[1];
          stampW = stampListI.get(i)[2];
          stampH = stampListI.get(i)[3];
        }
        catch (Exception e) 
        {
          f.message(e.toString());
          err = true;
        }
        if(transparentImage != null && err == false)
        { //描画する
          g2.drawImage(transparentImage, stampX, stampY, stampW, stampH, this);
        }
      }
    }
    
    //●●特殊画像処理命令「実行」グループの終わり

    //●●線および文字画像処理命令「実行」グループの始まり
    //●横平行線の描画（罫線または方眼線の描画に利用）
    if(parallelLineH_t > 0)
    {
      g2.setColor(foregroundColorL);
      if(parallelLineH_m == 0)
      	g2.setStroke(solidLineX); //実線に設定
      else if(parallelLineH_m == 1)
      	g2.setStroke(dotLine); //密の点線に設定
      else
      	g2.setStroke(dotLine2); //粗の点線に設定
      for(int i=0; i<parallelLineH_t; i++)
      { //描画実行（本来線）
        g2.draw(new Line2D.Double(
          parallelLineH_x+sX, 
          parallelLineH_y+parallelLineH_w*i+sY,
          parallelLineH_x+parallelLineH_l+sX, 
          parallelLineH_y+parallelLineH_w*i+sY));
      }
      for(int i=0; i<parallelLineH_t-1; i++)
      { //描画実行（中間線）
        g2.setStroke(dotLine2); //粗の点線に設定
        if(parallelLineH_h == 1)
        {
        	g2.draw(new Line2D.Double(
          	parallelLineH_x+sX, 
          	parallelLineH_y+parallelLineH_w*i+sY + (parallelLineH_w/2),
          	parallelLineH_x+parallelLineH_l+sX, 
          	parallelLineH_y+parallelLineH_w*i+sY + (parallelLineH_w/2)));
        }
      }
      g2.setStroke(solidLine); //太さ可変の実線に戻す
    }
    //●縦平行線の描画（罫線または方眼線の描画に利用）
    if(parallelLineV_t > 0)
    {
      g2.setColor(foregroundColorL);
      if(parallelLineV_m == 0)
      	g2.setStroke(solidLineX); //実線に設定
      else if(parallelLineV_m == 1)
      	g2.setStroke(dotLine); //密の点線に設定
      else
      	g2.setStroke(dotLine2); //粗の点線に設定
      for(int i=0; i<parallelLineV_t; i++)
      { //描画実行（本来線）
        g2.draw(new Line2D.Double(
          parallelLineV_x+parallelLineV_w*i+sX, 
          parallelLineV_y+sY,
          parallelLineV_x+parallelLineV_w*i+sX, 
          parallelLineV_y+parallelLineV_l+sY));
      }
      for(int i=0; i<parallelLineV_t-1; i++)
      { //描画実行（中間線）
        g2.setStroke(dotLine2); //粗の点線に設定
        if(parallelLineV_h == 1)
        {
        	g2.draw(new Line2D.Double(
          	parallelLineV_x+parallelLineV_w*i+sX + (parallelLineV_w/2), 
          	parallelLineV_y+sY,
          	parallelLineV_x+parallelLineV_w*i+sX + (parallelLineV_w/2), 
          	parallelLineV_y+parallelLineV_l+sY));
        }
      }
      g2.setStroke(solidLine); //太さ可変の実線に戻す
    }

    //●横平行線の描画（補助用）
    if(parallelLineH2_t > 0)
    {
      g2.setColor(foregroundColorL);
      if(parallelLineH2_m == 0)
      	g2.setStroke(solidLineX); //実線に設定
      else if(parallelLineH2_m == 1)
      	g2.setStroke(dotLine); //密の点線に設定
      else
      	g2.setStroke(dotLine2); //粗の点線に設定
      for(int i=0; i<parallelLineH2_t; i++)
      { //描画実行（本来線）
        g2.draw(new Line2D.Double(
          parallelLineH2_x+sX, 
          parallelLineH2_y+parallelLineH2_w*i+sY,
          parallelLineH2_x+parallelLineH2_l+sX, 
          parallelLineH2_y+parallelLineH2_w*i+sY));
      }
      for(int i=0; i<parallelLineH2_t-1; i++)
      { //描画実行（中間線）
        g2.setStroke(dotLine2); //粗の点線に設定
        if(parallelLineH2_h == 1)
        {
        	g2.draw(new Line2D.Double(
          	parallelLineH2_x+sX, 
          	parallelLineH2_y+parallelLineH2_w*i+sY + (parallelLineH2_w/2),
          	parallelLineH2_x+parallelLineH2_l+sX, 
          	parallelLineH2_y+parallelLineH2_w*i+sY + (parallelLineH2_w/2)));
        }
      }
      g2.setStroke(solidLine); //太さ可変の実線に戻す
    }
    //●縦平行線の描画（補助用）
    if(parallelLineV2_t > 0)
    {
      g2.setColor(foregroundColorL);
      if(parallelLineV2_m == 0)
      	g2.setStroke(solidLineX); //実線に設定
      else if(parallelLineV2_m == 1)
      	g2.setStroke(dotLine); //密の点線に設定
      else
      	g2.setStroke(dotLine2); //粗の点線に設定
      for(int i=0; i<parallelLineV2_t; i++)
      { //描画実行（本来線）
        g2.draw(new Line2D.Double(
          parallelLineV2_x+parallelLineV2_w*i+sX, 
          parallelLineV2_y+sY,
          parallelLineV2_x+parallelLineV2_w*i+sX, 
          parallelLineV2_y+parallelLineV2_l+sY));
      }
      for(int i=0; i<parallelLineV2_t-1; i++)
      { //描画実行（中間線）
        g2.setStroke(dotLine2); //粗の点線に設定
        if(parallelLineV2_h == 1)
        {
        	g2.draw(new Line2D.Double(
          	parallelLineV2_x+parallelLineV2_w*i+sX + (parallelLineV2_w/2), 
          	parallelLineV2_y+sY,
          	parallelLineV2_x+parallelLineV2_w*i+sX + (parallelLineV2_w/2), 
          	parallelLineV2_y+parallelLineV2_l+sY));
        }
      }
      g2.setStroke(solidLine); //太さ可変の実線に戻す
    }

    //●ベジェ曲線 Bezier Curve（ベジェは人名で自動車デザイナー）の描画
    for(int i=0; i<curveArr.length; i++)
    { //DrawCurve()
      g2.setColor(foregroundColorL);
      GeneralPath p = new GeneralPath();
      p.moveTo(curveArr[i][0]+sX,curveArr[i][1]+sY);
      p.curveTo(
        curveArr[i][2]+sX,curveArr[i][3]+sY,
        curveArr[i][4]+sX,curveArr[i][5]+sY,
        curveArr[i][6]+sX,curveArr[i][7]+sY);
      g2.draw(p);
      if(markFlag == true)
      { //▼補助線の描画
        g2.setColor(new Color(200,0,0));
      	g2.setStroke(solidLineX); //実線に設定
        g2.draw(new Line2D.Double(
          curveArr[i][0]+sX,curveArr[i][1]+sY,
          curveArr[i][2]+sX,curveArr[i][3]+sY));
        drawPointMark(curveArr[i][0], curveArr[i][1]); //丸印

        g2.setColor(new Color(200,0,0));
      	g2.setStroke(solidLineX); //実線に設定
        g2.draw(new Line2D.Double(
          curveArr[i][2]+sX,curveArr[i][3]+sY,
          curveArr[i][4]+sX,curveArr[i][5]+sY));
        drawPointMark(curveArr[i][2], curveArr[i][3]);
        drawPointMark(curveArr[i][4], curveArr[i][5]);

        g2.setColor(new Color(200,0,0));
      	g2.setStroke(solidLineX); //実線に設定
        g2.draw(new Line2D.Double(
          curveArr[i][4]+sX,curveArr[i][5]+sY,
          curveArr[i][6]+sX,curveArr[i][7]+sY));
        drawPointMark(curveArr[i][6], curveArr[i][7]);
        g2.setStroke(solidLine);
      }
      if(markFlag == true) //▼起点十字線の描画
        drawStartMark(curveArr[i][0], curveArr[i][1]);
    }
    //●スプライン曲線 Spline Curveの描画
    for(int i=0; splineArrX != null && i<splineArrX.length; i++)
    { //
      for(int j=1; splineArrX[i] != null && j<splineArrX[i].length; j++)
      {
        if(markFlag == true)
        { //▼補助線および目印の描画
          g2.setStroke(solidLineX); //太さ固定
          g2.setColor(new Color(200,0,0));
          g2.draw(new Line2D.Double(  //補助線
            splineArrX[i][j-1]+sX,splineArrY[i][j-1]+sY,
            splineArrX[i][j]+sX,splineArrY[i][j]+sY));
          if(j==1)
          {
            drawPointMark(splineArrX[i][j-1], splineArrY[i][j-1]);
            drawStartMark(splineArrX[i][j-1], splineArrY[i][j-1]);
          }
          drawPointMark(splineArrX[i][j], splineArrY[i][j]);
        }
        
        //（注）スプライン曲線は、短い直線を多数連結した「疑似曲線」として描画。
        //デフォルトは、1000本の直線に分割した「疑似曲線」。オプションで本数変更可能。
        //スプライン補間処理したデータによる「疑似曲線」描画
        double[][] interpolateXY = 
          new Spline3(splineArrX[i], splineArrY[i], splineOption).getInterpolateXY();
        g2.setColor(foregroundColorL);
        g2.setStroke(solidLine); //太さ可変
        for(int k=1; k<interpolateXY.length; k++)
        { //▼スプライン曲線の描画
          g2.draw(new Line2D.Double(
            interpolateXY[k-1][0]+sX,interpolateXY[k-1][1]+sY,
            interpolateXY[k][0]+sX,interpolateXY[k][1]+sY));
        }
      }
    }

    //●直線の描画
    for(int i=0; i<lineArr.length; i++)
    { //DrawLine()
      g2.setColor(foregroundColorL);
      g2.setStroke(solidLine); //太さ可変
      g2.draw(new Line2D.Double(
        lineArr[i][0]+sX,lineArr[i][1]+sY,
        lineArr[i][2]+sX,lineArr[i][3]+sY));
      if(markFlag == true) //目印描画
      {
        drawStartMark(lineArr[i][0], lineArr[i][1]); //十字線
        drawPointMark(lineArr[i][0], lineArr[i][1]); //丸印
        drawPointMark(lineArr[i][2], lineArr[i][3]); //丸印
        g2.setColor(foregroundColorL);
      }
    }
    //●仮直線（補助線）の描画
    for(int i=0; i<line2Arr.length; i++)
    { //Draw2()
      if(line2Arr[i][0]==0)
      { //▼直線部補助線（自由図形の直線部）
        g2.setColor(new Color(0,200,0));
        g2.setStroke(solidLineX); //太さ固定
        g2.draw(new Line2D.Double(
          line2Arr[i][1]+sX,line2Arr[i][2]+sY,
          line2Arr[i][3]+sX,line2Arr[i][4]+sY));
         g2.setStroke(solidLine);
      }
      else
      { //▼曲線部補助線（曲線、自由図形の曲線部）
        if(line2Arr[i][0]==1)
          g2.setColor(new Color(200,0,0));
        else if(line2Arr[i][0] == 2)
          g2.setColor(Color.red);
        else if(line2Arr[i][0] == 3)
          g2.setColor(Color.red);
        if(line2Arr[i][0]>=1 && line2Arr[i][0]<=3)
        {
          g2.setStroke(solidLineX);
          g2.draw(new Line2D.Double(
            line2Arr[i][1]+sX,line2Arr[i][2]+sY,
            line2Arr[i][3]+sX,line2Arr[i][4]+sY));
          g2.setStroke(solidLine);
        }
      }
    }

    //●折れ線の描画
    if(linkedLineFlag==true)
    {
      int unitCnt = 0;
      for(int i=0; i<linkedLineStatusList.size(); i++)
      {
        if(linkedLineStatusList.get(i) == 1 || linkedLineStatusList.get(i) == 2)
        {
          g2.setColor(Color.black);
          g2.draw(new Line2D.Double(
            linkedLineXList.get(i-1)+sX, linkedLineYList.get(i-1)+sY,
            linkedLineXList.get(i)+sX,   linkedLineYList.get(i)+sY));
        }
      }
      if(markFlag == true) //折れ線の目印描画
      {
        for(int i=0; i<linkedLineStatusList.size(); i++)
        {
          if(linkedLineStatusList.get(i) == 0)
            drawStartMark(linkedLineXList.get(i), linkedLineYList.get(i)); //十字線
          drawPointMark(linkedLineXList.get(i), linkedLineYList.get(i)); //丸印
        }
      }
      g2.setColor(foregroundColorL);
    }

    //●極太水平文字列（字）の描画
    for(int i=0; i<stringBArrD.length; i++)
    { //WriteStringB()
      //字形は二種類あり（"SANS_SERIF" / "SERIF"）
      g2.setColor(foregroundColorB);
      int z = 1;
      int Z = 2;
      Font fontT = new Font(stringBArrS[i][0],Font.PLAIN,
      (int)(stringBArrD[i][2])); 
      g2.setFont(fontT);
      g2.drawString(
        stringBArrS[i][1],(float)(stringBArrD[i][0]+sX),
        (float)(stringBArrD[i][1]+sY));

      g2.drawString(
        stringBArrS[i][1],(float)(stringBArrD[i][0]+sX),
        (float)(stringBArrD[i][1]+sY+Z));
      g2.drawString(
        stringBArrS[i][1],(float)(stringBArrD[i][0]+sX),
        (float)(stringBArrD[i][1]+sY-Z));
      g2.drawString(
        stringBArrS[i][1],(float)(stringBArrD[i][0]+sX+Z),
        (float)(stringBArrD[i][1]+sY));
      g2.drawString(
        stringBArrS[i][1],(float)(stringBArrD[i][0]+sX-Z),
        (float)(stringBArrD[i][1]+sY));
      g2.drawString(
        stringBArrS[i][1],(float)(stringBArrD[i][0]+sX+Z),
        (float)(stringBArrD[i][1]+sY+Z));
      g2.drawString(
        stringBArrS[i][1],(float)(stringBArrD[i][0]+sX-Z),
        (float)(stringBArrD[i][1]+sY-Z));
      g2.drawString(
        stringBArrS[i][1],(float)(stringBArrD[i][0]+sX+Z),
        (float)(stringBArrD[i][1]+sY-Z));
      g2.drawString(
        stringBArrS[i][1],(float)(stringBArrD[i][0]+sX-Z),
        (float)(stringBArrD[i][1]+sY+Z));

      g2.drawString(
        stringBArrS[i][1],(float)(stringBArrD[i][0]+sX+z),
        (float)(stringBArrD[i][1]+sY+Z));
      g2.drawString(
        stringBArrS[i][1],(float)(stringBArrD[i][0]+sX-z),
        (float)(stringBArrD[i][1]+sY-Z));
      g2.drawString(
        stringBArrS[i][1],(float)(stringBArrD[i][0]+sX+z),
        (float)(stringBArrD[i][1]+sY-Z));
      g2.drawString(
        stringBArrS[i][1],(float)(stringBArrD[i][0]+sX-z),
        (float)(stringBArrD[i][1]+sY+Z));

      g2.drawString(
        stringBArrS[i][1],(float)(stringBArrD[i][0]+sX+Z),
        (float)(stringBArrD[i][1]+sY+z));
      g2.drawString(
        stringBArrS[i][1],(float)(stringBArrD[i][0]+sX-Z),
        (float)(stringBArrD[i][1]+sY-z));
      g2.drawString(
        stringBArrS[i][1],(float)(stringBArrD[i][0]+sX+Z),
        (float)(stringBArrD[i][1]+sY-z));
      g2.drawString(
        stringBArrS[i][1],(float)(stringBArrD[i][0]+sX-Z),
        (float)(stringBArrD[i][1]+sY+z));

      if(markFlag == true)
      { //▼起点目印十字線の描画
        drawStartMark(stringBArrD[i][0], stringBArrD[i][1]);
      }
    }

    //●水平文字列の描画（極太文字より後に描画）
    for(int i=0; i<stringHArrD.length; i++)
    { //WriteStringH()
      //字形は二種類あり（"SANS_SERIF" / "SERIF"）
      g2.setColor(foregroundColorS);
      Font fontT = new Font(stringHArrS[i][0],Font.PLAIN,
      (int)(stringHArrD[i][2]));
      if(fontBold==true)
      {
        fontT = new Font(stringHArrS[i][0],Font.BOLD,
        (int)(stringHArrD[i][2]));
      }
      g2.setFont(fontT);
      g2.drawString(
        stringHArrS[i][1],(float)(stringHArrD[i][0]+sX),
        (float)(stringHArrD[i][1]+sY));
      if(markFlag == true) //▼起点目印十字線の描画
        drawStartMark(stringHArrD[i][0], stringHArrD[i][1]);
    }

    //●垂直文字列の描画（90度回転式の疑似縦書き）
    //「疑似縦書き」を実行すると、画像全体の精細度が低下するため、
    //この命令（WriteStringV）を使用しないことが望ましい。
    for(int i=0; i<stringVArrD.length; i++)
    {  //WriteStringV()
      //文字列を垂直（90度回転）に描画する
      Dimension dim;  //縦横サイズを保持するインスタンス
      BufferedImage bImg;  //一時的描画空間
      dim = getSize(); //最終描画先のf.canvasのサイズを取得
      bImg = new BufferedImage(dim.width, dim.height,
        BufferedImage.TYPE_INT_ARGB);   //透明可に設定
      Graphics2D g2x = bImg.createGraphics();  //Graphics2Dを返却
      g2x.setColor(new Color(0,0,0,0));  //背景は無色透明で
      g2x.fillRect(0, 0, dim.width, dim.height);
      g2x.rotate(1.57,  //90度回転実施（ラディアン単位で角度指定）
        (float)(stringVArrD[i][0]+sX), (float)(stringVArrD[i][1]+sY));
      g2x.setColor(foregroundColorL);  //文字は不透明色
      Font fontT = new Font(stringVArrS[i][0],Font.PLAIN,
      (int)(stringVArrD[i][2]));
      g2x.setFont(fontT);
      g2x.drawString(  //文字列描画
        stringVArrS[i][1],(float)(stringVArrD[i][0]+sX),
        (float)(stringVArrD[i][1]+sY));
      //f.canvasにbImgを無色透明色の背景で転写描画する
      g2.drawImage(bImg, 0, 0, new Color(0,0,0,0), null);
      if(markFlag == true) //▼起点目印十字線の描画
        drawStartMark(stringVArrD[i][0], stringVArrD[i][1]);
    }

    //●プレビューの場合だけ描画有効範囲枠線を表示。
    //●二点鎖線十字線および三角形目印。
    //●印刷時や画像保存時には、これらを消す。
    if(printMode==false && canSaveMode==false)
    { 
      g2.setColor(Color.red); //二点鎖線十字線
      g2.setStroke(dashLine);
      int z = (int)(imagePX_W/20);
      g2.draw(new Line2D.Double(5, imagePX_H, 
        imagePX_W-5, imagePX_H));
      g2.draw(new Line2D.Double(imagePX_W, 5, 
        imagePX_W, imagePX_H-5));
      g2.draw(new Line2D.Double(imagePX_W+5, imagePX_H, 
        imagePX_W*2, imagePX_H));
      g2.draw(new Line2D.Double(imagePX_W, imagePX_H+5, 
        imagePX_W, imagePX_H*2));

      g2.setColor(Color.black); //三角形目印
      g2.setStroke(solidLineX);
      GeneralPath gp1 = new GeneralPath();
      gp1.moveTo(imagePX_W,imagePX_H/2);
      gp1.lineTo(imagePX_W+z,imagePX_H/2-z);
      gp1.lineTo(imagePX_W+z,imagePX_H/2+z);
      gp1.closePath();
      g2.fill(gp1); 
      GeneralPath gp2 = new GeneralPath();
      gp2.moveTo(imagePX_W/2,imagePX_H);
      gp2.lineTo(imagePX_W/2-z,imagePX_H+z);
      gp2.lineTo(imagePX_W/2+z,imagePX_H+z);
      gp2.closePath();
      g2.fill(gp2);
    }

    //●ピクセル数とミリ数とを、メニュー・バーに表示
    int pw = ((int)(imagePX_W*affineScale*freeScale)); 
      //pw : pixelを切り出す範囲の巾
    int ph = ((int)(imagePX_H*affineScale*freeScale)); 
      //ph : pixelを切り出す範囲の丈
    f.pixelLabel.setText("   (i:"+pw+"x"+ ph+"px)");
    f.mmLabel.setText("   (p:"+(int)(printMM_W*freeScale)+"x"+
      (int)(printMM_H*freeScale)+"mm)");

    //●現在点（最新クリック点）の現在点目印Ｘ字線の描画
    
    if(markFlag == true && currentPointX != 0 && currentPointY != 0 && 
      f.cMouseLis.canvasMouseMode != 0)
    {
      g2.setColor(Color.red);
      g2.draw(new Line2D.Double(
        currentPointX+sX+5, currentPointY+sY-5,
        currentPointX+sX-5, currentPointY+sY+5));
      g2.setColor(Color.blue);
      g2.draw(new Line2D.Double(
        currentPointX+sX+5, currentPointY+sY+5,
        currentPointX+sX-5, currentPointY+sY-5));
    }
    //●●線および文字画像処理命令「実行」グループの終わり
  } //drawPage()の終了


  //■■以下は各種メソッド

  //■背景画像表示の準備
  void image()
  {
    imageListB = new LinkedList<BufferedImage>(); //この位置が重要
    for(int i=0; i<imageListF.size(); i++)
    {
      int imageX = imageListI.get(i)[0];
      int imageY = imageListI.get(i)[1];
      int imageW = imageListI.get(i)[2];
      int imageH = imageListI.get(i)[3];
      //●画像ファイル指定のチェック
      String path = imageListF.get(i).toLowerCase();
      if(path.endsWith(".jpeg") ||path.endsWith(".jpg") ||
        path.endsWith(".gif") ||path.endsWith(".png") ||
        path.endsWith(".bmp"))
      {
        ; //OK
      }
      else
      {
        f.message("背景画像ファイル指定が不適切："+path);
        return;
      }
      if(! new File(path).exists())
      {
        f.message("背景画像ファイルが存在しません："+path);
        return;
      }
      BufferedImage imageImage; //背景のピクセル画像
      try
      {
        imageImage = ImageIO.read(new File(path)); //背景のピクセル画像取得
      } 
      catch (Exception e) 
      {
        e.printStackTrace();
        f.message("背景画像ファイル指定が不適切："+path);
        return;
      }
      imageListB.add(imageImage); //背景画像をリストに追加
    }
  }

  //■スタンプ押印の準備
  void stamp()
  {
    stampListB = new LinkedList<BufferedImage>(); //この位置が重要
    for(int i=0; i<stampListF.size(); i++)
    {
      int stampX = stampListI.get(i)[0];
      int stampY = stampListI.get(i)[1];
      int stampW = stampListI.get(i)[2];
      int stampH = stampListI.get(i)[3];
      //●画像ファイル指定のチェック
      String path = stampListF.get(i).toLowerCase();
      if(path.endsWith(".jpeg") ||path.endsWith(".jpg") ||
        path.endsWith(".gif") ||path.endsWith(".png") ||
        path.endsWith(".bmp"))
      {
        ; //OK
      }
      else
      {
        f.message("スタンプ画像ファイル指定が不適切："+path);
        return;
      }
      if(! new File(path).exists())
      {
        f.message("スタンプ画像ファイルが存在しません："+path);
        return;
      }
      BufferedImage stampImage; //ピクセル画像
      try
      {
        stampImage = ImageIO.read(new File(path)); //スタンプのピクセル画像取得
      } 
      catch (Exception e) 
      {
        e.printStackTrace();
        f.message("スタンプ画像ファイル指定が不適切："+path);
        return;
      }

      // 画像データのカラー・モデルを確認して表示（デバッグに使用）
/*
      int colorType = stampImage.getType();
      if(colorType == BufferedImage.TYPE_3BYTE_BGR)
        f.message("不透明タイプ：TYPE_3BYTE_BGR >> " + colorType);
      else if(colorType == BufferedImage.TYPE_4BYTE_ABGR)
        f.message("透明可タイプ：TYPE_4BYTE_ABGR >> " + colorType);
      else if(colorType == BufferedImage.TYPE_4BYTE_ABGR_PRE)
        f.message("透明可タイプ：TYPE_4BYTE_ABGR_PRE >> " + colorType);
      else if(colorType == BufferedImage.TYPE_BYTE_BINARY)
        f.message("不透明タイプ：TYPE_BYTE_BINARY >> " + colorType);
      else if(colorType == BufferedImage.TYPE_BYTE_GRAY)
        f.message("灰色タイプ：TYPE_BYTE_GRAY >> " + colorType);
      else if(colorType == BufferedImage.TYPE_BYTE_INDEXED)
        f.message("特殊タイプ：TYPE_BYTE_INDEXED >> " + colorType);
      else if(colorType == BufferedImage.TYPE_CUSTOM)
        f.message("特殊タイプ：TYPE_CUSTOM >> " + colorType);
      else if(colorType == BufferedImage.TYPE_INT_ARGB)
        f.message("透明可タイプ：TYPE_INT_ARGB >> " + colorType);
      else if(colorType == BufferedImage.TYPE_INT_ARGB_PRE)
        f.message("透明可タイプ：TYPE_INT_ARGB_PRE >> " + colorType);
      else if(colorType == BufferedImage.TYPE_INT_BGR)
        f.message("不透明タイプ：TYPE_INT_BGR >> " + colorType);
      else if(colorType == BufferedImage.TYPE_INT_RGB)
        f.message("不透明タイプ：TYPE_INT_RGB >> " + colorType);
      else if(colorType == BufferedImage.TYPE_USHORT_555_RGB)
        f.message("不透明タイプ：TYPE_USHORT_555_RGB >> " + colorType);
      else if(colorType == BufferedImage.TYPE_USHORT_565_RGB)
        f.message("不透明タイプ：TYPE_USHORT_565_RGB >> " + colorType);
      else if(colorType == BufferedImage.TYPE_USHORT_GRAY)
        f.message("灰色タイプ：TYPE_USHORT_GRAY >> " + colorType);
      else
        f.message("カラー・モデル：不明" );
*/
      //スタンプの背景色（灰色）を無色透明に変換
      int w = stampImage.getWidth(this);
      int h = stampImage.getHeight(this);
      BufferedImage transparentImage = new BufferedImage(
        w, h, BufferedImage.TYPE_INT_ARGB); //透明可に設定
      //f.message("transparentColor=" + Integer.toHexString(transparentColor.getRGB()));
      for(int y=0;y<h;y++){
        for(int x=0;x<w;x++){
          if (stampImage.getRGB(x,y) == transparentColor.getRGB())
          { //特定位置で灰色一致なら３２ビットの無色透明色に変換
            transparentImage.setRGB(x,y,0x00000000);
          } 
          else 
          { //それ以外なら無変換で転写
            transparentImage.setRGB(x, y, stampImage.getRGB(x, y));
          }
        }
      }
      //無色透明化処理は、大抵の場合成功するが、失敗する場合もある。
      //１．スタンプ画像作成での、アンチエイリアシングの有無。
      //２．画像保存ファイル形式（gif、jpeg、bmp、png）。
      //これらが原因で、無色透明化に失敗する場合もある。
      //スタンプ画像（グレー・マスク画像）をこのGCreatorで作成し保存した場合、
      //簡易実験では、読み込み時、次の３種類のカラー・モデルに分かれた。
      //TYPE_BYTE_INDEXED →  透明化に成功または失敗（結果が不安定）
      //TYPE_3BYTE_BGR  →  透明化に成功
      //TYPE_BYTE_BINARY  →  透明化に成功
      //１．アンチエイリアシング（なめらか処理）を行なった時、失敗しやすい。
      //２．ファイル形式としては、「gif」が透明化に失敗しやすく、
      //「jpeg」は、透明化できても画像の乱れが激しく適切でなく、
      //「png」や「bmp」は、成功しやすく適切であるように思われる。
      //（結論）スタンプ画像は、png形式で保存すると、経験的に失敗が全くない。

      stampListB.add(transparentImage); //透明化したスタンプ画像をリストに追加
    }
  }

  //■図形の起点十字線の描画（赤線と青線の十字線）
  //直線、四角形、真円、楕円、自由図形などの起点表示に必要
  void drawStartMark(double x, double y)
  {
    BasicStroke solidLine = new BasicStroke( //太さ可変の実線
      lineWidth, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_MITER);
    BasicStroke solidLineX = new BasicStroke( //太さ固定の実線
      0.2f, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_MITER);
    g2.setStroke(solidLineX);
    g2.setColor(Color.red);
    g2.draw(new Line2D.Double(
      x+shiftX+5, y+shiftY,x+shiftX-5, y+shiftY));
    g2.setColor(Color.blue);
    g2.draw(new Line2D.Double(
      x+shiftX, y+shiftY+5,x+shiftX, y+shiftY-5));
    g2.setStroke(solidLine); //線規格を元に戻す
  }

  //■線の起点中間点終点の描画（真円の目印）
  void drawPointMark(double x, double y) //丸印の目印
  {
    BasicStroke solidLine = new BasicStroke( //太さ可変の実線
      lineWidth, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_MITER);
    BasicStroke solidLineX = new BasicStroke( //太さ固定の実線
      0.2f, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_MITER);
    g2.setStroke(solidLineX);
    g2.setColor(Color.red);
    g2.draw(new Ellipse2D.Double(
      x +shiftX -4, y +shiftX -4, 8, 8));
      //x座標, y座標, 楕円の幅, 楕円の高さ
    g2.setStroke(solidLine); //線規格を元に戻す
  }
  
  void drawPointMarkX(double x, double y) //二重丸の目印
  {  //自由図形の曲線部の制御点表示に必要
    BasicStroke solidLine = new BasicStroke( //太さ可変の実線
      lineWidth, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_MITER);
    BasicStroke solidLineX = new BasicStroke( //太さ固定の実線
      0.2f, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_MITER);
    g2.setStroke(solidLineX);
    g2.setColor(Color.red);
    g2.draw(new Ellipse2D.Double(
      x +shiftX -3, y +shiftX -3, 6, 6));
    g2.draw(new Ellipse2D.Double(
      x +shiftX -5, y +shiftX -5, 10, 10));
      //x座標, y座標, 楕円の幅, 楕円の高さ
    g2.setStroke(solidLine); //線規格を元に戻す
  }

  //■マトリックス柄紋のランダム配色決定（統一配色）
  short[][] getRandom_rgbSt()
  {  //色番号は、０から９までで、０だけ無色透明扱い
      short[][] random_rgbSt = new short[9][3];
      if(matrixPRedBaseSt > 240) matrixPRedBaseSt = 240; //明度上限設定
      if(matrixPGreenBaseSt > 240) matrixPGreenBaseSt = 240;
      if(matrixPBlueBaseSt > 240) matrixPBlueBaseSt = 240;
      for(int i=0; i<9; i++) //ランダム配色
      {
        random_rgbSt[i][0] = (short)(matrixPRedBaseSt + 
          (ran.nextInt(255-matrixPRedBaseSt))*matrixColorLimitSt/10);
        random_rgbSt[i][1] = (short)(matrixPGreenBaseSt + 
          (ran.nextInt(255-matrixPGreenBaseSt))*matrixColorLimitSt/10);
        random_rgbSt[i][2] = (short)(matrixPBlueBaseSt + 
          (ran.nextInt(255-matrixPBlueBaseSt))*matrixColorLimitSt/10);
      }
      return random_rgbSt;
  }

  //■マトリックス柄紋のランダム配色決定（個別配色）
  short[][][][][][] getRandom_rgbEx()
  {
    //このメソッドは、巨大配列を使うためヒープ領域不足になる可能性もあるので、
    //javaの実行時には、メモリー上のヒープ領域を増やすために、
    //以下の参考例を真似て数倍し、各自が最適値を考え、実行時オプションを加えること。
    //java -Xmx1024m GCreator
    //また、メモリーを多めに（Win11で8GB以上）、PCに搭載することが望ましい。
    //Win11で4GBの条件でも動作可能だが、常駐ソフトを最大限に無効化する必要があり、
    //仮想メモリーの設定も必要になる。
    //模様を複雑（ユニット巨大）かつ精細（コマ数微小）かつ巨大（全体サイズ巨大）
    //に指定すれば、ヒープ領域不足になる可能性がある。
    //メモリーが8GB以上あれば、問題は殆ど生じない。

    short[][][][][][] random_rgbEx = 
      new short[matrixRepeatH_Ex][matrixRepeatV_Ex][matrixUnitEx][matrixUnitEx][9][3];
    if(matrixPRedBaseEx > 250) matrixPRedBaseEx = 250; //明度上限設定
    if(matrixPGreenBaseEx > 250) matrixPGreenBaseEx = 250;
    if(matrixPBlueBaseEx > 250) matrixPBlueBaseEx = 250;
    //ランダム配色
    short color[][]= new short [9][3]; //9色のランダム色を一時格納
    //返却配列に格納
    for(int repH=0; repH<matrixRepeatH_Ex; repH++) //リピート数、横
    {
      for(int repV=0; repV<matrixRepeatV_Ex; repV++) //リピート数、縦
      {
        for(int h=0; h<matrixUnitEx; h++) //横コマ数
        {
            if(h==0)
            {
              for(int a=0; a<9; a++) //９色
              {
                  color[a][0] = (short)(matrixPRedBaseEx + 
                    (ran.nextInt(255-matrixPRedBaseEx))*matrixColorLimitEx/10);
                  color[a][1] = (short)(matrixPGreenBaseEx + 
                    (ran.nextInt(255-matrixPGreenBaseEx))*matrixColorLimitEx/10);
                  color[a][2] = (short)(matrixPBlueBaseEx + 
                    (ran.nextInt(255-matrixPBlueBaseEx))*matrixColorLimitEx/10);
              }
            }
          for(int v=0; v<matrixUnitEx; v++) //縦コマ数
          {
            for(int i=0; i<9; i++) //割り当てる配色決定（９色）
            {
              random_rgbEx[repH][repV][h][v][i][0] = (short)color[i][0];
              random_rgbEx[repH][repV][h][v][i][1] = (short)color[i][1];
              random_rgbEx[repH][repV][h][v][i][2] = (short)color[i][2];
            }
          }
        }
      }
    }
    return random_rgbEx;
  }

  //■マトリックス柄紋の柄紋番号での文様データ検索取得
  int[][] getPatternData(int patternNum, String colorType)
  {
    int patternUnit;
    int matrixPattern[][] = null;

    for(int i=0; i<patternArr.length; i++)
    {
      if((patternArr[i].startsWith("PTN")) && 
        (patternNum == Integer.parseInt(patternArr[i].substring(4,8))))
      {
        patternUnit = Integer.parseInt(patternArr[i].substring(9,11));
        if(colorType.equals("St"))
        {
          matrixUnitSt = patternUnit;
        }
        if(colorType.equals("Ex"))
        {
          matrixUnitEx = patternUnit;
        }
        matrixPattern = new int[patternUnit][patternUnit];
        for(int j=0; j<patternUnit; j++)
        {
          String[] s = Util.csv2Array(patternArr[i+j+1], patternUnit);
          for(int k=0; k<patternUnit; k++)
          {
            matrixPattern[j][k] = Integer.parseInt(s[k]);
          }
        }
        //if(patternNum != 0) f.message(""+patternArr[i]);
        break;
      }
      else
        continue;
    }
    if(matrixPattern == null) //柄紋検索エラー
    {
      matrixPattern = new int[][]{ 
        {0,1,1,1,1,0},
        {0,1,0,0,0,0},
        {0,1,1,1,1,0},
        {0,1,0,0,0,0},
        {0,1,1,1,1,0},
        {0,0,0,0,0,0}};
      if(colorType.equals("St"))
        matrixUnitSt = 6;
      if(colorType.equals("Ex"))
        matrixUnitEx = 6;
    }
    return matrixPattern;
  }

  //■ソフト・フォーカス効果
  void softFocus()
  {
    //●画像ファイル指定のチェック
    if(!softFocusFName.equals("") && softFocusFName != null)
    {
      String path0 = softFocusFName.toLowerCase();
      if(path0.endsWith(".jpeg") || path0.endsWith(".jpg") ||
        path0.endsWith(".gif") || path0.endsWith(".png") ||
        path0.endsWith(".bmp"))
        ; //OK
      else
      {
        f.message("ソフト・フォーカス対象ファイルが不適切です："+path0);
        return;
      }
      if(! new File(path0).exists())
      {
        f.message("ソフト・フォーカス対象ファイルが存在しません："+path0);
        return;
      }
      softFocusImage0 = new BufferedImage(
        (int)imagePX_W, (int)imagePX_H, BufferedImage.TYPE_INT_RGB); //不透明に設定
      softFocusImage1 = new BufferedImage(
        (int)imagePX_W, (int)imagePX_H, BufferedImage.TYPE_INT_RGB); //不透明に設定
      try
      {
        softFocusImage0 = ImageIO.read(new File(path0)); //画像読み取り
      } 
      catch (Exception e) 
      {
        e.printStackTrace();
        f.message("ソフト・フォーカス対象ファイルのリード・エラー："+path0);
        softFocusImage0 = null;
        return;
      }
      //ソフト・フォーカス加工
      int w = softFocusImage0.getWidth(this);
      int h = softFocusImage0.getHeight(this);
      int m,m1,m2,m3,m4,m5,m6,m7,m8;
      int R,G,B;
      int R1,G1,B1,R2,G2,B2,R3,G3,B3,R4,G4,B4;
      int R5,G5,B5,R6,G6,B6,R7,G7,B7,R8,G8,B8;
      for(int y=0; y<h; y++)
      {
        for(int x=0; x<w; x++)
        {
            m = softFocusImage0.getRGB(x,y);
            R = (m & 0x00ff0000)>>>16;
            G = (m & 0x0000ff00)>>> 8;
            B = (m & 0x000000ff);
            int color =  (R<<16) | (G<<8) | B; 
            softFocusImage1.setRGB(x, y, color);
        }
      }
      for(int z=0; z<softFocusDegree; z++) //ソフト度合いの累積処理
      {
        softFocusImage0 = softFocusImage1;
        for(int y=1; y<h-2; y++)
        {
          for(int x=1; x<w-2; x++)
          {
              m = softFocusImage0.getRGB(x,y);
              R = (m & 0x00ff0000)>>>16;
              G = (m & 0x0000ff00)>>> 8;
              B = (m & 0x000000ff);

              m1 = softFocusImage0.getRGB(x-1,y);
              R1 = (m1 & 0x00ff0000)>>>16;
              G1 = (m1 & 0x0000ff00)>>> 8;
              B1 = (m1 & 0x000000ff);
              m2 = softFocusImage0.getRGB(x,y-1);
              R2 = (m2 & 0x00ff0000)>>>16;
              G2 = (m2 & 0x0000ff00)>>> 8;
              B2 = (m2 & 0x000000ff);
              m3 = softFocusImage0.getRGB(x+1,y);
              R3 = (m3 & 0x00ff0000)>>>16;
              G3 = (m3 & 0x0000ff00)>>> 8;
              B3 = (m3 & 0x000000ff);
              m4 = softFocusImage0.getRGB(x,y+1);
              R4 = (m4 & 0x00ff0000)>>>16;
              G4 = (m4 & 0x0000ff00)>>> 8;
              B4 = (m4 & 0x000000ff);

              m5 = softFocusImage0.getRGB(x-1,y-1);
              R5 = (m5 & 0x00ff0000)>>>16;
              G5 = (m5 & 0x0000ff00)>>> 8;
              B5 = (m5 & 0x000000ff);
              m6 = softFocusImage0.getRGB(x+1,y+1);
              R6 = (m6 & 0x00ff0000)>>>16;
              G6 = (m6 & 0x0000ff00)>>> 8;
              B6 = (m6 & 0x000000ff);
              m7 = softFocusImage0.getRGB(x+1,y-1);
              R7 = (m7 & 0x00ff0000)>>>16;
              G7 = (m7 & 0x0000ff00)>>> 8;
              B7 = (m7 & 0x000000ff);
              m8 = softFocusImage0.getRGB(x-1,y+1);
              R8 = (m8 & 0x00ff0000)>>>16;
              G8 = (m8 & 0x0000ff00)>>> 8;
              B8 = (m8 & 0x000000ff);

              double Rd = (R+R1+R2+R3+R4+R5+R6+R7+R8)/9.0D;
              double Gd = (G+G1+G2+G3+G4+G5+G6+G7+G8)/9.0D;
              double Bd = (B+B1+B2+B3+B4+B5+B6+B7+B8)/9.0D;
              int Rx = (int)(Rd+0.5); //計算端数誤差を調整して整数化し、
              int Gx = (int)(Gd+0.5); //ソフト・フォーカス処理を正常化させる。
              int Bx = (int)(Bd+0.5); //
              //この疑似ソフト・フォーカス処理は、各ピクセルの隣接八方向の色相しか
              //計算に入れないので、全体的にぼやけた八角形の形状に、ぼかされる。
              //レンズによる全方向の真円形ソフト・フォーカスとは、微妙に違う。

              int color =  (Rx<<16) | (Gx<<8) | Bx;
              softFocusImage1.setRGB(x, y, color);
          }
        }
      }
    }
    //ソフト・フォーカス画像の描画自体は、drawPage()の中で行なう。
  }

  //■切り貼り画像処理
  void cutAndPaste()
  {
    //●切り貼り画像（外部からの読み込み画像）の上書き描画
    if(capMaskF != null && !capMaskF.equals("")) //マルチ・マスク・ファイル
    {
      //画像ファイル指定のチェック
      String path0 = capMaskF.toLowerCase();
      if(path0.endsWith(".jpeg") ||path0.endsWith(".jpg") ||
        path0.endsWith(".gif") ||path0.endsWith(".png") ||
        path0.endsWith(".bmp"))
        ; //OK
      else
      {
        f.message("マルチ・マスク・ファイルが不適切："+path0);
        return;
      }
      if(! new File(path0).exists())
      {
        f.message("マルチ・マスク・ファイルが存在しません："+path0);
        return;
      }

      int w = (int)imagePX_W;
      int h = (int)imagePX_H;
      try
      {
        capImageM = ImageIO.read(new File(path0)); //画像の読み込み
      } 
      catch (Exception e) 
      {
        e.printStackTrace();
        f.message("マルチ・マスク・ファイルのリード・エラー："+path0);
        capImageM = null;
        return;
      }
    }
    
    capImage0 = new BufferedImage( //切り貼り元模様画像（不透明）
      (int)imagePX_W, (int)imagePX_H, BufferedImage.TYPE_INT_RGB);
    capImage1 = new BufferedImage( //切り貼り後の画像（透明可）
      (int)imagePX_W, (int)imagePX_H, BufferedImage.TYPE_INT_ARGB);
    for(int i=0; i<capListF.size(); i++) //
    {
      //画像ファイル指定のチェック
      String path1 = capListF.get(i).toLowerCase();
      int maskR = capListI.get(i)[0];
      int maskG = capListI.get(i)[1];
      int maskB = capListI.get(i)[2];
      int maskRGB = (maskR<<16) | (maskG<<8) | maskB;

      if(path1.endsWith(".jpeg") ||path1.endsWith(".jpg") ||
        path1.endsWith(".gif") ||path1.endsWith(".png") ||
        path1.endsWith(".bmp"))
        ; //OK
      else
      {
        f.message("切り貼り元画像ファイル指定が不適切："+path1);
        return;
      }
      if(! new File(path1).exists())
      {
        f.message("切り貼り元画像ファイルが存在しません："+path1);
        return;
      }
      try
      {
        capImage0 = ImageIO.read(new File(path1));
      } 
      catch (Exception e) 
      {
        e.printStackTrace();
        f.message("切り貼り元画像ファイルのリード・エラー："+path1);
        return;
      }

      //
      for(int y=0;y<imagePX_H;y++)
      {
        for(int x=0;x<imagePX_W;x++)
        {
          if ((0x00ffffff & capImageM.getRGB(x,y)) == maskRGB)
          { //
            capImage1.setRGB(x, y, capImage0.getRGB(x, y));
          } 
        }
      }
    }
    //切り貼り完成画像の描画自体は、drawPage()の中で行なう。
  }

  //■連続色相変更処理
  void changeColorMulti()
  {
    //●元画像（外部からの読み込み画像）の上書き描画
    //主に、マルチ・マスク・ファイルの色相修正に用いる画像加工。
    //String         ccmMaskF  //ChangeColorMalti用　元画像のファイル名
    //BufferedImage  ccmImage0 //元画像（連続色相変更処理前画像）
    //BufferedImage  ccmImage1 //新画像（連続色相変更処理完了画像）
    //List<int[]>    ccmListI0 //旧色指定リスト
    //List<int[]>    ccmListI1 //新色指定リスト

    if(ccmMaskF != null && !ccmMaskF.equals("")) //主にマルチ・マスク・ファイル
    {
      //画像ファイル指定のチェック
      String path0 = ccmMaskF.toLowerCase();
      if(path0.endsWith(".jpeg") ||path0.endsWith(".jpg") ||
        path0.endsWith(".gif") ||path0.endsWith(".png") ||
        path0.endsWith(".bmp"))
        ; //OK
      else
      {
        f.message("元画像ファイルが不適切："+path0);
        return;
      }
      if(! new File(path0).exists())
      {
        f.message("元画像ファイルが存在しません："+path0);
        return;
      }
      try
      {
        ccmImage0 = ImageIO.read(new File(path0)); //元画像の読み込み
      } 
      catch (Exception e) 
      {
        e.printStackTrace();
        f.message("マルチ・マスク・ファイルのリード・エラー："+path0);
        ccmImage0 = null;
        return;
      }
    }
    
    int ccmW = ccmImage0.getWidth();
    int ccmH = ccmImage0.getHeight();
    
    if(ccmImage0 == null)
      return;
    ccmImage1 = new BufferedImage( //連続色相変更後の画像
      ccmW, ccmH, BufferedImage.TYPE_INT_RGB);
    for(int y=0; y<imagePX_H; y++) //初期状態として画像コピー
    {
      for(int x=0; x<imagePX_W; x++)
      {
        ccmImage1.setRGB(x, y, ccmImage0.getRGB(x, y));
      }
    }
    
    int rgb0 = 0, rgb1 = 0;
    for(int i=0; i<ccmListI0.size(); i++) //色相変更命令の回数だけループ
    {
      for(int y=0; y<imagePX_H; y++)
      {
        for(int x=0; x<imagePX_W; x++)
        {
          rgb0 = (ccmListI0.get(i)[0] << 16 | ccmListI0.get(i)[1] << 8 | ccmListI0.get(i)[2]);
          rgb1 = (ccmListI1.get(i)[0] << 16 | ccmListI1.get(i)[1] << 8 | ccmListI1.get(i)[2]);
          if ((ccmImage0.getRGB(x,y) & 0x00ffffff) == rgb0)
          { 
            ccmImage1.setRGB(x, y, rgb1);
          } 
        }
      }
    }
    //連続色相変更完成画像の描画自体は、drawPage()の中で行なう。
  }

  //■図形内部彩色（単独処理）
  void changeColorInner()
  {
    //●輪郭線で囲まれた図形内部を指定色で彩色。

    //輪郭線で囲まれた図形の内外を自動的に判定する方法はある。
    //それは、輪郭線の色を手掛かりにするもので、図形を含む全体領域（Size命令で指定の領域）を端から順にスキャンし、
    //そのピクセルの色が輪郭線の色かどうかを判別し、輪郭線を超えて色が変わるたびにカウントし、カウントが奇数であれば
    //図形の外側と判断し、偶数であれば内側と判断して、内側判定したピクセルを指定色で彩色するアルゴリズムである。
    //ただし、針状に尖った部分を持つ図形では、輪郭線がピクセル単位で重なり、偶数が奇数になる部分も生じるため、
    //x軸とy軸とでの双方向検査が必要になる。また、全体領域内に複数の図形がある場合、その区別が難しくなる。
    //いずれにせよ、このプログラムでは、上に述べた内外自動判定のテクニックを使わない。

    if(changeColorInnerFlag==false)
      return;
    if(changeColorInnerFlag==true && !changeColorInnerF.equals(""))
    {
      String path = changeColorInnerF.toLowerCase();
      if(path.endsWith(".jpeg") ||path.endsWith(".jpg") ||
        path.endsWith(".gif") ||path.endsWith(".png") ||
        path.endsWith(".bmp"))
        ; //OK
      else
      {
        f.message("図形内部彩色用元画像ファイルが不適切："+path);
        return;
      }
      if(! new File(path).exists())
      {
        f.message("図形内部彩色用画像ファイルが存在しません："+path);
        return;
      }
      try
      {
         cciImage = ImageIO.read(new File(path));
      } 
      catch (Exception e) 
      {
        e.printStackTrace();
        f.message("図形内部彩色用画像ファイルのリード・エラー："+path);
        cciImage = null;
        return;
      }
    }
    else
    {
      f.message("図形内部彩色元画像ファイル指定に問題あり。");
      return;
    }
    
    int w = cciImage.getWidth(this);
    int h = cciImage.getHeight(this);
    int outLineColor = 0xff<<24 | cciOutLineColorR<<16 | cciOutLineColorG<<8 | cciOutLineColorB;
    int innerColor   = 0xff<<24 | cciInnerColorR<<16 | cciInnerColorG<<8 | cciInnerColorB;
    if(cciInnerX == 0  && cciInnerY == 0)
      ; //何もしない。（図形内で一部事前着色がされている場合）
    else
      cciImage.setRGB(cciInnerX, cciInnerY, innerColor); //図形内部代表位置の１ピクセルを彩色。
    int nextPositionColor = 0;
    for(;;) //スキャンの反復
    {
      boolean scanFlag = false;
      for(int j=0; j<h-1; j++) //右向き彩色拡張
      {
        for(int k=0; k<w-1; k++)
        {
          nextPositionColor = cciImage.getRGB(k+1,j);
          if(cciImage.getRGB(k,j)==innerColor && nextPositionColor!=outLineColor && nextPositionColor!=innerColor)
          {
            cciImage.setRGB(k+1,j,innerColor);
            scanFlag = true;
          }
        }
      }
      for(int j=0; j<h-1; j++) //左向き彩色拡張
      {
        for(int k=w-1; k>0; k--)
        {
          nextPositionColor = cciImage.getRGB(k-1,j);
          if(cciImage.getRGB(k,j)==innerColor && nextPositionColor!=outLineColor && nextPositionColor!=innerColor)
          {
            cciImage.setRGB(k-1,j,innerColor);
            scanFlag = true;
          }
        }
      }
      for(int j=0; j<w-1; j++) //下向き彩色拡張
      {
        for(int k=0; k<h-1; k++)
        {
          nextPositionColor = cciImage.getRGB(j,k+1);
          if(cciImage.getRGB(j,k)==innerColor && nextPositionColor!=outLineColor && nextPositionColor!=innerColor)
          {
            cciImage.setRGB(j,k+1,innerColor);
            scanFlag = true;
          }
        }
      }
      for(int j=0; j<w-1; j++) //上向き彩色拡張
      {
        for(int k=h-1; k>0; k--)
        {
          nextPositionColor = cciImage.getRGB(j,k-1);
          if(cciImage.getRGB(j,k)==innerColor && nextPositionColor!=outLineColor && nextPositionColor!=innerColor)
          {
            cciImage.setRGB(j,k-1,innerColor);
            scanFlag = true;
          }
        }
      }
      if(scanFlag == false) //新たに色彩変更することがなかった場合
        break;
    }

    if(cciMode==0)
    { //何もしないで、輪郭線の色はそのまま
      ;
    }
    else if(cciMode==1)
    { //輪郭線を内部彩色と同一色に変換
      int currentPositionColor = 0;
      for(int i=0; i<h; i++)
      {
        for(int j=0; j<w; j++)
        {
          currentPositionColor = cciImage.getRGB(j,i);
          if(cciImage.getRGB(j,i)==outLineColor)
          {
            cciImage.setRGB(j,i,innerColor);
          }
        }
      }
    }
    else if(cciMode==2)
    { //輪郭線を黒色(0,0,0)に変換
      int currentPositionColor = 0;
      for(int i=0; i<h; i++)
      {
        for(int j=0; j<w; j++)
        {
          currentPositionColor = cciImage.getRGB(j,i);
          if(cciImage.getRGB(j,i)==outLineColor)
          {
            cciImage.setRGB(j,i,0xff000000);
          }
        }
      }
    }
    else if(cciMode==3)
    { //輪郭線を灰色(200,200,200)に変換
      int currentPositionColor = 0;
      for(int i=0; i<h; i++)
      {
        for(int j=0; j<w; j++)
        {
          currentPositionColor = cciImage.getRGB(j,i);
          if(cciImage.getRGB(j,i)==outLineColor)
          {
            cciImage.setRGB(j,i,0xff<<24 | 200<<16 | 200<<8 | 200);
          }
        }
      }
    }
    else if(cciMode==4)
    { //輪郭線を白色(255,255,255)に変換
      int currentPositionColor = 0;
      for(int i=0; i<h; i++)
      {
        for(int j=0; j<w; j++)
        {
          currentPositionColor = cciImage.getRGB(j,i);
          if(cciImage.getRGB(j,i)==outLineColor)
          {
            cciImage.setRGB(j,i,0xff<<24 | 0xff<<16 | 0xff<<8 | 0xff);
          }
        }
      }
    }
  }

  //■図形内部連続彩色
  void changeColorInnerMulti()
  {
    //●複雑に交叉した輪郭線で構成された図形内部区画を指定色で連続彩色。
    
    //輪郭線で囲まれた図形がジグザク状や渦巻き状に屈曲している場合、彩色完了までに、
    //長時間（巨大で複雑で多色の図形なら数分）を要する。
    //これを高速化するには、点描画機能を用い、輪郭線図形の彩色部の何箇所かに予め指定色を部分的に塗っておく。
    //こうすれば、複雑な図形でも、大抵は、数十秒で彩色が完了する。単純な図形なら、一秒以内で完了する。
    //詳しくは、「メニュー」の「見本」を参照のこと。
    
    if(changeColorInnerMultiFlag==false)
      return;
    if(changeColorInnerMultiFlag==true && !changeColorInnerMultiF.equals(""))
    {
      String path = changeColorInnerMultiF.toLowerCase();
      if(path.endsWith(".jpeg") ||path.endsWith(".jpg") ||
        path.endsWith(".gif") ||path.endsWith(".png") ||
        path.endsWith(".bmp"))
        ; //OK
      else
      {
        f.message("図形内部連続彩色用元画像ファイルが不適切："+path);
        return;
      }
      if(! new File(path).exists())
      {
        f.message("図形内部連続彩色用画像ファイルが存在しません："+path);
        return;
      }
      try
      {
         ccimImage = ImageIO.read(new File(path));
      } 
      catch (Exception e) 
      {
        e.printStackTrace();
        f.message("図形内部連続彩色用画像ファイルのリード・エラー："+path);
        ccimImage = null;
        return;
      }
    }
    else
    {
      f.message("図形内部連続彩色元画像ファイル指定に問題あり。");
      return;
    }

    int w = ccimImage.getWidth(this);
    int h = ccimImage.getHeight(this);
    int outLineColor = 0xff<<24 | ccimOutLineColorR<<16 | ccimOutLineColorG<<8 | ccimOutLineColorB;

    //連続彩色
    for(int i=0; i<ccimListI.size(); i++) 
    {
      int rgb = (ccimListI.get(i)[0] << 16 | ccimListI.get(i)[1] << 8 | ccimListI.get(i)[2]);
      int innerColor = 0xff<<24 | ccimListI.get(i)[0]<<16 | ccimListI.get(i)[1]<<8 | ccimListI.get(i)[2];
      int ccimInnerX = ccimListI.get(i)[3];
      int ccimInnerY = ccimListI.get(i)[4];
      if(ccimInnerX == 0  && ccimInnerY == 0)
        ; //何もしない。（図形内で一部事前着色がされている場合）
      else
        ccimImage.setRGB(ccimInnerX, ccimInnerY, innerColor); //図形内部代表位置の１ピクセルを彩色。
      int nextPositionColor = 0;
      for(;;) //スキャンの反復
      {
        boolean scanFlag = false;
        for(int j=0; j<h-1; j++) //右向き彩色拡張
        {
          for(int k=0; k<w-1; k++)
          {
            nextPositionColor = ccimImage.getRGB(k+1,j);
            if(ccimImage.getRGB(k,j)==innerColor && nextPositionColor!=outLineColor && nextPositionColor!=innerColor)
            {
              ccimImage.setRGB(k+1,j,innerColor);
              scanFlag = true;
            }
          }
        }
        for(int j=0; j<h-1; j++) //左向き彩色拡張
        {
          for(int k=w-1; k>0; k--)
          {
            nextPositionColor = ccimImage.getRGB(k-1,j);
            if(ccimImage.getRGB(k,j)==innerColor && nextPositionColor!=outLineColor && nextPositionColor!=innerColor)
            {
              ccimImage.setRGB(k-1,j,innerColor);
              scanFlag = true;
            }
          }
        }
        for(int j=0; j<w-1; j++) //下向き彩色拡張
        {
          for(int k=0; k<h-1; k++)
          {
            nextPositionColor = ccimImage.getRGB(j,k+1);
            if(ccimImage.getRGB(j,k)==innerColor && nextPositionColor!=outLineColor && nextPositionColor!=innerColor)
            {
              ccimImage.setRGB(j,k+1,innerColor);
              scanFlag = true;
            }
          }
        }
        for(int j=0; j<w-1; j++) //上向き彩色拡張
        {
          for(int k=h-1; k>0; k--)
          {
            nextPositionColor = ccimImage.getRGB(j,k-1);
            if(ccimImage.getRGB(j,k)==innerColor && nextPositionColor!=outLineColor && nextPositionColor!=innerColor)
            {
              ccimImage.setRGB(j,k-1,innerColor);
              scanFlag = true;
            }
          }
        }
        if(scanFlag == false) //新たに色彩変更することがなかった場合
          break;
      }
    }
  }

  //■二重像画像処理
  void mixImage()
  {
    //●ミックス用画像（外部からの読み込み画像）を用いた二重像合成描画
    //●写真の二重露光と同様の画像合成
    //●画像ファイル指定のチェック
    if(!mixFName0.equals(""))
    {
      String path0 = mixFName0.toLowerCase();
      if(path0.endsWith(".jpeg") ||path0.endsWith(".jpg") ||
        path0.endsWith(".gif") ||path0.endsWith(".png") ||
        path0.endsWith(".bmp"))
        ; //OK
      else
      {
        f.message("ミックス用画像ファイル0が不適切："+path0);
        return;
      }
      if(! new File(path0).exists())
      {
        f.message("ミックス用画像ファイル0が存在しません："+path0);
        return;
      }
      try
      {
        mixImage0 = ImageIO.read(new File(path0));
      } 
      catch (Exception e) 
      {
        e.printStackTrace();
        f.message("ミックス用画像ファイル0のリード・エラー："+path0);
        mixImage0 = null;
        return;
      }

      String path1= mixFName1.toLowerCase();
      if(path1.endsWith(".jpeg") ||path1.endsWith(".jpg") ||
        path1.endsWith(".gif") ||path1.endsWith(".png") ||
        path1.endsWith(".bmp"))
      {
        ; //OK
      }
      else
      {
        f.message("ミックス用画像ファイル1が不適切："+path1);
        return;
      }
      if(! new File(path1).exists())
      {
        f.message("ミックス用画像ファイル1が存在しません："+path1);
        return;
      }
      try
      {
        mixImage1 = ImageIO.read(new File(path1));
      } 
      catch (Exception e) 
      {
        e.printStackTrace();
        f.message("ミックス用画像ファイル1のリード・エラー："+path1);
        mixImage1 = null;
        return;
      }

      //●画像ミックス
      //●画像サイズを全て統一しておくこと
      int w = mixImage0.getWidth(this);
      int h = mixImage0.getHeight(this);
      int R,G,B;
      mixImage2 = new BufferedImage(
        w, h, BufferedImage.TYPE_INT_RGB); //不透明

      if(mixMode == 1) //通常の二重像モード
      {
        for(int y=0;y<h;y++)
        {
          for(int x=0;x<w;x++)
          {
            int m = mixImage0.getRGB(x,y); //元画像
            int t = mixImage1.getRGB(x,y); //追加画像

            if( //追加画像の背景部分が灰色（200,200,200）の場合、
                //混色せずに、元画像の色相を維持
              (((t & 0x00ff0000)>>>16) == 200) &&
            	(((t & 0x0000ff00)>>> 8) == 200) &&
              (( t & 0x000000ff)      == 200) )
            {
            	R = (m & 0x00ff0000)>>>16;
            	G = (m & 0x0000ff00)>>> 8;
            	B = (m & 0x000000ff);
              int color = (R<<16) | (G<<8) | B;
              mixImage2.setRGB(x, y, color);
            }
            else //元画像と追加画像とを、有色半透明で混色合成
            {
            	R = ((m & 0x00ff0000)>>>16)/2 + ((t & 0x00ff0000)>>>16)/2;
            	G = ((m & 0x0000ff00)>>> 8)/2 + ((t & 0x0000ff00)>>> 8)/2;
            	B = (m & 0x000000ff)/2       + ( t & 0x000000ff)/2;
              int color = (R<<16) | (G<<8) | B;
              mixImage2.setRGB(x, y, color);
            }
          }
        }
      }
      else if(mixMode == 0) //拡大縮小を伴わない画像全体の「簡易Stamp命令」として機能
      {
        for(int y=0;y<h;y++)
        {
          for(int x=0;x<w;x++)
          {
            int m = mixImage0.getRGB(x,y); //元画像
            int t = mixImage1.getRGB(x,y); //追加画像

            if( //追加画像の背景部分が灰色（200,200,200）の場合、
                //元画像の色相を維持
              (((t & 0x00ff0000)>>>16) == 200) &&
            	(((t & 0x0000ff00)>>> 8) == 200) &&
              (( t & 0x000000ff)      == 200) )
            {
            	R = (m & 0x00ff0000)>>>16;
            	G = (m & 0x0000ff00)>>> 8;
            	B = (m & 0x000000ff);
              int color = (R<<16) | (G<<8) | B ;
              mixImage2.setRGB(x, y, color);
            }
            else //追加画像の色相をコピー（簡易Stamp押印）
            {
            	R = (t & 0x00ff0000)>>>16;
            	G = (t & 0x0000ff00)>>> 8;
            	B = (t & 0x000000ff);
              int color = (R<<16) | (G<<8) | B;
              mixImage2.setRGB(x, y, color);
            }
          }
        }
      }
    }
    //二重像画像の描画自体は、drawPage()の中で行なう。
  }

  //■鏡像への画像反転処理（回転処理も一部含む）
  void reverseImage()
  {
    //●画像の鏡像反転（回転処理も一部含む）
    if(reverseImageFlag == true)
    {
      try
      { //反転対象既存画像の読み込み
        reverseImage0 = ImageIO.read(new File(reverseImageFile));
      } 
      catch (Exception e) 
      {
        e.printStackTrace();
        f.message("反転対象既存画像のリード・エラー");
        return;
      }

      int ww = reverseImage0.getWidth(this);
      int hh = reverseImage0.getHeight(this);
      
      reverseImage1 = new BufferedImage(
        ww, hh, BufferedImage.TYPE_INT_RGB); //不透明に設定
      reverseImageX = new BufferedImage( //回転
        hh, ww, BufferedImage.TYPE_INT_RGB); //不透明に設定

      if(reverseImageMode==1) //左右鏡像反転
      {
        for(int y=0; y<hh; y++)
        {
          for(int x=0, w1=ww; x<ww; x++, w1--)
          {
            reverseImage1.setRGB(w1-1, y, reverseImage0.getRGB(x,y));
          }
        }
      }
      else if(reverseImageMode==2) //上下鏡像反転
      {
        for(int y=0, h1=hh; y<hh; y++, h1--)
        {
          for(int x=0; x<ww; x++)
          {
            reverseImage1.setRGB(x, h1-1, reverseImage0.getRGB(x,y));
          }
        }
      }
      else if(reverseImageMode==3) //左半分を右半分に対称型に転写して左右対称にする。
      {
        int half;
        if(ww%2 == 0)
          half=ww/2;
        else
          half=ww/2+1;
        for(int y=0; y<hh; y++)
        {
          for(int x=0, w1=half; x<half; x++, w1--)
          {
            reverseImage1.setRGB(x, y, reverseImage0.getRGB(x,y));
            reverseImage1.setRGB(half+w1-1, y, reverseImage0.getRGB(x,y));
          }
        }
      }
      else if(reverseImageMode==4) //上半分を下半分に対称型に転写して上下対称にする。
      {
        int half;
        if(hh%2 == 0)
          half=hh/2;
        else
          half=hh/2+1;
        for(int y=0, h1=half; y<half; y++, h1--)
        {
          for(int x=0; x<ww; x++)
          {
            reverseImage1.setRGB(x, y, reverseImage0.getRGB(x,y));
            reverseImage1.setRGB(x, half+h1-1, reverseImage0.getRGB(x,y));
          }
        }
      }
      else if(reverseImageMode==6) //時計回りで、180度回転（半回転）させる。
      {
        for(int y=0, y1=hh-1; y<hh; y++, y1--)
        {
          for(int x=0, x1=ww-1; x<ww; x++, x1--)
          {
            reverseImage1.setRGB(x1,y1, reverseImage0.getRGB(x,y));
          }
        }
      }

      //これ以降のreverseImageMode（5 or 7）は、画像の幅と丈とが入れ替わるので、
      //通常の方法（メニューやボタンの操作）では、Size命令が邪魔をして、
      //正常に画像保存できないため、このメソッドの内部で、画像保存を強制的に行なう。
      
      else if(reverseImageMode==5) //時計回りで、90度回転させる。
      {
        for(int y=0, y1=hh-1; y<hh; y++, y1--)
        {
          for(int x=0; x<ww; x++)
          {
            reverseImageX.setRGB(y1,x, reverseImage0.getRGB(x,y)); //*********
          }
        }
        reverseImage1=reverseImageX;
        try
        {
          ImageIO.write(reverseImage1, "png", new File("./image/reverseImageTemp.png"));
        }
        catch (Exception e) 
        {
          e.printStackTrace();
          return;
        }
        f.message("./image/reverseImageTemp.pngに画像保存しました。");
      }
      else if(reverseImageMode==7) //時計回りで、270度回転させる。
      {
        for(int y=0; y<hh; y++)
        {
          for(int x=0, x1=ww-1; x<ww; x++,x1--)
          {
            reverseImageX.setRGB(y,x1, reverseImage0.getRGB(x,y));
          }
        }
        reverseImage1=reverseImageX;
        try
        {
          ImageIO.write(reverseImage1, "png", new File("./image/reverseImageTemp.png"));
        }
        catch (Exception e) 
        {
          e.printStackTrace();
          return;
        }
        f.message("./image/reverseImageTemp.pngに画像保存しました。");
      }
      else 
      {
        reverseImage1 = reverseImage0;
      }
    }
  }

  //■単独色相変更処理
  void colorChange(int mode)
  {
    //●単独色相変更機能の実現。
    //単独色相変更要求があれば、指定の旧色に対して指定の新色に変換する。
    //処理対象の画像が、既に画像ファイルとして存在するものとする。
    //階調画像からグレー・マスクを生成する機能を追加的に含む。
    if(mode==0)
    { 
      return; //何もしない（単独色相変更要求がない）
    }
    if(colorChangeFlag == true)
    {
      try
      {
        colorChangeImage0 = ImageIO.read(new File(colorChangeFile));
      } 
      catch (Exception e) 
      {
        e.printStackTrace();
        f.message("カラー・チェンジ既存画像のリード・エラー");
        colorChangeImage0 = null;
        return;
      }

      Color oldColor  = new Color(colorChange1R, colorChange1G, colorChange1B);
      Color newColor  = new Color(colorChange2R, colorChange2G, colorChange2B);
      int grayColor = (200<<16 | 200<<8 | 200);
      //単独色相変更実施
      int w_ = colorChangeImage0.getWidth(this);
      int h_ = colorChangeImage0.getHeight(this);
      colorChangeImage1 = new BufferedImage(
        w_, h_, BufferedImage.TYPE_INT_RGB); //不透明に設定
      for(int y=0;y<h_;y++)
      {
        for(int x=0;x<w_;x++)
        {
          int rgbX = colorChangeImage0.getRGB(x,y);
          int redX   = (rgbX & 0x00ff0000) >>> 16;
          int greenX = (rgbX & 0x0000ff00) >>> 8;
          int blueX  = (rgbX & 0x000000ff);
          if (  //近似色をも許容する
            (redX >= (oldColor.getRed() - acceptableLimitC) &&
             redX <= (oldColor.getRed() + acceptableLimitC))
              &&
            (greenX >= (oldColor.getGreen() - acceptableLimitC) &&
             greenX <= (oldColor.getGreen() + acceptableLimitC))
              &&
            (blueX >= (oldColor.getBlue() - acceptableLimitC) &&
             blueX <= (oldColor.getBlue() + acceptableLimitC))    )
          { //各種条件一致なら３２ビットの新色に変換
            colorChangeImage1.setRGB(x, y, newColor.getRGB());
          } 
          else
          {
            if(mode==1)
            { //旧色のまま転写
              colorChangeImage1.setRGB(x, y, colorChangeImage0.getRGB(x,y));
            }
            else if(mode==2)
            { //グレー色（200,200,200）を転写
              //階調画像からグレー・マスクを生成
              colorChangeImage1.setRGB(x, y, grayColor);
            }
            else if(mode>2)
            {
              f.message("colorChange命令のモード指定が異常。");
              return;
            }
          }
        }
      }
    }
    //単独色相変更の描画自体は、drawPage()の中で行なう。
  }

  //■色相調整効果
  void adjustRGB()
  {
    //●画像ファイル指定のチェック
    if(!adjustFName.equals("") && adjustFName != null)
    {
      String path0 = adjustFName.toLowerCase();
      if(path0.endsWith(".jpeg") || path0.endsWith(".jpg") ||
        path0.endsWith(".gif") || path0.endsWith(".png") ||
        path0.endsWith(".bmp"))
        ; //OK
      else
      {
        f.message("色相調整対象ファイルが不適切："+path0);
        return;
      }
      if(! new File(path0).exists())
      {
        f.message("色相調整対象ファイルが存在しません："+path0);
        return;
      }
      adjustImage0 = new BufferedImage(
        (int)imagePX_W, (int)imagePX_H, BufferedImage.TYPE_INT_RGB); //不透明に設定
      adjustImage1 = new BufferedImage(
        (int)imagePX_W, (int)imagePX_H, BufferedImage.TYPE_INT_RGB); //不透明に設定
      try
      {
        adjustImage0 = ImageIO.read(new File(path0)); //画像読み取り
      } 
      catch (Exception e) 
      {
        e.printStackTrace();
        f.message("色相調整対象ファイルのリード・エラー："+path0);
        adjustImage0 = null;
        return;
      }
      //色調調整
      int w = adjustImage0.getWidth(this);
      int h = adjustImage0.getHeight(this);
      int R,G,B; //元の色相
      int R1,G1,B1; //調整後の色相
      for(int y=0; y<h; y++)
      {
        for(int x=0; x<w; x++)
        {
            int m = adjustImage0.getRGB(x,y);
            R = (m & 0x00ff0000) >>> 16;
            G = (m & 0x0000ff00) >>> 8;
            B = (m & 0x000000ff) >>> 0;
            if(adjustR >0)
              R1 = R + (int)((255-R)*(adjustR/100.0));
            else
              R1 = R - (int)((0-R)*(adjustR/100.0));
            if(adjustG >0)
              G1 = G + (int)((255-G)*(adjustG/100.0));
            else
              G1 = G - (int)((0-G)*(adjustG/100.0));
            if(adjustB >0)
              B1 = B + (int)((255-B)*(adjustB/100.0));
            else
              B1 = B - (int)((0-B)*(adjustB/100.0));
            int color =  (R1<<16) | (G1<<8) | B1 ;
            adjustImage1.setRGB(x, y, color);
        }
      }
    }
    //色相調整の描画自体は、drawPage()の中で行なう。
  }

  //■色相濃淡階調効果（垂直線形グラデーション）
  void colorGradient()
  {
    //●画像ファイル指定のチェック
    if(!gradientFName.equals("") && gradientFName != null)
    {
      String path0 = gradientFName.toLowerCase();
      if(path0.endsWith(".jpeg") || path0.endsWith(".jpg") ||
        path0.endsWith(".gif") || path0.endsWith(".png") ||
        path0.endsWith(".bmp"))
        ; //OK
      else
      {
        f.message("色相濃淡階調加工対象ファイルが不適切："+path0);
        return;
      }
      if(! new File(path0).exists())
      {
        f.message("色相濃淡階調加工対象ファイルが存在しません："+path0);
        return;
      }
      gradientImage0 = new BufferedImage(
        (int)imagePX_W, (int)imagePX_H, BufferedImage.TYPE_INT_RGB); //不透明に設定
      gradientImage1 = new BufferedImage(
        (int)imagePX_W, (int)imagePX_H, BufferedImage.TYPE_INT_RGB); //不透明に設定
      try
      {
        gradientImage0 = ImageIO.read(new File(path0)); //画像読み取り
      } 
      catch (Exception e) 
      {
        e.printStackTrace();
        f.message("色相濃淡階調加工対象ファイルのリード・エラー："+path0);
        gradientImage0 = null;
        return;
      }
      //色調濃淡階調加工（グラデーション）
      int w = gradientImage0.getWidth(this);
      int h = gradientImage0.getHeight(this);
      int cR,cG,cB; //元の色相
      int cR1,cG1,cB1; //濃淡階調加工後の色相
      for(int y=0; y<h; y++)
      { //全体を単純にコピー
        for(int x=0; x<w; x++)
        { 
            int m = gradientImage0.getRGB(x,y);
            gradientImage1.setRGB(x, y, m);
        }
      }
      for(int y=gradientY1; y<gradientY2; y++)
      { //非対称グラデーションの上書き
        for(int x=0; x<w; x++)
        {
            int m = gradientImage0.getRGB(x,y);
            cR = (m & 0x00ff0000) >>> 16;
            cG = (m & 0x0000ff00) >>> 8;
            cB = (m & 0x000000ff) >>> 0;

            cR1 = cR + (int)(gradientR*(y-gradientY1));
            if(cR1>255)
              cR1=255;
            else if(cR1<0)
              cR1=0;

            cG1 = cG + (int)(gradientG*(y-gradientY1));
            if(cG1>255)
              cG1=255;
            else if(cG1<0)
              cG1=0;

            cB1 = cB + (int)(gradientB*(y-gradientY1));
            if(cB1>255)
              cB1=255;
            else if(cB1<0)
              cB1=0;

            int color = (cR1<<16) | (cG1<<8) | cB1 ;
            gradientImage1.setRGB(x, y, color);
        }
      }
      //対称グラデーションの追加上書き
      int gY =gradientY2-gradientY1;
      for(int y=gradientY2; y<(gradientY1+(gradientY2-gradientY1)*2) && (y<h); y++)
      { 
        gY--;
        for(int x=0; x<w; x++)
        {
            int m = gradientImage0.getRGB(x,y);
            cR = (m & 0x00ff0000) >>> 16;
            cG = (m & 0x0000ff00) >>> 8;
            cB = (m & 0x000000ff) >>> 0;

            cR1 = cR + (int)(gradientR*(gY));
            if(cR1>255)
              cR1=255;
            else if(cR1<0)
              cR1=0;

            cG1 = cG + (int)(gradientG*(gY));
            if(cG1>255)
              cG1=255;
            else if(cG1<0)
              cG1=0;

            cB1 = cB + (int)(gradientB*(gY));
            if(cB1>255)
              cB1=255;
            else if(cB1<0)
              cB1=0;

            int color = (cR1<<16) | (cG1<<8) | cB1 ;
            gradientImage1.setRGB(x, y, color);
        }
      }
    }
    //色相濃淡階調加工の描画自体は、drawPage()の中で行なう。
  }

  //■画像多段階回転重複描画
  void rotateMulti()
  {
    //●Size命令で設定された座標範囲の内側にある回転対象領域のみを多段階に回転させ上書きする。
    //画像多段階回転は、中心点を同じくする二重の正方形の枠組みの中で、回転させつつ多段階に描画する。
    //例えば、一枚の花弁を事前に描き、これを３０度づつ回転させ、花弁が１２枚の花を自動描画する用途に使用。
    //外枠の正方形（Size命令で指定）の中に内枠の正方形が入る形だが、一辺の比率は、３対２の比率にする。
    //内枠の中に、回転させる元画像が、はみ出さずに収まらねばならない。
    //外枠と内枠との中間領域は、回転する余裕を与える空間である。
    //回転対象画像は、グレー・マスク形式（背景がグレー）とし、完成画像も、グレー・マスク形式になる。

    if(rotateMultiFlag == false)
      return;
    BufferedImage rotateMultiImage1 = new BufferedImage(
      (int)imagePX_W, (int)imagePX_H, BufferedImage.TYPE_INT_RGB); //不透明に設定
    BufferedImage rotateMultiImage2 = new BufferedImage(
      (int)imagePX_W, (int)imagePX_H, BufferedImage.TYPE_INT_RGB); //不透明に設定

    try
    {
      rotateMultiImage1 = ImageIO.read(new File(rotateMultiF)); //画像読み取り
    } 
    catch (Exception e) 
    {
      e.printStackTrace();
      f.message("回転対象ファイルのリード・エラー："+rotateMultiF);
      rotateMultiImage = null;
      return;
    }
    
    int gray = (0xff<<24) | (200<<16) | (200<<8) | 200;
    for(int i=0; i<imagePX_W; i++) //グレーで初期化
    {
      for(int j=0; j<imagePX_H; j++)
      {
        rotateMultiImage2.setRGB(i,j,gray);
      }
    }

    int color1 = 0;
    int color2 = 0;
    int x2 = 0;
    int y2 = 0;
    
    int x1min = (int)(imagePX_W/6);
    int x1max = (int)((imagePX_W/6)*5);
    int y1min = (int)(imagePX_H/6);
    int y1max = (int)((imagePX_H/6)*5);
    int centerX = (int)(imagePX_W/2); //画像中心点
    int centerY = (int)(imagePX_H/2);
    double radian = 0;
    double startRadian = Math.toRadians(rotateMultiStartDegree);
    for( int k=0; k<rotateMultiTimes; k++) //回転回数分のループ
    {
      radian = startRadian + Math.toRadians(rotateMultiDegree*k);
      for(int j=x1min; j<x1max; j++) //回転させる位置変換
      {
        for(int i=y1min; i<y1max; i++)
        {
          x2 = (int)(((i-centerX)*Math.cos(radian) - (j-centerY)*Math.sin(radian))+centerX);
          y2 = (int)(((i-centerX)*Math.sin(radian) + (j-centerY)*Math.cos(radian))+centerY);
          color1 = rotateMultiImage1.getRGB(i,j);
          if(color1 != gray)
          {
            try
            { 
                rotateMultiImage2.setRGB(x2,y2,color1);
            }
            catch (Exception e) 
            {
              e.printStackTrace();
              return;
            }
          }
        }
      }
    }

    //●画像補正（多段階回転させた画像のピクセル欠けを埋める処理）
    //これから行なう画像補正は、三角関数による座標変換計算で生じた
    //実数座標を整数座標に変換する際に生じる画像のピクセル欠けを補正するものである。
    //ただし、多段階回転に適するのは、単色無階調画像（ベタ塗り画像）に限られ、
    //多色無階調画像（マトリックス模様など）や階調画像（ボカシ画像）は、一段階回転しか適さない。
    //多色画像を無理に多段階回転多重描画すると、モアレ模様が現われ、画像の精細さが失われる。
    //そもそも、多色が入り乱れる複雑な画像を、回転させながら多重描画する用途は、殆どない。
    //一回分の回転のたびに画像補正して多重上書きすれば、階調画像でもモアレ回避が可能のはずだが、
    //そこまでする必要がないので、そうしない。
    //なお、回転加工した画像は、三角関数計算値の整数化および画像補正により、輪郭線が多少乱れるので、
    //予め巨大画像で作成しておいて、スタンプ機能で利用する際に、画像縮小で目立たないようにする。
    //なお、回転させる画像は、「細い線」での線画を想定しておらず、「細い線」を無理に回転させると、
    //「細い線」が途切れて、補正も上手く行かない可能性が高い。

    for(int p=1; p<imagePX_H-1; p++)
    {
      for(int q=1; q<imagePX_W-1; q++)
      {
        int col0 = rotateMultiImage2.getRGB(q,p);
        int col1 = rotateMultiImage2.getRGB(q-1,p);
        int col2 = rotateMultiImage2.getRGB(q+1,p);
        int col3 = rotateMultiImage2.getRGB(q,p-1);
        int col4 = rotateMultiImage2.getRGB(q,p+1);
        
        int col1R = (col1 & 0x00ff0000)>>>16;
        int col1G = (col1 & 0x0000ff00)>>>8;
        int col1B = (col1 & 0x000000ff);
        int col2R = (col1 & 0x00ff0000)>>>16;
        int col2G = (col1 & 0x0000ff00)>>>8;
        int col2B = (col1 & 0x000000ff);
        int col3R = (col1 & 0x00ff0000)>>>16;
        int col3G = (col1 & 0x0000ff00)>>>8;
        int col3B = (col1 & 0x000000ff);
        int col4R = (col1 & 0x00ff0000)>>>16;
        int col4G = (col1 & 0x0000ff00)>>>8;
        int col4B = (col1 & 0x000000ff);
        
        if(col0==gray)
        {
          if(col1!=gray && col1==col2) //単色無階調画像の場合
            rotateMultiImage2.setRGB(q,p,col1);
          else if(col1!=gray && col3==col4) //単色無階調画像の場合
            rotateMultiImage2.setRGB(q,p,col3);
          else if(col1!=gray && col2!=gray && col3!=gray && col4!=gray)
            //多色無階調画像や階調画像の場合（一段階回転に限定すべき）
          {
            int colXR = (int)((col1R + col2R + col3R + col4R)/4);
            int colXG = (int)((col1G + col2G + col3G + col4G)/4);
            int colXB = (int)((col1B + col2B + col3B + col4B)/4);
            int colX  = 0xff<<24 | colXR<<16 | colXG<<8 | colXB;
            rotateMultiImage2.setRGB(q,p,col1);
          }
        }
      }
    }
    rotateMultiImage = rotateMultiImage2;
    //多段階回転加工処理した画像描画自体は、drawPage()の中で行なう。
  }


  //■フラット型スプレー
  //フラット型（円筒型分布）飛沫分散方式
  double[][] spray0(int sr, int st)
  { //srは飛沫群円盤の半径（最大斜辺）。
    //stは飛沫の個数。

    //●フラット型スプレー機能（円盤状飛沫集団形成）は、三平方の定理と乱数とで実現する。
    //一回のスプレー機能の呼び出しで、飛沫群円盤が一個できる。
    //直角三角形⊿で、底辺をa、高さをb、斜辺をc、左下角度（ディグリー）をtとする。
    //ただし、左下角を円盤中心、右上角を飛沫位置と見なす。
    //底辺と高さとが乱数で決まれば、斜辺が算出できる。

    //フラット型飛沫分散を実現する方法は、以下の通り。
    //飛沫円盤に外接する正方形は、一辺が sr*2 である。
    //この正方形内にフラット分散する飛沫位置は、
    //x軸もy軸も、sr*2 を最大値とする乱数で計算できる。
    //正方形の中心が円盤の中心であるから、その飛沫座標位置は、中心からの距離に調整する。
    //さらに、飛沫円盤から飛び出す飛沫位置は、除外することになる。

    
    double[][] sprayXY = new double[st][2]; //stは飛沫の個数
    double aX; //⊿底辺a
    double bX; //⊿高さb
      

    for(int i=0; i<st; )
    {
      aX = ranSpray.nextDouble(sr*2) - sr;
      bX = ranSpray.nextDouble(sr*2) - sr;
      
      if(Math.sqrt(aX*aX + bX*bX) < sr) //⊿斜辺の計算と長さ判定
      {
        sprayXY[i][0] = aX;
        sprayXY[i][1] = bX;
        i++;
      }
    }
    return sprayXY;
  }

  //■正規分布型スプレー
  //正規分布型（ベル型分布、ガウス型分布）飛沫分散方式
  double[][] spray1(int sr, int st)
  { //srは飛沫群円盤の半径（最大斜辺）。
    //stは飛沫の個数。

    //●スプレー機能（円盤状飛沫集団形成）は、正規分布型乱数と乱数とで実現する。
    //一回のスプレー機能の呼び出しで、飛沫群円盤が一個できる。
    //直角三角形⊿で、底辺をa、高さをb、斜辺をc、左下角度（ディグリー）をtとする。
    //ただし、左下角を円盤中心、右上角を飛沫位置と見なす。
    //底辺と高さとが乱数で決まれば、斜辺が算出できる。

    //フラット型飛沫分散を実現する方法は、以下の通り。
    //飛沫円盤に外接する正方形は、一辺が sr*2 である。
    //この正方形内にフラット分散する飛沫位置は、
    //x軸もy軸も、sr*2 を最大値とする乱数で計算できる。
    //乱数は、ガウス分布（正規分布）を指定する。
    //正方形の中心が円盤の中心であるから、その飛沫座標位置は、中心からの距離に調整する。
    //さらに、飛沫円盤から飛び出す飛沫位置は、除外することになる。

    double[][] sprayXY = new double[st][2]; //stは飛沫の個数
    double aX; //⊿底辺a
    double bX; //⊿高さb

    for(int i=0; i<st; )
    {
      aX = (ranSpray.nextGaussian())*(sr)*sprayGaussianOp;
      bX = (ranSpray.nextGaussian())*(sr)*sprayGaussianOp;
      if(Math.sqrt(aX*aX + bX*bX) < sr) //⊿斜辺の計算と円盤範囲制限
      //実際の正規分布では、円盤の外にも飛沫が飛ぶが、これを円盤内に制限する。
      //そのため、円盤の外部が除去された正規分布である。
      {
        sprayXY[i][0] = aX;
        sprayXY[i][1] = bX;
        i++;
      }
    }
    return sprayXY;
  }

  //■円錐型スプレー
  //円錐型飛沫分散方式
  double[][] spray2(int sr, int st)
  { //srは飛沫群円盤の半径（最大斜辺）。
    //stは飛沫の個数。

    //●スプレー機能（円盤状飛沫集団形成）は、三角関数と乱数とで実現する。
    //一回のスプレー機能の呼び出しで、飛沫群円盤が一個できる。
    //直角三角形⊿で、底辺をa、高さをb、斜辺をc、左下角度（ディグリー）をtとする。
    //円盤中心点から飛沫一個への距離cは、0から円盤半径までの整数乱数で決定し、
    //円盤中心点から飛沫一個への角度tは、0から360までの整数乱数で決定する。
    //関係する計算式は、以下の通り。
    //sin(t) = b/c　これを変形すれば、b = c*sin(t) または c = b/sin(t)
    //cos(t) = a/c　これを変形すれば、a = c*cos(t) または c = a/cos(t)
    //tan(t) = b/a　これを変形すれば、b = a*tan(t) または a = b/tan(t)
    //ただし、左下角を円盤中心、右上角を飛沫位置と見なす。

    //●このspray1メソッドは、円盤領域の中央に飛沫が集中する仕様であって、
    //円錐型の飛沫分布になるので、やや中央に集中し過ぎの感もある。
    //この仕様にした意図は、飛沫個数を少なくして、飛沫円盤をランダム・ウォーク方式で
    //多数描画して、全体として朦朧とした画像を形成する使用法を想定している。
    //また、中央に集中した飛沫集団で、朦朧としつつも芯のある画像の形成を想定する。

    Random ranS = new Random(new GregorianCalendar().get(Calendar.MILLISECOND));
    double aX; //⊿底辺a
    double bX; //⊿高さb
    double cX; //⊿斜辺c
    double tX; //⊿左下角度t
    double[][] sprayXY = new double[st][2]; //stは飛沫の個数
    for(int i=0; i<st; i++)
    {
      cX = ranSpray.nextDouble(sr); //飛沫一個への距離（斜辺）
      tX = ranSpray.nextDouble(360); //飛沫一個への角度（ディグリー）
      //斜辺と角度とが乱数で決まれば、底辺aと高さbとが三角関数で算出できる。
      aX = cX*Math.cos(Math.toRadians(tX)); //飛沫一個への底辺（x軸）
      bX = aX*Math.tan(Math.toRadians(tX)); //飛沫一個への高さ（y軸）
      sprayXY[i][0] = aX;
      sprayXY[i][1] = bX;
    }
    return sprayXY;
  }
} //EOF (GCreatorCanvasEngine.java)
